@@ -28,6 +28,8 @@
 #define USE_GL
 #define USE_UNSTABLE
 #define USE_BOMB
+#define HAVE_TTF
+#define HAVE_GLTT
 
 typedef struct {
 	int	 dummy;
@@ -106,11 +108,6 @@ LockStruct  LockProcs[] =
 	{"cage",
 	 1000, 1, 1, 1, 64, 1.0, "",
 	 "Shows the Impossible Cage, an Escher-like GL scene", 0, NULL, "#ifdef USE_GL"},
-#endif
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-	{"cartoon",
-	 10000, 1, 1, 1, 64, 1.0, "",
-	 "Shows bouncing cartoons", 0, NULL, "#if defined( USE_XPM ) || defined( USE_XPMINC )"},
 #endif
 	{"clock",
 	 100000, -16, 200, -200, 64, 1.0, "",
@@ -154,6 +151,11 @@ LockStruct  LockProcs[] =
 	{"fadeplot",
 	 30000, 10, 1500, 1, 64, 0.6, "",
 	 "Shows a fading plot of sine squared", 0, NULL, NULL},
+#ifdef USE_GL
+	{"fire",
+	 10000, 800, 1, 1, 64, 1.0, "",
+	 "Shows a 3D fire-like image", 0, NULL, "#ifdef USE_GL"},
+#endif
 	{"flag",
 	 50000, 1, 1000, -7, 64, 1.0, "",
 	 "Shows a waving flag image", 0, NULL, NULL},
@@ -215,10 +217,10 @@ LockStruct  LockProcs[] =
 	{"kumppa",
 	 10000, 1, 1, 1, 64, 1.0, "",
 	 "Shows kumppa", 0, NULL, NULL},
-#if defined( USE_GL ) && ( defined( USE_XPM ) || defined( USE_XPMINC ))
+#ifdef USE_GL
 	{"lament",
 	 10000, 1, 1, 1, 64, 1.0, "",
-	 "Shows Lemarchand's Box", 0, NULL, "#if defined( USE_GL ) && ( defined( USE_XPM ) || defined( USE_XPMINC ))"},
+	 "Shows Lemarchand's Box", 0, NULL, "#ifdef USE_GL"},
 #endif
 	{"laser",
 	 20000, -10, 200, 1, 64, 1.0, "",
@@ -282,6 +284,9 @@ LockStruct  LockProcs[] =
 	{"penrose",
 	 10000, 1, 1, -40, 64, 1.0, "",
 	 "Shows Penrose's quasiperiodic tilings", 0, NULL, NULL},
+	{"pentominoes",
+	 1000, 1, 1, 1, 64, 1.0, "",
+	 "Shows attempts to place pentominoes into a rectangle", 0, NULL, NULL},
 	{"petal",
 	 10000, -500, 400, 1, 64, 1.0, "",
 	 "Shows various GCD Flowers", 0, NULL, NULL},
@@ -303,12 +308,17 @@ LockStruct  LockProcs[] =
 	 100000, 25, 1, -64, 64, 0.6, "",
 	 "Shows a rolling ball", 0, NULL, NULL},
 	{"rotor",
-	 500, 4, 100, -6, 64, 0.3, "",
+	 100, 4, 100, -6, 64, 0.3, "",
 	 "Shows Tom's Roto-Rooter", 0, NULL, NULL},
 #ifdef USE_GL
 	{"rubik",
 	 10000, -30, 5, -6, 64, 1.0, "",
 	 "Shows an auto-solving Rubik's Cube", 0, NULL, "#ifdef USE_GL"},
+#endif
+#ifdef USE_GL
+	{"sballs",
+	 10000, 0, 10, 400, 64, 1.0, "",
+	 "Balls spinning like crazy in GL", 0, NULL, "#ifdef USE_GL"},
 #endif
 	{"shape",
 	 10000, 100, 256, 1, 64, 1.0, "",
@@ -375,7 +385,7 @@ LockStruct  LockProcs[] =
 	 250000, 1000, 60000, 0, 64, 1.0, "",
 	 "Shows a Flying Balls Clock Demo", 0, NULL, NULL},
 	{"tetris",
-	 50000, 1, 1, 0, 64, 1.0, "",
+	 50000, 1, 1, -100, 64, 1.0, "",
 	 "Shows an autoplaying tetris game", 0, NULL, NULL},
 #if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_TTF ) && defined( HAVE_GLTT ) && defined( USE_UNSTABLE )
 	{"text3d",
