@@ -659,7 +659,6 @@ bubble\
 bubble3d\
 bug\
 cage\
-cartoon\
 clock\
 coral\
 crystal\
@@ -674,6 +673,7 @@ drift\
 euler2d\
 eyes\
 fadeplot\
+fire\
 flag\
 flame\
 flow\
@@ -714,6 +714,7 @@ munch\
 nose\
 pacman\
 penrose\
+pentominoes\
 petal\
 pipes\
 puzzle\
@@ -722,6 +723,7 @@ qix\
 roll\
 rotor\
 rubik\
+sballs\
 shape\
 sierpinski\
 skewb\
@@ -741,6 +743,7 @@ swarm\
 swirl\
 t3d\
 tetris\
+text3d\
 thornbird\
 tik_tak\
 triangle\
