@@ -84,7 +84,6 @@ lst.addItem("bubble");
 lst.addItem("bubble3d");
 lst.addItem("bug");
 lst.addItem("cage");
-lst.addItem("cartoon");
 lst.addItem("clock");
 lst.addItem("coral");
 lst.addItem("crystal");
@@ -99,6 +98,7 @@ lst.addItem("drift");
 lst.addItem("euler2d");
 lst.addItem("eyes");
 lst.addItem("fadeplot");
+lst.addItem("fire");
 lst.addItem("flag");
 lst.addItem("flame");
 lst.addItem("flow");
@@ -139,6 +139,7 @@ lst.addItem("munch");
 lst.addItem("nose");
 lst.addItem("pacman");
 lst.addItem("penrose");
+lst.addItem("pentominoes");
 lst.addItem("petal");
 lst.addItem("pipes");
 lst.addItem("puzzle");
@@ -147,6 +148,7 @@ lst.addItem("qix");
 lst.addItem("roll");
 lst.addItem("rotor");
 lst.addItem("rubik");
+lst.addItem("sballs");
 lst.addItem("shape");
 lst.addItem("sierpinski");
 lst.addItem("skewb");
@@ -166,6 +168,7 @@ lst.addItem("swarm");
 lst.addItem("swirl");
 lst.addItem("t3d");
 lst.addItem("tetris");
+lst.addItem("text3d");
 lst.addItem("thornbird");
 lst.addItem("tik_tak");
 lst.addItem("triangle");
