@@ -63,7 +63,7 @@ XCOMM   *** END C++ CONFIG SECTION ***
 
 XCOMM   *** BEGIN XPM CONFIG SECTION ***
 
-XCOMM Only the bat.c, cartoon.c, flag.c, image.c, maze.c, and puzzle.c
+XCOMM Only the bat.c, flag.c, image.c, maze.c, and puzzle.c
 XCOMM modes use this.
 XCOMM If your system has libXpm, remove the 'XCOMM  ' from the next line.
 XCOMM  #define XpmLibrary
@@ -351,8 +351,12 @@ PASSWDDEF = -DHAVE_SHADOW
 XCOMM Problems finding libXext.so.0 when sticky bit is set
 EXTRA_LDOPTIONS = -R/usr/lib:/usr/openwin/lib:/usr/dt/lib:/usr/local/lib
 
+#if OSMajorVersion == 2 && OSMinorVersion < 7
 PIXMAPTYPE = sol
 #else
+PIXMAPTYPE = solaris
+#endif
+#else
 SYSTEMDEF = -DSUNOS4 -DUSE_MATHERR
 SLEEPDEF = -DHAVE_USLEEP
 #if HasShadowPasswd
@@ -428,12 +432,14 @@ XCOMM  PASSWDLIB += /usr/afsws/domestic/lib/librxkad.a usr/lib/libsocket.so.1 /u
 XCOMM You may need this one too.
 XCOMM  EXTRA_LIBRARIES = -laudit
 
-
 #else
 #ifdef UltrixArchitecture
 BITMAPTYPE = dec
 PIXMAPTYPE = dec
 EXTRA_LIBRARIES = -lauth
+XCOMM Use this if your site is using SIA:
+XCOMM  PASSWDDEF = -DSIA
+XCOMM  PASSWDLIB = -lsecurity 
 #else
 
 #ifdef SCOArchitecture
