@@ -2,7 +2,7 @@
 /* bat --- bouncing bats */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)bat.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)bat.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,11 +22,12 @@ static const char sccsid[] = "@(#)bat.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Compatible with xscreensaver
- * 18-Sep-95: 5 bats now in color <patol@info.isbiel.ch>
- * 20-Sep-94: 5 bats instead of bouncing balls, based on bounce.c
- *            <patol@info.isbiel.ch>
- * 2-Sep-93: bounce version David Bagley <bagleyd@tux.org>
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Compatible with xscreensaver
+ * 18-Sep-1995: 5 bats now in color <patol@info.isbiel.ch>
+ * 20-Sep-1994: 5 bats instead of bouncing balls, based on bounce.c
+ *              <patol@info.isbiel.ch>
+ * 02-Sep-1993: bounce version David Bagley <bagleyd@tux.org>
  * 1986: Sun Microsystems
  */
 
@@ -108,6 +109,19 @@ ModStruct   bat_description =
 #include "bitmaps/bat-3.xbm"
 #include "bitmaps/bat-4.xbm"
 
+/* aliases for vars defined in the bitmap file */
+#define BAT_WIDTH     image_width
+#define BAT_HEIGHT    image_height
+#define BAT_BITS      image_bits
+
+#include "bat.xbm"
+
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+#define BAT_NAME      image_name
+#include "bat.xpm"
+#define DEFAULT_XPM 1
+#endif
+
 #define MAX_STRENGTH 24
 #define FRICTION 15
 #define PENETRATION 0.4
@@ -145,11 +159,13 @@ typedef struct {
 	int         width, height;
 	int         nbats;
 	int         xs, ys;
+	int         floor;
 	int         avgsize;
 	int         restartnum;
 	int         graphics_format;
 	int         pixelmode;
 	GC          backGC;
+	XImage     *logo;
 	Pixmap      pixmap;
 	Colormap    cmap;
 	unsigned long black;
@@ -238,9 +254,13 @@ drawbat(ModeInfo * mi, batstruct * bat)
 			0, 0, bp->xs, bp->ys, bat->x, bat->y, 1L);
 	} else {
 		XSetForeground(display, bp->backGC, bat->color);
+		if (bp->logo)
+			(void) XPutImage(display, window, bp->backGC, bp->logo,
+				 0, 0, bat->x, bat->y, bp->xs, bp->ys);
+	 	else
 /* PURIFY 4.0.1 on SunOS4 and on Solaris 2 reports a 15985 byte memory leak on
    * the next line. */
-		(void) XPutImage(display, window, bp->backGC,
+			(void) XPutImage(display, window, bp->backGC,
 				 bp->images[(bat->orient > ORIENTS / 2) ?
 					ORIENTS - bat->orient : bat->orient],
 				 0, 0, bat->x, bat->y, bp->xs, bp->ys);
@@ -284,17 +304,21 @@ movebat(bouncestruct * bp, batstruct * bat)
 	}
 	bat->vy++;
 	bat->y += bat->vy;
-	if (bat->y >= (bp->height + bp->ys)) {	/* Don't see bat bounce */
+	if (bat->y >= (bp->height + bp->floor * bp->ys)) {
+		/* Do not want to see bat bounce */
 		/* Bounce off the bottom edge */
 		bat->y = (bp->height - bp->ys);
 		bat->vy = -bat->vy + bat->vy / FRICTION;
 		flapbat(bat, -1, &bat->vx, bp->avgsize);
-	}			/* else if (bat->y < 0) { */
-	/* Bounce off the top edge */
-	/*bat->y = -bat->y;
-	   bat->vy = -bat->vy + bat->vy / FRICTION;
-	   flapbat(bat, 1, &bat->vx, bp->avgsize);
-	   } */
+	}
+#if 0
+	else if (bat->y < 0) {
+		/* Bounce off the top edge */
+		bat->y = -bat->y;
+		bat->vy = -bat->vy + bat->vy / FRICTION;
+		flapbat(bat, 1, &bat->vx, bp->avgsize);
+	}
+#endif
 	if (bat->spindir) {
 		bat->spincount--;
 		if (!bat->spincount) {
@@ -320,6 +344,52 @@ collide(bouncestruct * bp, int a_bat)
 }
 
 static void
+free_stuff(Display * display, bouncestruct * bp)
+{
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+	if (bp->graphics_format == IS_XPM) {
+		int         i;
+
+		for (i = 0; i <= ORIENTS / 2; i++) {
+			if (bp->images[i]) {
+				(void) XDestroyImage(bp->images[i]);
+				bp->images[i] = None;
+			}
+		}
+		bp->graphics_format = IS_NONE;
+	}
+	if (bp->cmap != None) {
+		XFreeColormap(display, bp->cmap);
+		if (bp->backGC != None) {
+			XFreeGC(display, bp->backGC);
+			bp->backGC = None;
+		}
+		bp->cmap = None;
+	} else
+		bp->backGC = None;
+#endif
+	if (bp->logo != None) {
+		destroyImage(&bp->logo, &bp->graphics_format);
+		bp->logo = None;
+	}
+}
+
+static void
+free_bat(Display *display, bouncestruct *bp)
+{
+	free_stuff(display, bp);
+	if (bp->bats != NULL) {
+		(void) free((void *) bp->bats);
+		bp->bats = NULL;
+	}
+	if (bp->pixmap != None) {
+		XFreePixmap(display, bp->pixmap);
+		bp->pixmap = None;
+	}
+		
+}
+
+static Bool
 init_stuff(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -327,7 +397,20 @@ init_stuff(ModeInfo * mi)
 	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
 	int i;
 
-	if (!bp->cmap && !bp->pixelmode) {
+	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
+		if (bp->logo == None) {
+                getImage(mi, &bp->logo, BAT_WIDTH, BAT_HEIGHT, BAT_BITS,
+#if defined( USE_XPM ) || defined( USE_XPMINC )
+                         DEFAULT_XPM, BAT_NAME,
+#endif
+                         &bp->graphics_format, &bp->cmap, &bp->black);
+			if (bp->logo == None) {
+				free_bat(display, bp);
+				return False;
+			}
+		}
+	} else {
+	if (!bp->cmap) {
 #if defined( USE_XPM ) || defined( USE_XPMINC )
 		int         total = 0;
 
@@ -336,7 +419,11 @@ init_stuff(ModeInfo * mi)
 
 #ifndef STANDALONE
 			if (!fixedColors(mi)) {
-				bp->cmap = XCreateColormap(display, window, MI_VISUAL(mi), AllocNone);
+				if ((bp->cmap = XCreateColormap(display, window,
+						MI_VISUAL(mi), AllocNone)) == None) {
+					free_bat(display, bp);
+					return False;
+				}
 				attrib.colormap = bp->cmap;
 				reserveColors(mi, bp->cmap, &bp->black);
 			} else
@@ -389,6 +476,7 @@ init_stuff(ModeInfo * mi)
 				bp->images[i] = &(bimages[i]);
 		}
 	}
+  }
 	if (bp->cmap != None) {
 
 #ifndef STANDALONE
@@ -398,49 +486,27 @@ init_stuff(ModeInfo * mi)
 			XGCValues   xgcv;
 
 			xgcv.background = bp->black;
-			bp->backGC = XCreateGC(display, window, GCBackground, &xgcv);
+			if ((bp->backGC = XCreateGC(display, window,
+					GCBackground, &xgcv)) == None) {
+				free_bat(display, bp);
+				return False;
+			}
 		}
 	} else {
 		bp->black = MI_BLACK_PIXEL(mi);
 		bp->backGC = MI_GC(mi);
 	}
+	return True;
 }
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-static void
-free_stuff(Display * display, bouncestruct * bp)
-{
-	if (bp->graphics_format == IS_XPM) {
-		int         i;
-
-		for (i = 0; i <= ORIENTS / 2; i++) {
-			if (bp->images[i]) {
-				(void) XDestroyImage(bp->images[i]);
-				bp->images[i] = None;
-			}
-		}
-		bp->graphics_format = IS_NONE;
-	}
-	if (bp->cmap != None) {
-		XFreeColormap(display, bp->cmap);
-		if (bp->backGC != None) {
-			XFreeGC(display, bp->backGC);
-			bp->backGC = None;
-		}
-		bp->cmap = None;
-	} else
-		bp->backGC = None;
-}
-#endif
-
 void
 init_bat(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	bouncestruct *bp;
 	int         size = MI_SIZE(mi);
 	int         i, tryagain = 0;
+	bouncestruct *bp;
 
 	if (bounces == NULL) {
 		if ((bounces = (bouncestruct *) calloc(MI_NUM_SCREENS(mi),
@@ -449,10 +515,7 @@ init_bat(ModeInfo * mi)
 	}
 	bp = &bounces[MI_SCREEN(mi)];
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
 	free_stuff(display, bp);
-#endif
-
 	bp->width = MI_WIDTH(mi);
 	bp->height = MI_HEIGHT(mi);
 	if (bp->width < 2)
@@ -460,6 +523,7 @@ init_bat(ModeInfo * mi)
 	if (bp->height < 2)
 		bp->height = 2;
 	bp->restartnum = TIME;
+	bp->floor = NRAND(3) + 2;
 
 	bp->nbats = MI_COUNT(mi);
 	if (bp->nbats < -MINBATS) {
@@ -471,19 +535,32 @@ init_bat(ModeInfo * mi)
 		bp->nbats = NRAND(-bp->nbats - MINBATS + 1) + MINBATS;
 	} else if (bp->nbats < MINBATS)
 		bp->nbats = MINBATS;
-	if (!bp->bats)
-		bp->bats = (batstruct *) malloc(bp->nbats * sizeof (batstruct));
+	if (!bp->bats) {
+		if ((bp->bats = (batstruct *) malloc(bp->nbats *
+				 sizeof (batstruct))) == NULL) {
+			free_bat(display, bp);
+			return;
+		}
+	}
+	if (!init_stuff(mi))
+		return;
 	if (size == 0 ||
 	    MINGRIDSIZE * size > bp->width / 2 || MINGRIDSIZE * size > bp->height) {
-		if (bp->width > MINGRIDSIZE * bat0_width &&
-		    bp->height > MINGRIDSIZE * bat0_height) {
-			bp->pixelmode = False;
+		if (bp->logo) {
+			bp->xs = bp->logo->width;
+			bp->ys = bp->logo->height;
+		} else {
 			bp->xs = bat0_width;
 			bp->ys = bat0_height;
+		}
+		if (bp->width > MINGRIDSIZE * bp->xs &&
+		    bp->height > MINGRIDSIZE * bp->ys) {
+			bp->pixelmode = False;
 		} else {
 			bp->pixelmode = True;
 			bp->ys = MAX(MINSIZE, MIN(bp->width / 2, bp->height) / MINGRIDSIZE);
 			bp->xs = 2 * bp->ys;
+			free_stuff(display, bp); /* too big */
 		}
 	} else {
 		bp->pixelmode = True;
@@ -503,16 +580,26 @@ init_bat(ModeInfo * mi)
 		GC fg_gc, bg_gc;
 		XGCValues gcv;
 
-		bp->pixmap = XCreatePixmap(display, window,
-		    bp->xs, bp->ys, 1);
+		if ((bp->pixmap = XCreatePixmap(display, window, bp->xs, bp->ys, 1)) ==
+				None) {
+			free_bat(display, bp);
+			return;
+		}
 		gcv.foreground = 0;
 		gcv.background = 1;
-		bg_gc = XCreateGC(display, bp->pixmap,
-	         GCForeground | GCBackground, &gcv);
+		if ((bg_gc = XCreateGC(display, bp->pixmap,
+				 GCForeground | GCBackground, &gcv)) == None) {
+			free_bat(display, bp);
+			return;
+		}
 		gcv.background = 0;
 		gcv.foreground = 1;
-		fg_gc = XCreateGC(display, bp->pixmap,
-	        GCForeground | GCBackground, &gcv);
+		if ((fg_gc = XCreateGC(display, bp->pixmap,
+				 GCForeground | GCBackground, &gcv)) == None) {
+			XFreeGC(display, bg_gc);
+			free_bat(display, bp);
+			return;
+		}
 		XFillRectangle(display, bp->pixmap, bg_gc,
 	   		0, 0, bp->xs, bp->ys);
 		XFillArc(display, bp->pixmap, fg_gc,
@@ -535,8 +622,6 @@ init_bat(ModeInfo * mi)
 
 		bp->black = MI_BLACK_PIXEL(mi);
 		bp->backGC = MI_GC(mi);
-	} else {
-		init_stuff(mi);
 	}
 
 	i = 0;
@@ -569,11 +654,16 @@ init_bat(ModeInfo * mi)
 void
 draw_bat(ModeInfo * mi)
 {
-	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
 	int           i;
+	bouncestruct *bp;
+
+	if (bounces == NULL)
+		return;
+	bp = &bounces[MI_SCREEN(mi)];
+	if (bp->bats == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	for (i = 0; i < bp->nbats; i++) {
 		drawbat(mi, &bp->bats[i]);
 		movebat(bp, &bp->bats[i]);
@@ -592,16 +682,8 @@ release_bat(ModeInfo * mi)
 	if (bounces != NULL) {
 		int        screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			bouncestruct *bp = &bounces[screen];
-
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-			free_stuff(MI_DISPLAY(mi), bp);
-#endif
-
-			if (bp->bats != NULL)
-				(void) free((void *) bp->bats);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_bat(MI_DISPLAY(mi), &bounces[screen]);
 		(void) free((void *) bounces);
 		bounces = NULL;
 	}
@@ -610,13 +692,25 @@ release_bat(ModeInfo * mi)
 void
 refresh_bat(ModeInfo * mi)
 {
-	bouncestruct *bp = &bounces[MI_SCREEN(mi)];
+	bouncestruct *bp;
+
+	if (bounces == NULL)
+		return;
+	bp = &bounces[MI_SCREEN(mi)];
+	if (bp->bats == NULL)
+		return;
 
-	MI_CLEARWINDOWCOLORMAP(mi, bp->backGC, bp->black);
 #if defined( USE_XPM ) || defined( USE_XPMINC )
-	/* This is needed when another program changes the colormap. */
-	free_stuff(MI_DISPLAY(mi), bp);
-	init_stuff(mi);
+	/* This is only needed when another program changes the colormap. */
+	if (MI_BITMAP(mi) && strlen(MI_BITMAP(mi))) {
+		init_bat(mi);
+	} else {
+		MI_CLEARWINDOWCOLORMAP(mi, bp->backGC, bp->black);
+		free_stuff(MI_DISPLAY(mi), bp);
+		(void) init_stuff(mi);
+	}
+#else
+	MI_CLEARWINDOWCOLORMAP(mi, bp->backGC, bp->black);
 #endif
 }
 
