@@ -2,7 +2,7 @@
 /* crystal --- polygons moving according to plane group rules */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)crystal.c	4.12 98/09/10 xlockmore";
+static const char sccsid[] = "@(#)crystal.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -33,40 +33,42 @@ static const char sccsid[] = "@(#)crystal.c	4.12 98/09/10 xlockmore";
  *   hexagonal groups 12 through 17 where the angle gamma is 120 degrees
  *
  * Revision History:
- * 03-Dec-98: Random inversion of y-axis included to simulate hexagonal groups
- *             with an angle of 60 degrees.
- * 10-Sep-98: new colour scheme
- * 24-Feb-98: added option centre which turns on/off forcing the centre of
+ * 01-Nov-2000: Allocation checks
+ * 03-Dec-1998: Random inversion of y-axis included to simulate hexagonal
+ *              groups with an angle of 60 degrees.
+ * 10-Sep-1998: new colour scheme
+ * 24-Feb-1998: added option centre which turns on/off forcing the centre of
  *              the screen to be used
- *            added option maxsize which forces the dimensions to be chasen
- *              in such ua way that the largest possible part of the screen is
+ *              added option maxsize which forces the dimensions to be chosen
+ *              in such a way that the largest possible part of the screen is
  *              used
- *            When only one unit cell is drawn, it is chosen at random
- * 18-Feb-98: added support for negative numbers with -nx and -ny meaning
- *            "random" choice with given maximum
- *            added +/-grid option. If -cell is specified this option
- *            determines if one or all unit cells are drawn.
- *            -batchcount is now a parameter for all the objects on the screen
- *            instead of the number of "unique" objects
- *            The maximum size of the objects now scales with the part
- *            of the screen used.
- *            fixed "size" problem. Now very small non-vissable objects
- *            are not allowed
- * 13-Feb-98: randomized the unit cell size
- *            runtime options -/+cell (turn on/off unit cell drawing)
- *             -nx num (number of translational symmetries in x-direction
- *             -ny num (idem y-direction but ignored for square and
- *               hexagonal space groups
- *               i.e. try xlock -mode crystal -nx 3 -ny 2
- *            Fullrandom overrules the -/+cell option.
- * 05-Feb-98: Revision + bug repairs
- *            shows unit cell
- *            use part of the screen for unit cell
- *            in hexagonal and square groups a&b axis forced to be equal
- *            cell angle for oblique groups randomly chosen between 60 and 120
- *   bugs solved: planegroups with cell angles <> 90.0 now work properly
- * 19-Sep-97: Added remaining hexagonal groups
- * 12-Jun-97: Created
+ *              When only one unit cell is drawn, it is chosen at random
+ * 18-Feb-1998: added support for negative numbers with -nx and -ny meaning
+ *              "random" choice with given maximum
+ *              added +/-grid option. If -cell is specified this option
+ *              determines if one or all unit cells are drawn.
+ *              -batchcount is now a parameter for all the objects on the
+ *              screen instead of the number of "unique" objects
+ *              The maximum size of the objects now scales with the part
+ *              of the screen used.
+ *              fixed "size" problem. Now very small non-visable objects
+ *              are not allowed
+ * 13-Feb-1998: randomized the unit cell size
+ *              runtime options -/+cell (turn on/off unit cell drawing)
+ *              -nx num (number of translational symmetries in x-direction
+ *              -ny num (idem y-direction but ignored for square and
+ *              hexagonal space groups
+ *              i.e. try xlock -mode crystal -nx 3 -ny 2
+ *              Fullrandom overrules the -/+cell option.
+ * 05-Feb-1998: Revision + bug repairs
+ *              shows unit cell
+ *              use part of the screen for unit cell
+ *              in hexagonal and square groups a&b axis forced to be equal
+ *              cell angle for oblique groups randomly chosen between 60 and 120
+ *              bugs solved: planegroups with cell angles <> 90.0 now work
+ *              properly
+ * 19-Sep-1997: Added remaining hexagonal groups
+ * 12-Jun-1997: Created
  */
 
 #ifdef STANDALONE
@@ -268,6 +270,7 @@ typedef struct {
 	Bool        cycle_p, mono_p, no_colors;
 	unsigned long blackpixel, whitepixel, fg, bg;
 	int         direction , invert;
+	ModeInfo   *mi;
 } crystalstruct;
 
 static crystalstruct *crystals = NULL;
@@ -559,15 +562,54 @@ crystal_drawatom(ModeInfo * mi, crystalatom * atom0)
 	}
 }
 
+static void
+free_crystal(Display *display, crystalstruct *cryst)
+{
+	ModeInfo *mi = cryst->mi;
+
+	if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
+		MI_WHITE_PIXEL(mi) = cryst->whitepixel;
+		MI_BLACK_PIXEL(mi) = cryst->blackpixel;
+#ifndef STANDALONE
+		MI_FG_PIXEL(mi) = cryst->fg;
+		MI_BG_PIXEL(mi) = cryst->bg;
+#endif
+		if (cryst->colors != NULL) {
+			if (cryst->ncolors && !cryst->no_colors)
+				free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
+			(void) free((void *) cryst->colors);
+			cryst->colors = NULL;
+		}
+		if (cryst->cmap != None) {
+			XFreeColormap(display, cryst->cmap);
+			cryst->cmap = None;
+		}
+	}
+	if (cryst->gc != None) {
+		XFreeGC(display, cryst->gc);
+		cryst->gc = None;
+	}
+	if (cryst->atom != NULL) {
+		(void) free((void *) cryst->atom);
+		cryst->atom = NULL;
+	}
+}
+
 void
 draw_crystal(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
-	crystalstruct *cryst = &crystals[MI_SCREEN(mi)];
 	int         i;
+	crystalstruct *cryst;
+
+	if (crystals == NULL)
+		return;
+	cryst = &crystals[MI_SCREEN(mi)];
+	if (cryst->atom == NULL)
+		return;
 
 	if (cryst->no_colors) {
-		release_crystal(mi);
+		free_crystal(display, cryst);
 		init_crystal(mi);
 		return;
 	}
@@ -621,8 +663,14 @@ refresh_crystal(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	crystalstruct *cryst = &crystals[MI_SCREEN(mi)];
 	int         i;
+	crystalstruct *cryst;
+
+	if (crystals == NULL)
+		return;
+	cryst = &crystals[MI_SCREEN(mi)];
+	if (cryst->atom == NULL)
+		return;
 
 	if (!cryst->painted)
 		return;
@@ -786,33 +834,11 @@ refresh_crystal(ModeInfo * mi)
 void
 release_crystal(ModeInfo * mi)
 {
-	Display    *display = MI_DISPLAY(mi);
-
 	if (crystals != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			crystalstruct *cryst = &crystals[screen];
-
-			if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
-				MI_WHITE_PIXEL(mi) = cryst->whitepixel;
-				MI_BLACK_PIXEL(mi) = cryst->blackpixel;
-#ifndef STANDALONE
-				MI_FG_PIXEL(mi) = cryst->fg;
-				MI_BG_PIXEL(mi) = cryst->bg;
-#endif
-				if (cryst->colors && cryst->ncolors && !cryst->no_colors)
-					free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
-				if (cryst->colors)
-					(void) free((void *) cryst->colors);
-                                if (cryst->cmap)
-				XFreeColormap(display, cryst->cmap);
-			}
-			if (cryst->gc != NULL)
-				XFreeGC(display, cryst->gc);
-			if (cryst->atom != NULL)
-				(void) free((void *) cryst->atom);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_crystal(MI_DISPLAY(mi), &crystals[screen]);
 		(void) free((void *) crystals);
 		crystals = NULL;
 	}
@@ -836,6 +862,7 @@ init_crystal(ModeInfo * mi)
 			return;
 	}
 	cryst = &crystals[MI_SCREEN(mi)];
+	cryst->mi = mi;
 
 	if (!cryst->gc) {
 		if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
@@ -850,8 +877,11 @@ init_crystal(ModeInfo * mi)
 #endif
 			cryst->blackpixel = MI_BLACK_PIXEL(mi);
 			cryst->whitepixel = MI_WHITE_PIXEL(mi);
-			cryst->cmap = XCreateColormap(display, window,
-						   MI_VISUAL(mi), AllocNone);
+			if ((cryst->cmap = XCreateColormap(display, window,
+					MI_VISUAL(mi), AllocNone)) == None) {
+				free_crystal(display, cryst);
+				return;
+			}
 			XSetWindowColormap(display, window, cryst->cmap);
 			(void) XParseColor(display, cryst->cmap, "black", &color);
 			(void) XAllocColor(display, cryst->cmap, &color);
@@ -867,12 +897,14 @@ init_crystal(ModeInfo * mi)
 			(void) XAllocColor(display, cryst->cmap, &color);
 			MI_FG_PIXEL(mi) = color.pixel;
 #endif
-			cryst->colors = 0;
+			cryst->colors = NULL;
 			cryst->ncolors = 0;
 		}
 		if ((cryst->gc = XCreateGC(display, MI_WINDOW(mi),
-			     (unsigned long) 0, (XGCValues *) NULL)) == None)
+			     (unsigned long) 0, (XGCValues *) NULL)) == None) {
+			free_crystal(display, cryst);
 			return;
+		}
 	}
 /* Clear Display */
 	MI_CLEARWINDOW(mi);
@@ -948,8 +980,11 @@ init_crystal(ModeInfo * mi)
 		cryst->num_atom = cryst->num_atom / neqv + 1;
 
 	if (cryst->atom == NULL)
-		cryst->atom = (crystalatom *) calloc(max_atoms, sizeof (
-							       crystalatom));
+		if ((cryst->atom = (crystalatom *) calloc(max_atoms,
+				sizeof (crystalatom))) == NULL) {
+			free_crystal(display, cryst);
+			return;
+		}
 
 	if (maxsize) {
 		if (cryst->planegroup < 13) {
@@ -1160,11 +1195,12 @@ init_crystal(ModeInfo * mi)
 	XSetFunction(display, cryst->gc, GXxor);
 	if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
 /* Set up colour map */
-		if (cryst->colors && cryst->ncolors && !cryst->no_colors)
-			free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
-		if (cryst->colors)
+		if (cryst->colors != NULL) {
+			if (cryst->ncolors && !cryst->no_colors)
+				free_colors(display, cryst->cmap, cryst->colors, cryst->ncolors);
 			(void) free((void *) cryst->colors);
-		cryst->colors = 0;
+			cryst->colors = NULL;
+		}
 		cryst->ncolors = MI_NCOLORS(mi);
 		if (cryst->ncolors < 2)
 			cryst->ncolors = 2;
@@ -1174,9 +1210,13 @@ init_crystal(ModeInfo * mi)
 			cryst->mono_p = False;
 
 		if (cryst->mono_p)
-			cryst->colors = 0;
+			cryst->colors = NULL;
 		else
-			cryst->colors = (XColor *) malloc(sizeof (*cryst->colors) * (cryst->ncolors + 1));
+			if ((cryst->colors = (XColor *) malloc(sizeof (*cryst->colors) *
+					(cryst->ncolors + 1))) == NULL) {
+				free_crystal(display, cryst);
+				return;
+			}
 		cryst->cycle_p = has_writable_cells(mi);
 		if (cryst->cycle_p) {
 			if (MI_IS_FULLRANDOM(mi)) {
