@@ -2,7 +2,7 @@
 /* dclock --- floating digital clock or message */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,17 +22,18 @@ static const char sccsid[] = "@(#)dclock.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 07-May-99: New "savers" added for y2k and second millennium countdowns.
- *            Tom Schmidt <tschmidt@micron.com>
- * 04-Dec-98: New "savers" added for hiv, veg, and lab.
- *            hiv one due to Kenneth Stailey <kstailey@disclosure.com>
- * 10-Aug-98: Population Explosion and Tropical Forest Countdown stuff
- *            I tried to get precise numbers but they may be off a few percent.
- *            Whether or not, its still pretty scary IMHO.
- *            Although I am a US citizen... I have the default for area in
- *            metric.  David Bagley <bagleyd@tux.org>
- * 10-May-97: Compatible with xscreensaver
- * 29-Aug-95: Written.
+ * 01-Nov-2000: Allocation checks
+ * 07-May-1999: New "savers" added for y2k and second millennium countdowns.
+ *              Tom Schmidt <tschmidt@micron.com>
+ * 04-Dec-1998: New "savers" added for hiv, veg, and lab.
+ *              hiv one due to Kenneth Stailey <kstailey@disclosure.com>
+ * 10-Aug-1998: Population Explosion and Tropical Forest Countdown stuff
+ *              I tried to get precise numbers but they may be off a few
+ *              percent.  Whether or not, its still pretty scary IMHO.
+ *              Although I am a US citizen... I have the default for area in
+ *              metric.  David Bagley <bagleyd@tux.org>
+ * 10-May-1997: Compatible with xscreensaver
+ * 29-Aug-1995: Written.
  */
 
 /*-
@@ -374,6 +375,14 @@ static XFontStruct *mode_font = None;
 extern XFontStruct *getFont(Display * display);
 #endif
 
+#ifdef USE_MB
+#define free_font(d) if (mode_font!=None){XFreeFontSet(d,mode_font); \
+mode_font = None;}
+#else
+#define free_font(d) if (mode_font!=None){XFreeFont(d,mode_font); \
+mode_font = None;}
+#endif
+
 #define STRSIZE 50
 
 typedef struct {
@@ -405,7 +414,7 @@ timeAtLastNewYear(long timeNow)
 {
   struct tm *t;
 
-  t = localtime(&timeNow);
+  t = localtime((const time_t *) &timeNow);
   return (unsigned long)(t->tm_year);
 }
 
@@ -421,7 +430,7 @@ convert(double x, char *string)
 #ifdef HAVE_SNPRINTF
 /* Also old C compiler can not accept this syntax, but this syntax awares
    locale.  Known to work with gcc-2.95.2 and glibc-2.1.3. */
-	snprintf(string, STRSIZE, "%'.0f", x);
+	(void) snprintf(string, STRSIZE, "%'.0f", x);
 #else
 	
 	int         i, j, k = 0;
@@ -440,7 +449,7 @@ convert(double x, char *string)
 			string[k++] = ',';
 		}
 	}
-	string[k++] = '\0';
+	string[k] = '\0';
 #endif
 }
 
@@ -608,6 +617,23 @@ drawanumber(ModeInfo * mi, int startx, int starty, int digit)
 	}
 }
 
+static void
+free_dclock(Display *display, dclockstruct *dp)
+{
+	if (dp->fgGC != None) {
+		XFreeGC(display, dp->fgGC);
+		dp->fgGC = None;
+	}
+	if (dp->bgGC) {
+		XFreeGC(display, dp->bgGC);
+		dp->bgGC = None;
+	}
+	if (dp->pixmap) {
+		XFreePixmap(display, dp->pixmap);
+		dp->pixmap = None;
+	}
+}
+
 static void
 drawDclock(ModeInfo * mi)
 {
@@ -616,7 +642,6 @@ drawDclock(ModeInfo * mi)
 	GC          gc = MI_GC(mi);
 	dclockstruct *dp = &dclocks[MI_SCREEN(mi)];
 	short       xold, yold;
-	char       *tmppt;
 
 	if (dp->led) {
 		dp->timeold = dp->timenew = time((time_t *) NULL);
@@ -629,9 +654,9 @@ drawDclock(ModeInfo * mi)
 
 			if (!dp->popex && !dp->forest && !dp->hiv && !dp->lab &&
 			    !dp->veg && !dp->y2k && !dp->millennium) {
-				strftime(dp->str1pta, STRSIZE, "%I:%M:%S %p", localtime(&(dp->timeold)));
+				(void) strftime(dp->str1pta, STRSIZE, "%I:%M:%S %p", localtime(&(dp->timeold)));
 			}
-			strftime(dp->str2pta, STRSIZE, "%a %b %d %Y", localtime(&(dp->timeold)));
+			(void) strftime(dp->str2pta, STRSIZE, "%a %b %d %Y", localtime(&(dp->timeold)));
 		}
 	} else {
 		long timeNow, timeLocal;
@@ -668,18 +693,18 @@ drawDclock(ModeInfo * mi)
 			dp->str2ptb = dp->str2pta;
 		} else if (dp->y2k) {
 			if (Y2K_TIME_START >= timeLocal)
-				dp->str1pta = Y2K_STRING;
+				dp->str1pta = (char *) Y2K_STRING;
 			else
-				dp->str1pta = POST_Y2K_STRING;
+				dp->str1pta = (char *) POST_Y2K_STRING;
 			dp->str1ptb = dp->str1pta;
 			dayhrminsec(Y2K_TIME_START - timeLocal, dp->tzoffset, dp->str2);
 			dp->str2pta = dp->str2;
 			dp->str2ptb = dp->str2pta;
 		} else if (dp->millennium) {
 			if (Y2001_TIME_START >= timeLocal)
-				dp->str1pta = Y2001_STRING;
+				dp->str1pta = (char *) Y2001_STRING;
 			else
-				dp->str1pta = POST_Y2001_STRING;
+				dp->str1pta = (char *) POST_Y2001_STRING;
 			dp->str1ptb = dp->str1pta;
 			dayhrminsec(Y2001_TIME_START - timeLocal, dp->tzoffset, dp->str2);
 			dp->str2pta = dp->str2;
@@ -690,10 +715,7 @@ drawDclock(ModeInfo * mi)
 	xold = dp->clockx;
 	yold = dp->clocky;
 	if (!dp->led) {
-#ifndef USE_MB
-		dp->text_width1 = XTextWidth(mode_font, dp->str1pta, strlen(dp->str1pta));
-		dp->text_width2 = XTextWidth(mode_font, dp->str2pta, strlen(dp->str2pta));
-#else
+#ifdef USE_MB
 		{
 			XRectangle ink, logical;
 
@@ -702,6 +724,9 @@ drawDclock(ModeInfo * mi)
 			XmbTextExtents(mode_font, dp->str2pta, strlen(dp->str2pta), &ink, &logical);
 			dp->text_width2 = logical.width;
 		}
+#else
+		dp->text_width1 = XTextWidth(mode_font, dp->str1pta, strlen(dp->str1pta));
+		dp->text_width2 = XTextWidth(mode_font, dp->str2pta, strlen(dp->str2pta));
 #endif
 
 		if (dp->text_width1 > dp->text_width2) {
@@ -748,40 +773,51 @@ drawDclock(ModeInfo * mi)
 	    dp->pixh != (1 + !dp->led) * dp->text_height) {
 		XGCValues   gcv;
 
-		if (dp->fgGC)
-			XFreeGC(display, dp->fgGC);
-		if (dp->bgGC)
-			XFreeGC(display, dp->bgGC);
-		if (dp->pixmap) {
-			XFreePixmap(display, dp->pixmap);
+		if (dp->pixmap)
 			MI_CLEARWINDOWCOLORMAPFAST(mi, gc, MI_BLACK_PIXEL(mi));
-		}
+		free_dclock(display, dp);
 		dp->pixw = dp->text_width;
 		if (dp->led)
 			dp->pixh = dp->text_height;
 		else
 			dp->pixh = (2 + DELTA) * dp->text_height;
-		dp->pixmap = XCreatePixmap(display, window, dp->pixw, dp->pixh, 1);
+		if ((dp->pixmap =
+			XCreatePixmap(display, window, dp->pixw, dp->pixh, 1)) == None) {
+			free_dclock(display, dp);
+			dp->pixw = 0;
+			dp->pixh = 0;
+			return;
+		}
 #ifndef USE_MB
 		gcv.font = mode_font->fid;
 #endif
 		gcv.background = 0;
 		gcv.foreground = 1;
 		gcv.graphics_exposures = False;
-		dp->fgGC = XCreateGC(display, dp->pixmap,
+		if ((dp->fgGC = XCreateGC(display, dp->pixmap,
 			GCForeground | GCBackground | GCGraphicsExposures
 #ifndef USE_MB
 			| GCFont
 #endif
-			, &gcv);
+			, &gcv)) == None) {
+			free_dclock(display, dp);
+			dp->pixw = 0;
+			dp->pixh = 0;
+			return;
+		}
 		gcv.foreground = 0;
-		dp->bgGC = XCreateGC(display, dp->pixmap,
+		if ((dp->bgGC = XCreateGC(display, dp->pixmap,
 			GCForeground | GCBackground | GCGraphicsExposures
 #ifndef USE_MB
 			| GCFont
 #endif
-			, &gcv);
-		XSetLineAttributes(MI_DISPLAY(mi), dp->fgGC,
+			, &gcv)) == None) {
+			free_dclock(display, dp);
+			dp->pixw = 0;
+			dp->pixh = 0;
+			return;
+		}
+		XSetLineAttributes(display, dp->fgGC,
 		  (unsigned int) (LED_WIDTH),
 		  LineSolid, CapButt, JoinMiter);
 	}
@@ -807,18 +843,18 @@ drawDclock(ModeInfo * mi)
 		startx += (int) (LED_XS + LED_WIDTH + LED_INC);
 		drawanumber(mi, startx, starty, dp->str[18] - '0');
 	} else {
-#ifndef USE_MB
-		(void) XDrawString(display, dp->pixmap, dp->fgGC,
+#ifdef USE_MB
+		(void) XmbDrawString(display, dp->pixmap, mode_font, dp->fgGC,
 			dp->text_start1, dp->text_ascent,
 			dp->str1pta, strlen(dp->str1pta));
-		(void) XDrawString(display, dp->pixmap, dp->fgGC,
+		(void) XmbDrawString(display, dp->pixmap, mode_font, dp->fgGC,
 			dp->text_start2, dp->text_ascent + dp->text_height,
 			dp->str2pta, strlen(dp->str2pta));
 #else
-		(void) XmbDrawString(display, dp->pixmap, mode_font, dp->fgGC,
+		(void) XDrawString(display, dp->pixmap, dp->fgGC,
 			dp->text_start1, dp->text_ascent,
 			dp->str1pta, strlen(dp->str1pta));
-		(void) XmbDrawString(display, dp->pixmap, mode_font, dp->fgGC,
+		(void) XDrawString(display, dp->pixmap, dp->fgGC,
 			dp->text_start2, dp->text_ascent + dp->text_height,
 			dp->str2pta, strlen(dp->str2pta));
 #endif
@@ -939,11 +975,15 @@ init_dclock(ModeInfo * mi)
 	}
 
 	if (mode_font == None) {
-#ifndef USE_MB
-		mode_font = getFont(display);
-#else
+#ifdef USE_MB
 		mode_font = getFontSet(display);
+#else
+		mode_font = getFont(display);
 #endif
+		if (mode_font == None) {
+			release_dclock(mi);
+			return;
+		}
 	}
 	if (!dp->done) {
 		dp->done = 1;
@@ -1007,46 +1047,46 @@ init_dclock(ModeInfo * mi)
 		else
 			dp->clocky = NRAND(dp->maxy);
 	} else {
-#ifndef USE_MB
-		dp->text_descent = mode_font->descent;
-		dp->text_ascent = mode_font->ascent;
-#else
+#ifdef USE_MB
 		dp->text_descent = 0;
 		dp->text_ascent = font_height(mode_font);
+#else
+		dp->text_descent = mode_font->descent;
+		dp->text_ascent = mode_font->ascent;
 #endif
 		if (dp->popex) {
-			dp->str1pta = POPEX_STRING;
+			dp->str1pta = (char *) POPEX_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->forest) {
-			dp->str1pta = FOREST_STRING;
+			dp->str1pta = (char *) FOREST_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->hiv) {
-			dp->str1pta = HIV_STRING;
+			dp->str1pta = (char *) HIV_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->lab) {
-			dp->str1pta = LAB_STRING;
+			dp->str1pta = (char *) LAB_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->veg) {
-			dp->str1pta = VEG_STRING;
+			dp->str1pta = (char *) VEG_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->y2k) {
 			if (Y2K_TIME_START >= timeLocal)
-				dp->str1pta = Y2K_STRING;
+				dp->str1pta = (char *) Y2K_STRING;
 			else
-				dp->str1pta = POST_Y2K_STRING;
+				dp->str1pta = (char *) POST_Y2K_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else if (dp->millennium) {
 			if (Y2001_TIME_START >= timeLocal)
-				dp->str1pta = Y2001_STRING;
+				dp->str1pta = (char *) Y2001_STRING;
 			else
-				dp->str1pta = POST_Y2001_STRING;
+				dp->str1pta = (char *) POST_Y2001_STRING;
 			dp->str1ptb = dp->str1pta;
 		} else {
-			struct tm *t = localtime(&timeLocal);
+			struct tm *t = localtime((const time_t *) &timeLocal);
 
-			strftime(dp->str1, STRSIZE, "%I:%M:%S %p", t);
+			(void) strftime(dp->str1, STRSIZE, "%I:%M:%S %p", t);
 			dp->hour = t->tm_hour;
-			strftime(dp->str2, STRSIZE, "%a %b %d %Y", t);
+			(void) strftime(dp->str2, STRSIZE, "%a %b %d %Y", t);
 
 			dp->str1pta = dp->str1;
 			dp->str1ptb = dp->str1old;
@@ -1090,10 +1130,7 @@ init_dclock(ModeInfo * mi)
 		}
 
 		dp->text_height = font_height(mode_font);
-#ifndef USE_MB
-		dp->text_width1 = XTextWidth(mode_font, dp->str1pta, strlen(dp->str1pta));
-		dp->text_width2 = XTextWidth(mode_font, dp->str2pta, strlen(dp->str2pta));
-#else
+#ifdef USE_MB
 		{
 			XRectangle ink, logical;
 
@@ -1102,6 +1139,9 @@ init_dclock(ModeInfo * mi)
 			XmbTextExtents(mode_font, dp->str2pta, strlen(dp->str2pta), &ink, &logical);
 			dp->text_width2 = logical.width;
 		}
+#else
+		dp->text_width1 = XTextWidth(mode_font, dp->str1pta, strlen(dp->str1pta));
+		dp->text_width2 = XTextWidth(mode_font, dp->str2pta, strlen(dp->str2pta));
 #endif
 		if (dp->text_width1 > dp->text_width2) {
 			dp->text_width = dp->text_width1;
@@ -1138,10 +1178,15 @@ init_dclock(ModeInfo * mi)
 void
 draw_dclock(ModeInfo * mi)
 {
-	dclockstruct *dp = &dclocks[MI_SCREEN(mi)];
+	dclockstruct *dp;
+
+	if (dclocks == NULL)
+		return;
+	dp = &dclocks[MI_SCREEN(mi)];
+	if (mode_font == None)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	drawDclock(mi);
 	if (MI_NPIXELS(mi) > 2) {
 		if (++dp->color >= MI_NPIXELS(mi))
@@ -1155,30 +1200,12 @@ release_dclock(ModeInfo * mi)
 	if (dclocks != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			dclockstruct *dp = &dclocks[screen];
-			Display    *display = MI_DISPLAY(mi);
-
-			if (dp->fgGC)
-				XFreeGC(display, dp->fgGC);
-			if (dp->bgGC)
-				XFreeGC(display, dp->bgGC);
-			if (dp->pixmap)
-				XFreePixmap(display, dp->pixmap);
-
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_dclock(MI_DISPLAY(mi), &dclocks[screen]);
 		(void) free((void *) dclocks);
 		dclocks = NULL;
 	}
-	if (mode_font != None) {
-#ifndef USE_MB
-		XFreeFont(MI_DISPLAY(mi), mode_font);
-#else
-		XFreeFontSet(MI_DISPLAY(mi), mode_font);
-#endif
-
-		mode_font = None;
-	}
+	free_font(MI_DISPLAY(mi));
 }
 
 void
