@@ -2,7 +2,7 @@
 /* drift --- drifting recursive fractal cosmic flames */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)drift.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)drift.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,12 +22,13 @@ static const char sccsid[] = "@(#)drift.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Jamie Zawinski <jwz@jwz.org> compatible with xscreensaver
- * 01-Jan-97: Moved new flame to drift.  Compile time options now run time.
- * 01-Jun-95: Updated by Scott Draves.
- * 27-Jun-91: vary number of functions used.
- * 24-Jun-91: fixed portability problem with integer mod (%).
- * 06-Jun-91: Written, received from Scott Draves <spot@cs.cmu.edu>
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Jamie Zawinski <jwz@jwz.org> compatible with xscreensaver
+ * 01-Jan-1997: Moved new flame to drift.  Compile time options now run time.
+ * 01-Jun-1995: Updated by Scott Draves.
+ * 27-Jun-1991: vary number of functions used.
+ * 24-Jun-1991: fixed portability problem with integer mod (%).
+ * 06-Jun-1991: Written, received from Scott Draves <spot@cs.cmu.edu>
  */
 
 #ifdef STANDALONE
@@ -261,6 +262,19 @@ pick_df_coefs(ModeInfo * mi)
 	}
 }
 
+static void
+free_drift(driftstruct *dp)
+{
+	if (dp->ncpoints != NULL) {
+		(void) free((void *) dp->ncpoints);
+		dp->ncpoints = NULL;
+	}
+	if (dp->cpts != NULL) {
+		(void) free((void *) dp->cpts);
+		dp->cpts = NULL;
+	}
+}
+
 static void
 initfractal(ModeInfo * mi)
 {
@@ -272,11 +286,21 @@ initfractal(ModeInfo * mi)
 	dp->fuse = FUSE;
 	dp->total_points = 0;
 
-	if (!dp->ncpoints)
-		dp->ncpoints = (int *) malloc(sizeof (int) * MI_NCOLORS(mi));
+	if (!dp->ncpoints) {
+		if ((dp->ncpoints = (int *) malloc(sizeof (int) * MI_NCOLORS(mi))) ==
+			NULL) {
+			free_drift(dp);
+			return;
+		}
+	}
+	if (!dp->cpts) {
+		if ((dp->cpts = (XPoint *) malloc(MAXBATCH2 * sizeof (XPoint) *
+			 MI_NCOLORS(mi))) == NULL) {
+			free_drift(dp);
+			return;
+		}
+	}
 
-	if (!dp->cpts)
-		dp->cpts = (XPoint *) malloc(MAXBATCH2 * sizeof (XPoint) * MI_NCOLORS(mi));
 	if (dp->rainbow)
 		for (i = 0; i < MI_NPIXELS(mi); i++)
 			dp->ncpoints[i] = 0;
@@ -583,12 +607,16 @@ void
 draw_drift(ModeInfo * mi)
 {
 	Window      window = MI_WINDOW(mi);
-	driftstruct *dp = &drifts[MI_SCREEN(mi)];
+	driftstruct *dp;
 
-	dp->timer = 3000;
+	if (drifts == NULL)
+		return;
+	dp = &drifts[MI_SCREEN(mi)];
+	if (dp->ncpoints == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
+	dp->timer = 3000;
 	while (dp->timer) {
 		iter(dp);
 		draw(mi, dp, window);
@@ -628,12 +656,8 @@ release_drift(ModeInfo * mi)
 	if (drifts != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			driftstruct *dp = &drifts[screen];
-
-			(void) free((void *) dp->ncpoints);
-			(void) free((void *) dp->cpts);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_drift(&drifts[screen]);
 		(void) free((void *) drifts);
 		drifts = NULL;
 	}
