@@ -2,7 +2,7 @@
 /* euler2d --- 2 Dimensional Incompressible Inviscid Fluid Flow */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)euler2d.c 4.17 2000/16/06 xlockmore";
+static const char sccsid[] = "@(#)euler2d.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,9 +22,18 @@ static const char sccsid[] = "@(#)euler2d.c 4.17 2000/16/06 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 05-Jun-00: Adapted from flow.c Copyright (c) 1996 by Tim Auckland
- * 18-Jul-96: Adapted from swarm.c Copyright (c) 1991 by Patrick J. Naughton.
- * 31-Aug-90: Adapted from xswarm by Jeff Butterworth. (butterwo@ncsc.org)
+ * 04-Nov-2000: Added an option eulerpower.  This allows for example the
+ *              quasi-geostrophic equation by setting eulerpower to 2.
+ * 01-Nov-2000: Allocation checks.
+ * 10-Sep-2000: Added optimizations, and removed subtle_perturb, by stephen.
+ * 03-Sep-2000: Changed method of solving ode to Adams-Bashforth of order 2.
+ *              Previously used a rather compilcated method of order 4.
+ *              This doubles the speed of the program.  Also it seems
+ *              to have improved numerical stability.  Done by stephen.
+ * 27-Aug-2000: Added rotation of region to maximize screen fill by stephen.
+ * 05-Jun-2000: Adapted from flow.c Copyright (c) 1996 by Tim Auckland
+ * 18-Jul-1996: Adapted from swarm.c Copyright (c) 1991 by Patrick J. Naughton.
+ * 31-Aug-1990: Adapted from xswarm by Jeff Butterworth. (butterwo@ncsc.org)
  */
 
 /*
@@ -51,25 +60,30 @@ static const char sccsid[] = "@(#)euler2d.c 4.17 2000/16/06 xlockmore";
 
 #define DEF_EULERTAIL "10"
 
-#define DEBUG_FLOATING_OVERFLOW 0
 #define DEBUG_POINTED_REGION    0
 
 static int tail_len;
 static int variable_boundary = 1;
+static float power = 1;
 
 static XrmOptionDescRec opts[] =
 {
   {(char* ) "-eulertail", (char *) ".euler2d.eulertail",
    XrmoptionSepArg, (caddr_t) NULL},
+  {(char* ) "-eulerpower", (char *) ".euler2d.eulerpower",
+   XrmoptionSepArg, (caddr_t) NULL},
 };
 static argtype vars[] =
 {
   {(caddr_t *) &tail_len, (char *) "eulertail",
    (char *) "EulerTail", (char *) DEF_EULERTAIL, t_Int},
+  {(caddr_t *) &power, (char *) "eulerpower",
+   (char *) "EulerPower", (char *) "1", t_Float},
 };
 static OptionStruct desc[] =
 {
   {(char *) "-eulertail len", (char *) "Length of Euler2d tails"},
+  {(char *) "-eulerpower power", (char *) "power of interaction law for points for Euler2d"},
 };
 
 ModeSpecOpt euler2d_opts =
@@ -90,19 +104,12 @@ ModStruct   euler2d_description = {
 #define positive_rand(v)	(LRAND()/MAXRAND*(v))	/* positive random */
 
 #define number_of_vortex_points 20
-#define delta_t 0.001
-
-#define DO_SUBTLE_PERTURB
-/*
-  This feature, if switched on, reduces (but does not remove) problems
-  where particles near the edge of the boundary bounce around.  The
-  way it works is to map points x in the unit disk to points y
-  in the plane, where y = f(|x|) x, with f(t) = -log(1-t)/t.
-*/
 
 #define n_bound_p 500
 #define deg_p 6
 
+static double delta_t;
+
 typedef struct {
 	int        width;
 	int        height;
@@ -113,28 +120,36 @@ typedef struct {
         int        N;
         int        Nvortex;
 
-/*
-x[2n+0] = x coord for nth point
-x[2n+1] = y coord for nth point
-w[n] = vorticity at nth point
+/*  x[2i+0] = x coord for nth point
+    x[2i+1] = y coord for nth point
+    w[i] = vorticity at nth point
 */
-
 	double     *x;
 	double     *w;
 
 	double     *diffx;
+	double     *olddiffx;
+	double     *tempx;
+	double     *tempdiffx;
+/*  (xs[2i+0],xs[2i+1]) is reflection of (x[2i+0],x[2i+1]) about unit circle
+    xs[2i+0] = x[2i+0]/nx
+    xs[2i+1] = x[2i+1]/nx
+    where
+    nx = x[2i+0]*x[2i+0] + x[2i+1]*x[2i+1]
 
-	double     *olddiffs[3];
-	int        olddiffindex[3];
-#define olddiff(i) sp->olddiffs[sp->olddiffindex[i]]
-	double     *k1, *k2, *k3, *k4;
-        double     *tempx;
-#ifdef DO_SUBTLE_PERTURB
-	double     *tempk;
-#endif
+    x_is_zero[i] = (nx < 1e-10)
+*/
+	double     *xs;
+	short      *x_is_zero;
 
-/* sometimes in our calculations we get nan.  If that
-   happens with the point x[2*i+0], x[2*i+1], we set dead[i].
+/*  (p[2i+0],p[2i+1]) is image of (x[2i+0],x[2i+1]) under polynomial p.
+    mod_dp2 is |p'(z)|^2 when z = (x[2i+0],x[2i+1]).
+*/
+	double	   *p;
+	double	   *mod_dp2;
+
+/* Sometimes in our calculations we get overflow or numbers that are too big.  
+   If that happens with the point x[2*i+0], x[2*i+1], we set dead[i].
 */
 	short      *dead;
 
@@ -154,57 +169,6 @@ w[n] = vorticity at nth point
 
 static euler2dstruct *euler2ds = NULL;
 
-/*
-  greens_fun calculates the effect of a vortex point at (a1,a2)
-  at the point (z1,z2), returning the vector field in
-  (u1,u2).
-
-  In the plane, this is given by the formula
-
-  z = x-a/|x-a|^2  or zero if z=a.
-
-  However, in the unit disk we have to subtract from the
-  above:
-
-  x-as/|x-as|^2
-
-  where as = a/|a|^2 is the reflection of a about the unit circle.
-*/
-
-static void
-greens_fun(double z1, double z2, double a1, double a2, double *u1, double *u2, 
-           short *dead)
-{
-  double na,za1,za2,nza,as1,as2,zas1=0.0,zas2=0.0,nzas=0.0;
-  int azero, self;
-
-  na = a1*a1+a2*a2;
-  azero = na < 1e-10;
-  za1 = z1 - a1;
-  za2 = z2 - a2;
-  nza = za1*za1+za2*za2;
-  self = nza < 1e-4;
-  if (!azero)
-  {
-/* as is the reflection of a in the unit sphere */
-    as1 = a1/na;
-    as2 = a2/na;
-    zas1 = z1 - as1;
-    zas2 = z2 - as2;
-    nzas = zas1*zas1+zas2*zas2;
-    if (nzas < 1e-5)
-      *dead = 1;
-  }
-
-  if (!*dead)
-  {
-    *u1 = - (self?0.0:(-za2/nza)) - (azero?0.0:(zas2/nzas));
-    *u2 =   (self?0.0:(-za1/nza)) + (azero?0.0:(zas1/nzas));
-  }
-  else
-    *u1 = *u2 = 0.0;
-}
-
 /*
   If variable_boundary == 1, then we make a variable boundary.
   The way this is done is to map the unit disk under a 
@@ -240,104 +204,213 @@ static double
 calc_mod_dp2(double z1, double z2, double p_coef[])
 {
   int i;
-  double temp,p1,p2;
+  double temp,mp1,mp2;
 
-  p1=0;
-  p2=0;
+  mp1=0;
+  mp2=0;
   for(i=deg_p;i>=2;i--)
   {
-    add(p1,p2,i*p_coef[(i-2)*2],i*p_coef[(i-2)*2+1]);
+    add(mp1,mp2,i*p_coef[(i-2)*2],i*p_coef[(i-2)*2+1]);
+    mult(mp1,mp2,z1,z2);
+  }
+  add(mp1,mp2,1,0);
+  return mp1*mp1+mp2*mp2;
+}
+
+static void
+calc_all_p(euler2dstruct *sp)
+{
+  int i,j;
+  double temp,p1,p2,z1,z2;
+  for(j=(sp->hide_vortex?sp->Nvortex:0);j<sp->N;j++) if(!sp->dead[j])
+  {
+    p1=0;
+    p2=0;
+    z1=sp->x[2*j+0];
+    z2=sp->x[2*j+1];
+    for(i=deg_p;i>=2;i--)
+    {
+      add(p1,p2,sp->p_coef[(i-2)*2],sp->p_coef[(i-2)*2+1]);
+      mult(p1,p2,z1,z2);
+    }
+    add(p1,p2,1,0);
     mult(p1,p2,z1,z2);
+    sp->p[2*j+0] = p1;
+    sp->p[2*j+1] = p2;
+  }
+}
+
+static void
+calc_all_mod_dp2(double *x, euler2dstruct *sp)
+{
+  int i,j;
+  double temp,mp1,mp2,z1,z2;
+  for(j=0;j<sp->N;j++) if(!sp->dead[j])
+  {
+    mp1=0;
+    mp2=0;
+    z1=x[2*j+0];
+    z2=x[2*j+1];
+    for(i=deg_p;i>=2;i--)
+    {
+      add(mp1,mp2,i*sp->p_coef[(i-2)*2],i*sp->p_coef[(i-2)*2+1]);
+      mult(mp1,mp2,z1,z2);
+    }
+    add(mp1,mp2,1,0);
+    sp->mod_dp2[j] = mp1*mp1+mp2*mp2;
   }
-  add(p1,p2,1,0);
-  return p1*p1+p2*p2;
 }
 
 static void
 derivs(double *x, euler2dstruct *sp)
 {
   int i,j;
-  double u1,u2;
-  double mod_dp2;
+  double u1,u2,x1,x2,xij1,xij2,nxij;
+  double nx;
 
-  memset(sp->diffx,0,sizeof(double)*2*sp->N);
+  if (variable_boundary)
+    calc_all_mod_dp2(sp->x,sp);
 
-  for (i=0;i<sp->N;i++) if (!sp->dead[i])
+  for (j=0;j<sp->Nvortex;j++) if (!sp->dead[j])
   {
-    for (j=0;j<sp->Nvortex;j++) if (!sp->dead[j])
-    {
-      greens_fun(x[2*i+0],x[2*i+1],x[2*j+0],x[2*j+1],&u1,&u2,sp->dead+i);
-      if (!sp->dead[i])
-      {
-        sp->diffx[2*i+0] += u1*sp->w[j];
-        sp->diffx[2*i+1] += u2*sp->w[j];
-      }
-#if DEBUG_FLOATING_OVERFLOW
-      else
-        printf("dead greens_fun %d\n",i);fflush(stdout);
-#endif
-    }
-    if (!sp->dead[i] && variable_boundary)
-    {
-      mod_dp2=calc_mod_dp2(x[2*i+0],x[2*i+1],sp->p_coef);
-      if (mod_dp2 < 1e-5)
-      {
-#if DEBUG_FLOATING_OVERFLOW
-        printf("dead mod_dp2 %d\n",i);fflush(stdout);
-#endif
-        sp->dead[i] = 1;
-      }
-      else
-      {
-        sp->diffx[2*i+0] /= mod_dp2;
-        sp->diffx[2*i+1] /= mod_dp2;
-      }
+    nx = x[2*j+0]*x[2*j+0] + x[2*j+1]*x[2*j+1];
+    if (nx < 1e-10)
+      sp->x_is_zero[j] = 1;
+    else {
+      sp->x_is_zero[j] = 0;
+      sp->xs[2*j+0] = x[2*j+0]/nx;
+      sp->xs[2*j+1] = x[2*j+1]/nx;
     }
   }
-}
 
-#ifdef DO_SUBTLE_PERTURB
-/* This is described above */
-
-static void
-subtle_perturb(double ret[], double x[], double k[], euler2dstruct *sp)
-{
-  int i;
-  double t1,t2,mag,mag2,mlog1mmag,memmagdmag,xdotk;
-  double d1,d2,x1,x2;
+  (void) memset(sp->diffx,0,sizeof(double)*2*sp->N);
 
   for (i=0;i<sp->N;i++) if (!sp->dead[i])
   {
     x1 = x[2*i+0];
     x2 = x[2*i+1];
+    for (j=0;j<sp->Nvortex;j++) if (!sp->dead[j])
+    {
+/*
+  Calculate the Biot-Savart kernel, that is, effect of a 
+  vortex point at a = (x[2*j+0],x[2*j+1]) at the point 
+  x = (x1,x2), returning the vector field in (u1,u2).
+
+  In the plane, this is given by the formula
+
+  u = (x-a)/|x-a|^2  or zero if x=a.
+
+  However, in the unit disk we have to subtract from the
+  above:
+
+  (x-as)/|x-as|^2
+
+  where as = a/|a|^2 is the reflection of a about the unit circle.
+
+  If however power != 1, then
+
+  u = (x-a)/|x-a|^(power+1)  -  |a|^(1-power) (x-as)/|x-as|^(power+1)
+
+*/
+
+      xij1 = x1 - x[2*j+0];
+      xij2 = x2 - x[2*j+1];
+      nxij = (power==1.0) ? xij1*xij1+xij2*xij2 : pow(xij1*xij1+xij2*xij2,(power+1)/2.0);
+
+      if(nxij >= 1e-4)  {
+        u1 =  xij2/nxij;
+        u2 = -xij1/nxij;
+      }
+      else
+        u1 = u2 = 0.0;
+
+      if (!sp->x_is_zero[j])
+      {
+        xij1 = x1 - sp->xs[2*j+0];
+        xij2 = x2 - sp->xs[2*j+1];
+        nxij = (power==1.0) ? xij1*xij1+xij2*xij2 : pow(xij1*xij1+xij2*xij2,(power+1)/2.0);
+
+        if (nxij < 1e-5)
+        {
+          sp->dead[i] = 1;
+          u1 = u2 = 0.0;
+        }
+        else
+        {
+          u1 -= xij2/nxij;
+          u2 += xij1/nxij;
+        }
+      }
+
+      if (!sp->dead[i])
+      {
+        sp->diffx[2*i+0] += u1*sp->w[j];
+        sp->diffx[2*i+1] += u2*sp->w[j];
+      }
+    }
+
+    if (!sp->dead[i] && variable_boundary)
+    {
+      if (sp->mod_dp2[i] < 1e-5)
+        sp->dead[i] = 1;
+      else
+      {
+        sp->diffx[2*i+0] /= sp->mod_dp2[i];
+        sp->diffx[2*i+1] /= sp->mod_dp2[i];
+      }
+    }
+  }
+}
+
+/*
+  What perturb does is effectively
+    ret = x + k,
+  where k should be of order delta_t.  
+
+  We have the option to do this more subtly by mapping points x 
+  in the unit disk to points y in the plane, where y = f(|x|) x, 
+  with f(t) = -log(1-t)/t.
+
+  This might reduce (but does not remove) problems where particles near 
+  the edge of the boundary bounce around.
+
+  But it seems to be not that effective, so for now switch it off.
+*/
+
+#define SUBTLE_PERTURB 0
+
+static void
+perturb(double ret[], double x[], double k[], euler2dstruct *sp)
+{
+  int i;
+  double x1,x2,k1,k2;
+
+#if SUBTLE_PERTURB
+  double d1,d2,t1,t2,mag,mag2,mlog1mmag,memmagdmag,xdotk;
+  for (i=0;i<sp->N;i++) if (!sp->dead[i])
+  {
+    x1 = x[2*i+0];
+    x2 = x[2*i+1];
+    k1 = k[2*i+0];
+    k2 = k[2*i+1];
     mag2 = x1*x1 + x2*x2;
     if (mag2 < 1e-10)
     {
-      ret[2*i+0] = x1+k[2*i+0];
-      ret[2*i+1] = x2+k[2*i+1];
+      ret[2*i+0] = x1+k1;
+      ret[2*i+1] = x2+k2;
     }
     else if (mag2 > 1-1e-5)
-    {
-#if DEBUG_FLOATING_OVERFLOW
-      printf("dead mag2 %d\n",i);fflush(stdout);
-#endif
       sp->dead[i] = 1;
-    }
     else
     {
       mag = sqrt(mag2);
       mlog1mmag = -log(1-mag);
-      xdotk = x1*k[2*i+0] + x2*k[2*i+1];
-      t1 = (x1 + k[2*i+0])*mlog1mmag/mag + x1*xdotk*(1.0/(1-mag)-mlog1mmag/mag)/mag/mag;
-      t2 = (x2 + k[2*i+1])*mlog1mmag/mag + x2*xdotk*(1.0/(1-mag)-mlog1mmag/mag)/mag/mag;
+      xdotk = x1*k1 + x2*k2;
+      t1 = (x1 + k1)*mlog1mmag/mag + x1*xdotk*(1.0/(1-mag)-mlog1mmag/mag)/mag/mag;
+      t2 = (x2 + k2)*mlog1mmag/mag + x2*xdotk*(1.0/(1-mag)-mlog1mmag/mag)/mag/mag;
       mag = sqrt(t1*t1+t2*t2);
       if (mag > 11.5 /* log(1e5) */)
-      {
-#if DEBUG_FLOATING_OVERFLOW
-        printf("dead exp %d %g\n",i,mag);fflush(stdout);
-#endif
         sp->dead[i] = 1;
-      }
       else
       {
         memmagdmag = (mag>1e-5) ? ((1.0-exp(-mag))/mag) : (1-mag/2.0);
@@ -345,119 +418,76 @@ subtle_perturb(double ret[], double x[], double k[], euler2dstruct *sp)
         ret[2*i+1] = t2*memmagdmag;
       }
     }
-#if DEBUG_FLOATING_OVERFLOW
-    if (!sp->dead[i] && (isnan(ret[2*i+0]) || isnan(ret[2*i+1])))
-    {
-      printf("dead isnan %d\n",i);fflush(stdout);
-      sp->dead[i] = 1;
-    }
-    else
-#endif
     if (!sp->dead[i])
     {
       d1 = ret[2*i+0]-x1;
       d2 = ret[2*i+1]-x2;
       if (d1*d1+d2*d2 > 0.1)
-      {
         sp->dead[i] = 1;
-#if DEBUG_FLOATING_OVERFLOW
-        printf("dead big %d\n",i);fflush(stdout);
-#endif
-      }
     }
   }
-}
-#endif
 
-static void
-runge_kutta_4(euler2dstruct *sp)
-{
-  int i;
-
-#ifdef DO_SUBTLE_PERTURB
-  derivs(sp->x,sp);
-  for (i=0;i<2*sp->N;i++) sp->k1[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempk[i] = sp->k1[i]/2;
-  subtle_perturb(sp->tempx,sp->x,sp->tempk,sp);
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k2[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempk[i] = sp->k2[i]/2;
-  subtle_perturb(sp->tempx,sp->x,sp->tempk,sp);
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k3[i] = delta_t*sp->diffx[i];
-  subtle_perturb(sp->tempx,sp->x,sp->k3,sp);
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k4[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempk[i] = (sp->k1[i]+2*sp->k2[i]+2*sp->k3[i]+sp->k4[i])/6;
-  subtle_perturb(sp->x,sp->x,sp->k3,sp);
 #else
-  derivs(sp->x,sp);
-  for (i=0;i<2*sp->N;i++) sp->k1[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempx[i] = sp->x[i] + sp->k1[i]/2;
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k2[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempx[i] = sp->x[i] + sp->k2[i]/2;
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k3[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->tempx[i] = sp->x[i] + sp->k2[i];
-  derivs(sp->tempx,sp);
-  for (i=0;i<2*sp->N;i++) sp->k4[i] = delta_t*sp->diffx[i];
-  for (i=0;i<2*sp->N;i++) sp->x[i] += (sp->k1[i]+2*sp->k2[i]+2*sp->k3[i]+sp->k4[i])/6;
-#endif
-}
 
-static void
-pushdiff(euler2dstruct *sp)
-{
-  int i,t;
-
-  t = sp->olddiffindex[2];
-  for (i=1;i>=0;i--)
-    sp->olddiffindex[i+1] = sp->olddiffindex[i];
-  sp->olddiffindex[0] = t;
-  memcpy(olddiff(0),sp->diffx,sizeof(double)*2*sp->N);
-}
-
-static void
-error_predictor(euler2dstruct *sp)
-{
-  int i;
-
-#ifdef DO_SUBTLE_PERTURB
-  for (i=0;i<2*sp->N;i++)
-    sp->tempk[i] = delta_t/12 * (23*olddiff(0)[i]-16*olddiff(1)[i]+5*olddiff(2)[i]);
-  subtle_perturb(sp->x,sp->x,sp->tempk,sp);
-  derivs(sp->x,sp);
-  for (i=0;i<2*sp->N;i++)
-    sp->tempk[i] = delta_t/24 * (9*sp->diffx[i]+19*olddiff(0)[i]-5*olddiff(1)[i]+olddiff(2)[i]);
-  subtle_perturb(sp->x,sp->x,sp->tempk,sp);
-#else
-  for (i=0;i<2*sp->N;i++)
-    sp->x[i] += delta_t/12 * (23*olddiff(0)[i]-16*olddiff(1)[i]+5*olddiff(2)[i]);
-  derivs(sp->x,sp);
-  for (i=0;i<2*sp->N;i++)
-    sp->x[i] += delta_t/24 * (9*sp->diffx[i]+19*olddiff(0)[i]-5*olddiff(1)[i]+olddiff(2)[i]);
+  for (i=0;i<sp->N;i++) if (!sp->dead[i])
+  {
+    x1 = x[2*i+0];
+    x2 = x[2*i+1];
+    k1 = k[2*i+0];
+    k2 = k[2*i+1];
+    if (k1*k1+k2*k2 > 0.1 || x1*x1+x2*x2 > 1-1e-5)
+      sp->dead[i] = 1;
+    else
+    {
+      ret[2*i+0] = x1+k1;
+      ret[2*i+1] = x2+k2;
+    }
+  }
 #endif
 }
 
 static void
 ode_solve(euler2dstruct *sp)
 {
-  if (sp->count <= 3)
-    runge_kutta_4(sp);
-  else
-    error_predictor(sp);
-  derivs(sp->x,sp);
-  pushdiff(sp);
+  int i;
+  double *temp;
+
+  if (sp->count < 1) {
+    /* midpoint method */
+    derivs(sp->x,sp);
+    (void) memcpy(sp->olddiffx,sp->diffx,sizeof(double)*2*sp->N);
+    for (i=0;i<sp->N;i++) if (!sp->dead[i]) {
+      sp->tempdiffx[2*i+0] = 0.5*delta_t*sp->diffx[2*i+0];
+      sp->tempdiffx[2*i+1] = 0.5*delta_t*sp->diffx[2*i+1];
+    }
+    perturb(sp->tempx,sp->x,sp->tempdiffx,sp);
+    derivs(sp->tempx,sp);
+    for (i=0;i<sp->N;i++) if (!sp->dead[i]) {
+      sp->tempdiffx[2*i+0] = delta_t*sp->diffx[2*i+0];
+      sp->tempdiffx[2*i+1] = delta_t*sp->diffx[2*i+1];
+    }
+    perturb(sp->x,sp->x,sp->tempdiffx,sp);
+  } else {
+    /* Adams Basforth */
+    derivs(sp->x,sp);
+    for (i=0;i<sp->N;i++) if (!sp->dead[i]) {
+      sp->tempdiffx[2*i+0] = delta_t*(1.5*sp->diffx[2*i+0] - 0.5*sp->olddiffx[2*i+0]);
+      sp->tempdiffx[2*i+1] = delta_t*(1.5*sp->diffx[2*i+1] - 0.5*sp->olddiffx[2*i+1]);
+    }
+    perturb(sp->x,sp->x,sp->tempdiffx,sp);
+    temp = sp->olddiffx;
+    sp->olddiffx = sp->diffx;
+    sp->diffx = temp;
+  }
 }
 
 #define deallocate(p) if (p != NULL) {(void) free((void *) p); p = NULL; }
+#define allocate(p,t,s) if ((p=(t*)malloc(sizeof(t)*s))==NULL)\
+{free_euler2d(sp);return;}
 
 static void
-deallocateall(euler2dstruct *sp)
+free_euler2d(euler2dstruct *sp)
 {
-	int i;
-
 	deallocate(sp->csegs);
 	deallocate(sp->old_segs);
 	deallocate(sp->nold_segs);
@@ -465,29 +495,33 @@ deallocateall(euler2dstruct *sp)
 	deallocate(sp->x);
 	deallocate(sp->diffx);
 	deallocate(sp->w);
-	for (i=0;i<3;i++) {
-		deallocate(sp->olddiffs[i]);
-	}
-	deallocate(sp->k1);
-	deallocate(sp->k2);
-	deallocate(sp->k3);
-	deallocate(sp->k4);
-#ifdef DO_SUBTLE_PERTURB
-	deallocate(sp->tempk);
-#endif
+	deallocate(sp->olddiffx);
+	deallocate(sp->tempdiffx);
 	deallocate(sp->tempx);
 	deallocate(sp->dead);
 	deallocate(sp->boundary);
+	deallocate(sp->xs);
+	deallocate(sp->x_is_zero);
+	deallocate(sp->p);
+	deallocate(sp->mod_dp2);
 }
 
 void
 init_euler2d(ModeInfo * mi)
 {
+#define nr_rotates 18 /* how many rotations to try to fill as much of screen as possible - must be even number */
 	euler2dstruct *sp;
-	static int  allocated = 0;
 	int         i,k,n,np;
 	double      r,theta,x,y,w;
-	double      mag,xlow,xhigh,ylow,yhigh,xscale,yscale,p1,p2;
+	double      mag,xscale,yscale,p1,p2;
+	double      low[nr_rotates],high[nr_rotates],pp1,pp2,pn1,pn2,angle1,angle2,tempangle,dist,scale,bestscale,temp;
+	int	    besti = 0;
+
+        if (power<0.5) power = 0.5;
+        if (power>3.0) power = 3.0;
+	variable_boundary &= power == 1.0;
+	delta_t = 0.001;
+        if (power>1.0) delta_t *= pow(0.1,power-1);
 
 	if (euler2ds == NULL) {
 		if ((euler2ds = (euler2dstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -517,42 +551,33 @@ init_euler2d(ModeInfo * mi)
 	/* Clear the background. */
 	MI_CLEARWINDOW(mi);
 	 
-	if(!allocated || sp->N != allocated){ /* reallocate */
-	  deallocateall(sp);
-	}
+	free_euler2d(sp);
 
 	/* Allocate memory. */
 
-	if (!sp->csegs) {
-		sp->csegs = (XSegment *) malloc(sizeof (XSegment) * sp->N );
-		sp->old_segs = (XSegment *) malloc(sizeof (XSegment) * sp->N * tail_len);
-		sp->nold_segs = (int *) malloc(sizeof(int) * tail_len);
-		sp->lastx = (short *) malloc(sizeof (short) * sp->N * 2);
-		sp->x = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->diffx = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->w = (double *) malloc(sizeof (double) * sp->Nvortex);
-		for (i=0;i<3;i++) {
-			sp->olddiffs[i] = (double *) malloc(sizeof (double) * sp->N * 2);
-		}
-		sp->k1 = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->k2 = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->k3 = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->k4 = (double *) malloc(sizeof (double) * sp->N * 2);
-#ifdef DO_SUBTLE_PERTURB
-		sp->tempk = (double *) malloc(sizeof (double) * sp->N * 2);
-#endif
-		sp->tempx = (double *) malloc(sizeof (double) * sp->N * 2);
-		sp->dead = (short *) malloc(sizeof (short) * sp->N);
-		sp->boundary = (XSegment *) malloc(sizeof (double) * n_bound_p);
+	if (sp->csegs == NULL) {
+		allocate(sp->csegs, XSegment, sp->N);
+		allocate(sp->old_segs, XSegment, sp->N * tail_len);
+		allocate(sp->nold_segs, int, tail_len);
+		allocate(sp->lastx, short, sp->N * 2);
+		allocate(sp->x, double, sp->N * 2);
+		allocate(sp->diffx, double, sp->N * 2);
+		allocate(sp->w, double, sp->Nvortex);
+		allocate(sp->olddiffx, double, sp->N * 2);
+		allocate(sp->tempdiffx, double, sp->N * 2);
+		allocate(sp->tempx, double, sp->N * 2);
+		allocate(sp->dead, short, sp->N);
+		allocate(sp->boundary, XSegment, n_bound_p);
+		allocate(sp->xs, double, sp->Nvortex * 2);
+		allocate(sp->x_is_zero, short, sp->Nvortex);
+		allocate(sp->p, double, sp->N * 2);
+		allocate(sp->mod_dp2, double, sp->N);
 	}
 	for (i=0;i<tail_len;i++) {
 		sp->nold_segs[i] = 0;
 	}
 	sp->c_old_seg = 0;
-	for (i=0;i<3;i++) {
-		sp->olddiffindex[i]=i;
-	}
-	memset(sp->dead,0,sp->N*sizeof(short));
+	(void) memset(sp->dead,0,sp->N*sizeof(short));
 
 	if (variable_boundary)
 	{
@@ -586,27 +611,75 @@ init_euler2d(ModeInfo * mi)
 		sp->p_coef[2*(6-2)+0] = 1.0/6.0;
 #endif
 
-	/* Initialize boundary and scaling factors */
-		xlow = 1e5;
-		xhigh = -1e5;
-		ylow = 1e5;
-		yhigh = -1e5;
-		for(k=0;k<n_bound_p;k++)
-		{
-			calc_p(&p1,&p2,cos((double)k/(n_bound_p)*2*M_PI),sin((double)k/(n_bound_p)*2*M_PI),sp->p_coef);
-			if (p1<xlow) xlow = p1;
-			if (p1>xhigh) xhigh = p1;
-			if (p2<ylow) ylow = p2;
-			if (p2>yhigh) yhigh = p2;
+
+/* Here we figure out the best rotation of the domain so that it fills as
+   much of the screen as possible.  The number of angles we look at is determined
+   by nr_rotates (we look every 180/nr_rotates degrees).  
+   While we figure out the best angle to rotate, we also figure out the correct scaling factors.
+*/
+
+		for(k=0;k<nr_rotates;k++) {
+			low[k] = 1e5;
+			high[k] = -1e5;
 		}
-		xscale = (sp->width-5.0)/(xhigh-xlow);
-		yscale = (sp->height-5.0)/(yhigh-ylow);
-		if (xscale > yscale)
-			sp->scale = yscale;
+
+		for(k=0;k<n_bound_p;k++) {
+			calc_p(&p1,&p2,cos((double)k/(n_bound_p)*2*M_PI),sin((double)k/(n_bound_p)*2*M_PI),sp->p_coef);
+			calc_p(&pp1,&pp2,cos((double)(k-1)/(n_bound_p)*2*M_PI),sin((double)(k-1)/(n_bound_p)*2*M_PI),sp->p_coef);
+			calc_p(&pn1,&pn2,cos((double)(k+1)/(n_bound_p)*2*M_PI),sin((double)(k+1)/(n_bound_p)*2*M_PI),sp->p_coef);
+			angle1 = nr_rotates/M_PI*atan2(p2-pp2,p1-pp1)-nr_rotates/2;
+			angle2 = nr_rotates/M_PI*atan2(pn2-p2,pn1-p1)-nr_rotates/2;
+			while (angle1<0) angle1+=nr_rotates*2;
+			while (angle2<0) angle2+=nr_rotates*2;
+			if (angle1>nr_rotates*1.75 && angle2<nr_rotates*0.25) angle2+=nr_rotates*2;
+			if (angle1<nr_rotates*0.25 && angle2>nr_rotates*1.75) angle1+=nr_rotates*2;
+                        if (angle2<angle1) {
+				tempangle=angle1;
+				angle1=angle2;
+				angle2=tempangle;
+			}
+			for(i=(int)floor(angle1);i<(int)ceil(angle2);i++) {
+				dist = cos((double)i*M_PI/nr_rotates)*p1 + sin((double)i*M_PI/nr_rotates)*p2;
+				if (i%(nr_rotates*2)<nr_rotates) {
+					if (dist>high[i%nr_rotates]) high[i%nr_rotates] = dist;
+					if (dist<low[i%nr_rotates]) low[i%nr_rotates] = dist;
+				}
+				else {
+					if (-dist>high[i%nr_rotates]) high[i%nr_rotates] = -dist;
+					if (-dist<low[i%nr_rotates]) low[i%nr_rotates] = -dist;
+				}
+			}
+		}
+		bestscale = 0;
+		for (i=0;i<nr_rotates;i++) {
+			xscale = (sp->width-5.0)/(high[i]-low[i]);
+			yscale = (sp->height-5.0)/(high[(i+nr_rotates/2)%nr_rotates]-low[(i+nr_rotates/2)%nr_rotates]);
+			scale = (xscale>yscale) ? yscale : xscale;
+			if (scale>bestscale) {
+				bestscale = scale;
+				besti = i;
+			}
+		}
+/* Here we do the rotation.  The way we do this is to replace the
+   polynomial p(z) by a^{-1} p(a z) where a = exp(i best_angle).
+*/
+		p1 = 1;
+		p2 = 0;
+		for(k=2;k<=deg_p;k++)
+		{
+			mult(p1,p2,cos((double)besti*M_PI/nr_rotates),sin((double)besti*M_PI/nr_rotates));
+			mult(sp->p_coef[2*(k-2)+0],sp->p_coef[2*(k-2)+1],p1,p2);
+		}
+
+		sp->scale = bestscale;
+		sp->xshift = -(low[besti]+high[besti])/2.0*sp->scale+sp->width/2;
+                if (besti<nr_rotates/2)
+			sp->yshift = -(low[besti+nr_rotates/2]+high[besti+nr_rotates/2])/2.0*sp->scale+sp->height/2;
 		else
-			sp->scale = xscale;
-		sp->xshift = -(xlow+xhigh)/2.0*sp->scale+sp->width/2;
-		sp->yshift = -(ylow+yhigh)/2.0*sp->scale+sp->height/2;
+			sp->yshift = (low[besti-nr_rotates/2]+high[besti-nr_rotates/2])/2.0*sp->scale+sp->height/2;
+
+
+/* Initialize boundary */
 
 		for(k=0;k<n_bound_p;k++)
 		{
@@ -679,13 +752,20 @@ draw_euler2d(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	euler2dstruct *sp = &euler2ds[MI_SCREEN(mi)];
 	int         b, col, n_non_vortex_segs;
-	double      p1,p2;
+	euler2dstruct *sp;
 
 	MI_IS_DRAWN(mi) = True;
 
+	if (euler2ds == NULL)
+		return;
+	sp = &euler2ds[MI_SCREEN(mi)];
+	if (sp->csegs == NULL)
+		return;
+
 	ode_solve(sp);
+	if (variable_boundary)
+		calc_all_p(sp);
 
 	sp->cnsegs = 0;
 	for(b=sp->Nvortex;b<sp->N;b++) if(!sp->dead[b])
@@ -694,9 +774,8 @@ draw_euler2d(ModeInfo * mi)
 		sp->csegs[sp->cnsegs].y1 = sp->lastx[2*b+1];
 		if (variable_boundary)
 		{
-			calc_p(&p1,&p2,sp->x[2*b+0],sp->x[2*b+1],sp->p_coef);
-			sp->csegs[sp->cnsegs].x2 = (short)(p1*sp->scale+sp->xshift);
-			sp->csegs[sp->cnsegs].y2 = (short)(p2*sp->scale+sp->yshift);
+			sp->csegs[sp->cnsegs].x2 = (short)(sp->p[2*b+0]*sp->scale+sp->xshift);
+			sp->csegs[sp->cnsegs].y2 = (short)(sp->p[2*b+1]*sp->scale+sp->yshift);
 		}
 		else
 		{
@@ -715,9 +794,8 @@ draw_euler2d(ModeInfo * mi)
 		sp->csegs[sp->cnsegs].y1 = sp->lastx[2*b+1];
 		if (variable_boundary)
 		{
-			calc_p(&p1,&p2,sp->x[2*b+0],sp->x[2*b+1],sp->p_coef);
-			sp->csegs[sp->cnsegs].x2 = (short)(p1*sp->scale+sp->xshift);
-			sp->csegs[sp->cnsegs].y2 = (short)(p2*sp->scale+sp->yshift);
+			sp->csegs[sp->cnsegs].x2 = (short)(sp->p[2*b+0]*sp->scale+sp->xshift);
+			sp->csegs[sp->cnsegs].y2 = (short)(sp->p[2*b+1]*sp->scale+sp->yshift);
 		}
 		else
 		{
@@ -765,7 +843,7 @@ draw_euler2d(ModeInfo * mi)
 				 (int) (2*sp->radius) + 2, (int) (2* sp->radius) + 2, 0, 64*360);
 
 		/* Copy to erase-list */
-		memcpy(sp->old_segs+sp->c_old_seg*sp->N, sp->csegs, sp->cnsegs*sizeof(XSegment));
+		(void) memcpy(sp->old_segs+sp->c_old_seg*sp->N, sp->csegs, sp->cnsegs*sizeof(XSegment));
 		sp->nold_segs[sp->c_old_seg] = sp->cnsegs;
 		sp->c_old_seg++;
 		if (sp->c_old_seg >= tail_len)
@@ -783,9 +861,8 @@ release_euler2d(ModeInfo * mi)
 	if (euler2ds != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			deallocateall(&euler2ds[screen]);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_euler2d(&euler2ds[screen]);
 		(void) free((void *) euler2ds);
 		euler2ds = NULL;
 	}
