@@ -2,7 +2,7 @@
 /* forest --- binary trees in a fractal forest */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)forest.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)forest.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -24,7 +24,8 @@ static const char sccsid[] = "@(#)forest.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Compatible with xscreensaver
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Compatible with xscreensaver
  *
  */
 
@@ -169,12 +170,15 @@ draw_forest(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
-	foreststruct *fp = &forests[MI_SCREEN(mi)];
 	short       x, y, x_2, y_2, len, c = 0;
 	float       a, as;
+	foreststruct *fp;
+
+	if (forests == NULL)
+		return;
+	fp = &forests[MI_SCREEN(mi)];
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (fp->time < fp->ntrees) {
 
 		x = RANGE_RAND(0, fp->width);
