@@ -2,7 +2,7 @@
 /* galaxy --- spinning galaxies */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)galaxy.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)galaxy.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -26,17 +26,18 @@ static const char sccsid[] = "@(#)galaxy.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Compatible with xscreensaver
- * 18-Apr-97: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
- * 07-Apr-97: Modified by Dave Mitchell <davem@magnet.com>
- *            random star sizes
- *            colors change depending on velocity
- * 23-Oct-94: Modified by David Bagley <bagleyd@tux.org>
- * 10-Oct-94: Add colors by Hubert Feyer
- * 30-Sep-94: Initial port by Hubert Feyer
- * 09-Mar-94: VMS can generate a random number 0.0 which results in a
- *            division by zero, corrected by Jouk Jansen
- *            <joukj@hrem.stm.tudelft.nl>
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Compatible with xscreensaver
+ * 18-Apr-1997: Memory leak fixed by Tom Schmidt <tschmidt@micron.com>
+ * 07-Apr-1997: Modified by Dave Mitchell <davem@magnet.com>
+ *              random star sizes
+ *              colors change depending on velocity
+ * 23-Oct-1994: Modified by David Bagley <bagleyd@tux.org>
+ * 10-Oct-1994: Add colors by Hubert Feyer
+ * 30-Sep-1994: Initial port by Hubert Feyer
+ * 09-Mar-1994: VMS can generate a random number 0.0 which results in a
+ *              division by zero, corrected by Jouk Jansen
+ *              <joukj@hrem.stm.tudelft.nl>
  */
 
 #ifdef STANDALONE
@@ -203,7 +204,10 @@ startover(ModeInfo * mi)
 	else if (gp->ngalaxies < MINGALAXIES)
 		gp->ngalaxies = MINGALAXIES;
 	if (gp->galaxies == NULL)
-		gp->galaxies = (Galaxy *) calloc(gp->ngalaxies, sizeof (Galaxy));
+		if ((gp->galaxies = (Galaxy *) calloc(gp->ngalaxies,
+				sizeof (Galaxy))) == NULL) {
+			return;
+	}
 
 	for (i = 0; i < gp->ngalaxies; ++i) {
 		Galaxy     *gt = &gp->galaxies[i];
@@ -223,7 +227,11 @@ startover(ModeInfo * mi)
 			gt->stars = NULL;
 		}
 		gt->nstars = (NRAND(MAX_STARS / 2)) + MAX_STARS / 2;
-		gt->stars = (Star *) malloc(gt->nstars * sizeof (Star));
+		if ((gt->stars = (Star *) malloc(gt->nstars *
+				sizeof (Star))) == NULL) {
+			free_galaxies(gp);
+			return;
+		}
 		w1 = 2.0 * M_PI * FLOATRAND;
 		w2 = 2.0 * M_PI * FLOATRAND;
 		sinw1 = SINF(w1);
@@ -341,12 +349,17 @@ draw_galaxy(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	unistruct  *gp = &universes[MI_SCREEN(mi)];
 	double      d;		/* tmp */
 	int         i, j, k;	/* more tmp */
+	unistruct  *gp;
+
+	if (universes == NULL)
+		return;
+	gp = &universes[MI_SCREEN(mi)];
+	if (gp->galaxies == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	for (i = 0; i < gp->ngalaxies; ++i) {
 		Galaxy     *gt = &gp->galaxies[i];
 
