@@ -210,7 +210,7 @@ Init(atlantisstruct * ap)
 	{0.4, 0.4, 0.4, 1.0};
 	static float lmodel_localviewer[] =
 	{0.0};
-	float       fblue = 0.0, fgreen;
+	float       fblue, fgreen;
 
 	glFrontFace(GL_CW);
 
@@ -339,8 +339,8 @@ init_atlantis(ModeInfo * mi)
 	}
 	ap->sharkspeed = MI_CYCLES(mi);		/* has influence on the "width"
 						   of the movement */
-	ap->sharksize = MI_SIZE(mi);	/* has influence on the "distance"
-					   of the sharks */
+	ap->sharksize = MI_SIZE(mi)?MI_SIZE(mi):SHARKSIZE;	/* has influence on the "distance"
+					                           of the sharks */
 	ap->whalespeed = whalespeed;
 	ap->wire = MI_IS_WIREFRAME(mi);
 
