@@ -2,7 +2,7 @@
 /* moebius --- Moebius Strip II, an Escher-like GL scene with ants. */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)moebius.c	4.08 97/01/04 xlockmore";
+static const char sccsid[] = "@(#)moebius.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -36,7 +36,7 @@ static const char sccsid[] = "@(#)moebius.c	4.08 97/01/04 xlockmore";
  * OpenGL at home.
  *
  * Since I'm not a native English speaker, my apologies for any grammatical
- * mistake.
+ * mistakes.
  *
  * My e-mail address is
  * m-vianna@usa.net
@@ -44,24 +44,24 @@ static const char sccsid[] = "@(#)moebius.c	4.08 97/01/04 xlockmore";
  * Marcelo F. Vianna (Jun-01-1997)
  *
  * Revision History:
- * 01-Jan-98: Mode separated from escher and renamed
- * 08-Jun-97: New scene implemented: "Impossible Cage" based in a M.C. Escher's
- *            painting with the same name (quite similar). The first GL mode
- *            to use texture mapping.
- *            The "Impossible Cage" scene doesn't use DEPTH BUFFER, the
- *            wood planks are drawn consistently using GL_CULL_FACE, and
- *            the painter's algorithm is used to sort the planks.
- *            Marcelo F. Vianna.
- * 07-Jun-97: Speed ups in Moebius Strip using GL_CULL_FACE.
- *            Marcelo F. Vianna.
- * 03-Jun-97: Initial Release (Only one scene: "Moebius Strip")
- *            The Moebius Strip scene was inspirated in a M.C. Escher's
- *            painting named Moebius Strip II in wich ants walk across a
- *            Moebius Strip path, sometimes meeting each other and sometimes
- *            being in "opposite faces" (note that the moebius strip has
- *            only one face and one edge).
- *            Marcelo F. Vianna.
- *
+ * 01-Nov-2000: Allocation checks
+ * 01-Jan-1998: Mode separated from escher and renamed
+ * 08-Jun-1997: New scene implemented: "Impossible Cage" based in a M.C.
+ *              Escher's painting with the same name (quite similar). The
+ *              first GL mode to use texture mapping.
+ *              The "Impossible Cage" scene doesn't use DEPTH BUFFER, the
+ *              wood planks are drawn consistently using GL_CULL_FACE, and
+ *              the painter's algorithm is used to sort the planks.
+ *              Marcelo F. Vianna.
+ * 07-Jun-1997: Speed ups in Moebius Strip using GL_CULL_FACE.
+ *              Marcelo F. Vianna.
+ * 03-Jun-1997: Initial Release (Only one scene: "Moebius Strip")
+ *              The Moebius Strip scene was inspirated in a M.C. Escher's
+ *              painting named Moebius Strip II in wich ants walk across a
+ *              Moebius Strip path, sometimes meeting each other and sometimes
+ *              being in "opposite faces" (note that the moebius strip has
+ *              only one face and one edge).
+ *              Marcelo F. Vianna.
  */
 
 /*-
@@ -151,7 +151,7 @@ typedef struct {
 	GLint       WindH, WindW;
 	GLfloat     step;
 	GLfloat     ant_position;
-	int         AreObjectsDefined[MaxObj];
+	Bool        AreObjectsDefined[MaxObj];
 	GLXContext *glx_context;
 } moebiusstruct;
 
@@ -198,29 +198,33 @@ static float MaterialGray8[] =
 static moebiusstruct *moebius = NULL;
 static GLuint objects = 0;
 
-static void
+static Bool
 mySphere(float radius)
 {
 	GLUquadricObj *quadObj;
 
-	quadObj = gluNewQuadric();
+	if ((quadObj = gluNewQuadric()) == 0)
+		return False;
 	gluQuadricDrawStyle(quadObj, (GLenum) GLU_FILL);
 	gluSphere(quadObj, radius, 16, 16);
 	gluDeleteQuadric(quadObj);
+	return True;
 }
 
-static void
+static Bool
 myCone(float radius)
 {
 	GLUquadricObj *quadObj;
 
-	quadObj = gluNewQuadric();
+	if ((quadObj = gluNewQuadric()) == 0)
+		return False;
 	gluQuadricDrawStyle(quadObj, (GLenum) GLU_FILL);
 	gluCylinder(quadObj, radius, 0, radius * 3, 8, 1);
 	gluDeleteQuadric(quadObj);
+	return True;
 }
 
-static void
+static Bool
 draw_moebius_ant(moebiusstruct * mp, float *Material, int mono)
 {
 	static float ant_step = 0;
@@ -237,31 +241,39 @@ draw_moebius_ant(moebiusstruct * mp, float *Material, int mono)
 		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Material);
 	if (!mp->AreObjectsDefined[ObjAntBody]) {
 		glNewList(objects + ObjAntBody, GL_COMPILE_AND_EXECUTE);
+		if (glGetError() != GL_NO_ERROR) {
+			return False;
+		}
 		glEnable(GL_CULL_FACE);
 		glPushMatrix();
 		glScalef(1, 1.3, 1);
-		mySphere(0.18);
+		if (!mySphere(0.18))
+			return False;
 		glScalef(1, 1 / 1.3, 1);
 		glTranslatef(0.00, 0.30, 0.00);
-		mySphere(0.2);
+		if (!mySphere(0.2))
+			return False;
 
 		glTranslatef(-0.05, 0.17, 0.05);
 		glRotatef(-90, 1, 0, 0);
 		glRotatef(-25, 0, 1, 0);
-		myCone(0.05);
+		if (!myCone(0.05))
+			return False;
 		glTranslatef(0.00, 0.10, 0.00);
-		myCone(0.05);
+		if (!myCone(0.05))
+			return False;
 		glRotatef(25, 0, 1, 0);
 		glRotatef(90, 1, 0, 0);
 
 		glScalef(1, 1.3, 1);
 		glTranslatef(0.15, -0.65, 0.05);
-		mySphere(0.25);
+		if (!mySphere(0.25))
+			return False;
 		glScalef(1, 1 / 1.3, 1);
 		glPopMatrix();
 		glDisable(GL_CULL_FACE);
 		glEndList();
-		mp->AreObjectsDefined[ObjAntBody] = 1;
+		mp->AreObjectsDefined[ObjAntBody] = True;
 #ifdef DEBUG_LISTS
 		(void) printf("Ant drawn SLOWLY\n");
 #endif
@@ -387,6 +399,7 @@ draw_moebius_ant(moebiusstruct * mp, float *Material, int mono)
 	glEnable(GL_LIGHTING);
 
 	ant_step += 0.3;
+	return True;
 }
 
 static void
@@ -412,7 +425,7 @@ RotateAaroundU(float Ax, float Ay, float Az,
 
 #define MoebiusDivisions 40
 #define MoebiusTransversals 4
-static void
+static Bool
 draw_moebius_strip(ModeInfo * mi)
 {
 	GLfloat     Phi, Theta;
@@ -425,7 +438,9 @@ draw_moebius_strip(ModeInfo * mi)
 
 	if (!mp->AreObjectsDefined[ObjMoebiusStrip]) {
 		glNewList(objects + ObjMoebiusStrip, GL_COMPILE_AND_EXECUTE);
-
+		if (glGetError() != GL_NO_ERROR) {
+			return False;
+		}
 		if (solidmoebius) {
 			glBegin(GL_QUAD_STRIP);
 			Phi = 0;
@@ -492,7 +507,7 @@ draw_moebius_strip(ModeInfo * mi)
 		}
 
 		glEndList();
-		mp->AreObjectsDefined[ObjMoebiusStrip] = 1;
+		mp->AreObjectsDefined[ObjMoebiusStrip] = True;
 #ifdef DEBUG_LISTS
 		(void) printf("Strip drawn SLOWLY\n");
 #endif
@@ -510,7 +525,8 @@ draw_moebius_strip(ModeInfo * mi)
 		glTranslatef(3, 0, 0);
 		glRotatef(mp->ant_position / 2 + 90, 0, 1, 0);
 		glTranslatef(0.28, 0, -0.45);
-		draw_moebius_ant(mp, MaterialYellow, mono);
+		if (!draw_moebius_ant(mp, MaterialYellow, mono))
+			return False;
 		glPopMatrix();
 
 		/* DRAW YELLOW ANT */
@@ -519,7 +535,8 @@ draw_moebius_strip(ModeInfo * mi)
 		glTranslatef(3, 0, 0);
 		glRotatef(mp->ant_position / 2, 0, 1, 0);
 		glTranslatef(0.28, 0, -0.45);
-		draw_moebius_ant(mp, MaterialBlue, mono);
+		if (!draw_moebius_ant(mp, MaterialBlue, mono))
+			return False;
 		glPopMatrix();
 
 		/* DRAW GREEN ANT */
@@ -529,7 +546,8 @@ draw_moebius_strip(ModeInfo * mi)
 		glRotatef(-mp->ant_position / 2, 0, 1, 0);
 		glTranslatef(0.28, 0, 0.45);
 		glRotatef(180, 1, 0, 0);
-		draw_moebius_ant(mp, MaterialGreen, mono);
+		if (!draw_moebius_ant(mp, MaterialGreen, mono))
+			return False;
 		glPopMatrix();
 
 		/* DRAW CYAN ANT */
@@ -539,10 +557,12 @@ draw_moebius_strip(ModeInfo * mi)
 		glRotatef(-mp->ant_position / 2 + 90, 0, 1, 0);
 		glTranslatef(0.28, 0, 0.45);
 		glRotatef(180, 1, 0, 0);
-		draw_moebius_ant(mp, MaterialCyan, mono);
+		if (!draw_moebius_ant(mp, MaterialCyan, mono))
+			return False;
 		glPopMatrix();
 	}
 	mp->ant_position += 1;
+	return True;
 }
 #undef MoebiusDivisions
 #undef MoebiusTransversals
@@ -551,24 +571,18 @@ static void
 reshape(ModeInfo * mi, int width, int height)
 {
 	moebiusstruct *mp = &moebius[MI_SCREEN(mi)];
+	int i;
 
 	glViewport(0, 0, mp->WindW = (GLint) width, mp->WindH = (GLint) height);
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 	glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0);
 	glMatrixMode(GL_MODELVIEW);
-	if (width >= 1024) {
-		glLineWidth(3);
-		glPointSize(3);
-	} else if (width >= 512) {
-		glLineWidth(2);
-		glPointSize(2);
-	} else {
-		glLineWidth(1);
-		glPointSize(1);
-	}
-	mp->AreObjectsDefined[ObjMoebiusStrip] = 0;
-	mp->AreObjectsDefined[ObjAntBody] = 0;
+	i = width / 512 + 1;
+	glLineWidth(i);
+	glPointSize(i);
+	mp->AreObjectsDefined[ObjMoebiusStrip] = False;
+	mp->AreObjectsDefined[ObjAntBody] = False;
 }
 
 static void
@@ -614,7 +628,6 @@ pinit(void)
 void
 init_moebius(ModeInfo * mi)
 {
-	int         screen = MI_SCREEN(mi);
 	moebiusstruct *mp;
 
 	if (moebius == NULL) {
@@ -622,16 +635,19 @@ init_moebius(ModeInfo * mi)
 					    sizeof (moebiusstruct))) == NULL)
 			return;
 	}
-	mp = &moebius[screen];
+	mp = &moebius[MI_SCREEN(mi)];
 	mp->step = NRAND(90);
 	mp->ant_position = NRAND(90);
 
 	if ((mp->glx_context = init_GL(mi)) != NULL) {
-
 		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
 		if (!glIsList(objects))
-			objects = glGenLists(MaxObj);
+			if ((objects = glGenLists(MaxObj)) == 0) {
+				MI_CLEARWINDOW(mi);
+				release_cage(mi);
+				return;
+			}
 		pinit();
 	} else {
 		MI_CLEARWINDOW(mi);
@@ -641,9 +657,13 @@ init_moebius(ModeInfo * mi)
 void
 draw_moebius(ModeInfo * mi)
 {
-	moebiusstruct *mp = &moebius[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	moebiusstruct *mp;
+
+	if (moebius == NULL)
+		return;
+	mp = &moebius[MI_SCREEN(mi)];
 
 	MI_IS_DRAWN(mi) = True;
 
@@ -668,7 +688,10 @@ draw_moebius(ModeInfo * mi)
 	glRotatef(mp->step * 100, 1, 0, 0);
 	glRotatef(mp->step * 95, 0, 1, 0);
 	glRotatef(mp->step * 90, 0, 0, 1);
-	draw_moebius_strip(mi);
+	if (!draw_moebius_strip(mi)) {
+		release_moebius(mi);
+		return;
+	}
 
 	glPopMatrix();
 
@@ -695,11 +718,17 @@ void
 release_moebius(ModeInfo * mi)
 {
 	if (moebius != NULL) {
-		moebiusstruct *mp = &moebius[MI_SCREEN(mi)];
+		int screen;
 
-		if (mp->glx_context) {
-			if (glIsList(objects))
-				glDeleteLists(objects, 3);
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
+			moebiusstruct *mp = &moebius[screen];
+
+			if (mp->glx_context) {
+				if (glIsList(objects)) {
+					glDeleteLists(objects, MaxObj);
+					objects = 0;
+				}
+			}
 		}
 		(void) free((void *) moebius);
 		moebius = NULL;
