@@ -2,7 +2,7 @@
 /* skewb --- Shows an auto-solving Skewb */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)skewb.c	4.17 2000/04/27 xlockmore";
+static const char sccsid[] = "@(#)skewb.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -31,8 +31,9 @@ static const char sccsid[] = "@(#)skewb.c	4.17 2000/04/27 xlockmore";
  * Based on rubik.c by Marcelo F. Vianna
  *
  * Revision History:
- * 27-Apr-2000: done?
- *
+ * 01-Nov-2000: Allocation checks
+ * 27-Apr-2000: Started writing, only have corners drawn and algorithm
+ *              compiled in.
  */
 
 /*-
@@ -537,7 +538,7 @@ typedef struct {
 	GLfloat     rotatestep;
 	GLfloat     PX, PY, VX, VY;
 	GLXContext *glx_context;
-	int         AreObjectsDefined[2];
+	Bool        AreObjectsDefined[2];
 } skewbstruct;
 
 static float front_shininess[] =
@@ -627,12 +628,15 @@ pickcolor(int C, int mono)
 	}
 }
 
-static void
+static Bool
 draw_stickerless_cubit(skewbstruct * sp)
 {
 	if (!sp->AreObjectsDefined[ObjCubit]) {
 		/* Draw one and rotate it */
 		glNewList(objects + ObjCubit, GL_COMPILE_AND_EXECUTE);
+		if (glGetError() != GL_NO_ERROR) {
+			return False;
+		}
 		glBegin(GL_QUADS);
 		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialGray);
 		/* Edge of cubit */
@@ -694,7 +698,7 @@ draw_stickerless_cubit(skewbstruct * sp)
 		glVertex3f(CUBEROUND, -CUBELEN, CUBEROUND);
 		glEnd();
 		glEndList();
-		sp->AreObjectsDefined[ObjCubit] = 1;
+		sp->AreObjectsDefined[ObjCubit] = True;
 #ifdef DEBUG_LISTS
 		(void) printf("Cubit drawn SLOWLY\n");
 #endif
@@ -704,14 +708,18 @@ draw_stickerless_cubit(skewbstruct * sp)
 		(void) printf("Cubit drawn quickly\n");
 #endif
 	}
+	return True;
 }
 
-static void
+static Bool
 draw_stickerless_facit(skewbstruct * sp)
 {
 	if (!sp->AreObjectsDefined[ObjFacit]) {
 		/* Draw one and rotate it */
 		glNewList(objects + ObjFacit, GL_COMPILE_AND_EXECUTE);
+		if (glGetError() != GL_NO_ERROR) {
+			return False;
+		}
 		glBegin(GL_QUADS);
 		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialPink);
 		/* Edge of facit */
@@ -759,7 +767,7 @@ draw_stickerless_facit(skewbstruct * sp)
 #endif
 		glEnd();
 		glEndList();
-		sp->AreObjectsDefined[ObjFacit] = 1;
+		sp->AreObjectsDefined[ObjFacit] = True;
 #ifdef DEBUG_LISTS
 		(void) printf("Facit drawn SLOWLY\n");
 #endif
@@ -769,6 +777,7 @@ draw_stickerless_facit(skewbstruct * sp)
 		(void) printf("Facit drawn quickly\n");
 #endif
 	}
+	return True;
 }
 
 static void
@@ -957,10 +966,12 @@ draw_facit(ModeInfo * mi,
 	}
 }
 
-static void
+static Bool
 draw_cube(ModeInfo * mi)
 {
 #define S1 1
+#define DRAW_STICKERLESS_FACIT(sp) if (!draw_stickerless_facit(sp)) return False
+#define DRAW_STICKERLESS_CUBIT(sp) if (!draw_stickerless_cubit(sp)) return False
 
 	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
 	SkewbLoc  slice;
@@ -996,30 +1007,30 @@ draw_cube(ModeInfo * mi)
 
 			glTranslatef(-0.5, -0.5, -0.5);
 			/* glTranslatef(S1, 0, S1); */
-			draw_stickerless_facit(sp);
+			DRAW_STICKERLESS_FACIT(sp);
 			glPushMatrix();
 			glRotatef(90.0, 0, 1, 0);
 			glRotatef(180.0, 1, 0, 0);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 0, 6, 2, 6, 4, 6);
 			glTranslatef(0, 0, S1);
 			glPushMatrix();
 			glRotatef(180.0, 0, 0, 1);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 6, 1, 2, 6, 4, 6); /* BL */
 			glTranslatef(S1, 0, -S1);
 			glPushMatrix();
 			glRotatef(90.0, 0, 1, 0);
 			glRotatef(90.0, 1, 0, 0);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 0, 6, 6, 3, 4, 6);
 			glTranslatef(0, 0, S1);
 			glPushMatrix();
 			glRotatef(90.0, 1, 0, 0);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 6, 1, 6, 3, 4, 6); /* BR */
 			glPopMatrix();
@@ -1028,27 +1039,28 @@ draw_cube(ModeInfo * mi)
 			glPushMatrix();
 			glRotatef(90.0, 0, 1, 0);
 			glRotatef(90.0, -1, 0, 0);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 0, 6, 2, 6, 6, 5);
 			glTranslatef(0, 0, S1);
 			glPushMatrix();
 			glRotatef(90.0, 0, 0, 1);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 6, 1, 2, 6, 6, 5); /* UL */
 			glTranslatef(S1, 0, -S1);
 			glPushMatrix();
 			glRotatef(90.0, 0, 1, 0);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			glPopMatrix();
 			draw_cubit(mi, 0, 6, 6, 3, 6, 5);
 			glTranslatef(0, 0, S1);
-			draw_stickerless_cubit(sp);
+			DRAW_STICKERLESS_CUBIT(sp);
 			draw_cubit(mi, 6, 1, 6, 3, 6, 5); /* UR */
 			glPopMatrix();
 			break;
 	}
+	return True;
 #undef S1
 }
 
@@ -1280,7 +1292,7 @@ compare_moves(skewbstruct * sp, SkewbMove move1, SkewbMove move2, Bool opp)
 	return False;
 }
 
-static void
+static Bool
 shuffle(ModeInfo * mi)
 {
 	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
@@ -1301,12 +1313,10 @@ shuffle(ModeInfo * mi)
 		sp->storedmoves = NRAND(-sp->storedmoves) + 1;
 	}
 	if ((sp->storedmoves) && (sp->moves == NULL))
-		if ((sp->moves =
-		     (SkewbMove *) calloc(sp->storedmoves + 1,
-				sizeof (SkewbMove))) == NULL)
-			(void) fprintf(stderr,
-			"Could not allocate memory for skewb move buffer\n");
-
+		if ((sp->moves = (SkewbMove *) calloc(sp->storedmoves + 1,
+				sizeof (SkewbMove))) == NULL) {
+			return False;
+		}
 	if (MI_CYCLES(mi) <= 1) {
 		sp->anglestep = 180.0;
 	} else {
@@ -1340,6 +1350,7 @@ shuffle(ModeInfo * mi)
 	sp->action = hideshuffling ? ACTION_SOLVE : ACTION_SHUFFLE;
 	sp->shufflingmoves = 0;
 	sp->done = 0;
+	return True;
 }
 
 static void
@@ -1353,10 +1364,11 @@ reshape(ModeInfo * mi, int width, int height)
 	glFrustum(-1.0, 1.0, -1.0, 1.0, 5.0, 15.0);
 	glMatrixMode(GL_MODELVIEW);
 
-	sp->AreObjectsDefined[ObjCubit] = 0;
+	sp->AreObjectsDefined[ObjFacit] = False;
+	sp->AreObjectsDefined[ObjCubit] = False;
 }
 
-static void
+static Bool
 pinit(ModeInfo * mi)
 {
 	glClearDepth(1.0);
@@ -1382,13 +1394,21 @@ pinit(ModeInfo * mi)
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_shininess);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_specular);
 
-	shuffle(mi);
+	return (shuffle(mi));
+}
+
+static void
+free_skewb(skewbstruct *sp)
+{
+	if (sp->moves != NULL) {
+		(void) free((void *) sp->moves);
+		sp->moves = NULL;
+	}
 }
 
 void
 init_skewb(ModeInfo * mi)
 {
-	int	 screen = MI_SCREEN(mi);
 	skewbstruct *sp;
 
 	if (skewb == NULL) {
@@ -1396,7 +1416,8 @@ init_skewb(ModeInfo * mi)
 					      sizeof (skewbstruct))) == NULL)
 			return;
 	}
-	sp = &skewb[screen];
+	sp = &skewb[MI_SCREEN(mi)];
+
 	sp->step = NRAND(180);
 	sp->PX = ((float) LRAND() / (float) MAXRAND) * 2.0 - 1.0;
 	sp->PY = ((float) LRAND() / (float) MAXRAND) * 2.0 - 1.0;
@@ -1406,8 +1427,18 @@ init_skewb(ModeInfo * mi)
 		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
 		if (!glIsList(objects))
-			objects = glGenLists(MaxObj);
-		pinit(mi);
+			if ((objects = glGenLists(MaxObj)) == 0) {
+				MI_CLEARWINDOW(mi);
+				release_skewb(mi);
+				return;
+			}
+		if (!pinit(mi)) {
+			free_skewb(sp);
+			if (MI_IS_VERBOSE(mi)) {
+				 (void) fprintf(stderr,
+					"Could not allocate memory for skewb\n");
+			}
+		}
 	} else {
 		MI_CLEARWINDOW(mi);
 	}
@@ -1417,13 +1448,17 @@ void
 draw_skewb(ModeInfo * mi)
 {
 	Bool bounced = False;
-
-	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	skewbstruct *sp;
+
+	if (skewb == NULL)
+		return;
+	sp = &skewb[MI_SCREEN(mi)];
+	if ((sp->storedmoves) && (sp->moves == NULL))
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (!sp->glx_context)
 		return;
 
@@ -1481,7 +1516,10 @@ draw_skewb(ModeInfo * mi)
 	glRotatef(sp->step * 100, 1, 0, 0);
 	glRotatef(sp->step * 95, 0, 1, 0);
 	glRotatef(sp->step * 90, 0, 0, 1);
-	draw_cube(mi);
+	if (!draw_cube(mi)) {
+		release_skewb(mi);
+		return;
+	}
 	glXSwapBuffers(display, window);
 
 #if 0
@@ -1517,7 +1555,13 @@ draw_skewb(ModeInfo * mi)
 	} else {
 		if (sp->done) {
 			if (++sp->rotatestep > DELAY_AFTER_SOLVING)
-				shuffle(mi);
+				if (!shuffle(mi)) {
+					free_skewb(sp);
+					if (MI_IS_VERBOSE(mi)) {
+						 (void) fprintf(stderr,
+							"Could not allocate memory for skewb\n");
+					}
+				}
 		} else {
 			if (sp->movement.face == NO_FACE) {
 				if (sp->storedmoves > 0) {
@@ -1554,11 +1598,21 @@ draw_skewb(ModeInfo * mi)
 void
 change_skewb(ModeInfo * mi)
 {
-	skewbstruct *sp = &skewb[MI_SCREEN(mi)];
+	skewbstruct *sp;
+
+	if (skewb == NULL)
+		return;
+	sp = &skewb[MI_SCREEN(mi)];
 
 	if (!sp->glx_context)
 		return;
-	pinit(mi);
+	if (!pinit(mi)) {
+		free_skewb(sp);
+		if (MI_IS_VERBOSE(mi)) {
+			 (void) fprintf(stderr,
+				"Could not allocate memory for skewb\n");
+		}
+	}
 }
 
 void
@@ -1570,12 +1624,12 @@ release_skewb(ModeInfo * mi)
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
 			skewbstruct *sp = &skewb[screen];
 
-			if (sp->moves != NULL)
-				(void) free((void *) sp->moves);
+			free_skewb(sp);
 			if (sp->glx_context) {
-				glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(sp->glx_context));
-				if (glIsList(objects))
-			                glDeleteLists(objects, 2);
+				if (glIsList(objects)) {
+			                glDeleteLists(objects, MaxObj);
+					objects = 0;
+				}
 			}
 		}
 		(void) free((void *) skewb);
