@@ -2,7 +2,7 @@
 /* superquadrics --- 3D mathematical shapes */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)superquadrics.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)superquadrics.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -31,19 +31,20 @@ static const char sccsid[] = "@(#)superquadrics.c	4.07 97/11/24 xlockmore";
  * quadric surface in a similar fashion.
  *
  * Revision History:
- * 30-Mar-97: Turned into a module for xlockmore 4.02 alpha.  The code
+ * 01-Nov-2000: Allocation checks
+ * 30-Mar-1997: Turned into a module for xlockmore 4.02 alpha.  The code
  *    is almost unrecognizable now from the first revision, except for
  *    a few remaining two-letter variable names.  I still don't have
  *    the normal vectors working right (I wrote the buggy normal vector
  *    code myself, can you tell?)
- * 07-Jan-97: A legend reborn; Superquadrics make an appearance as a
+ * 07-Jan-1997: A legend reborn; Superquadrics make an appearance as a
  *    real OpenGL program written in C.  I can even render them with
  *    proper lighting and specular highlights.  Gee, they look almost
  *    as good now as the original color plates of them that my uncle
  *    showed me as a child in 1981.  I don't know what computer hardware
  *    he was using at the time, but it's taken a couple decades for the
  *    PC clone hardware to catch up to it.
- * 05-Jan-97: After almost a decade, Superquadrics had almost faded away
+ * 05-Jan-1997: After almost a decade, Superquadrics had almost faded away
  *    into the myths and folklore of all the things my brother and I played
  *    with on computers when we were kids with too much time on our hands.
  *    I had since gotten involved in Unix, OpenGL, and other things.
@@ -713,8 +714,6 @@ init_superquadrics(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
-	int         screen = MI_SCREEN(mi);
-
 	superquadricsstruct *sp;
 
 	if (superquadrics == NULL) {
@@ -722,7 +721,8 @@ init_superquadrics(ModeInfo * mi)
 				      sizeof (superquadricsstruct))) == NULL)
 			return;
 	}
-	sp = &superquadrics[screen];
+	sp = &superquadrics[MI_SCREEN(mi)];
+
 	sp->mono = (MI_IS_MONO(mi) ? 1 : 0);
 
 	if ((sp->glx_context = init_GL(mi)) != NULL) {
@@ -742,12 +742,15 @@ init_superquadrics(ModeInfo * mi)
 void
 draw_superquadrics(ModeInfo * mi)
 {
-	superquadricsstruct *sp = &superquadrics[MI_SCREEN(mi)];
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
+	superquadricsstruct *sp;
+
+	if (superquadrics == NULL)
+		return;
+	sp = &superquadrics[MI_SCREEN(mi)];
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (!sp->glx_context)
 		return;
 
