@@ -2,7 +2,7 @@
 /* text3d --- Shows moving 3D texts */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)text3d.cc	2.2 99/10/28 xlockmore";
+static const char sccsid[] = "@(#)text3d.cc	5.0 2000/11/01 xlockmore";
 
 #endif
 
@@ -25,31 +25,32 @@ static const char sccsid[] = "@(#)text3d.cc	2.2 99/10/28 xlockmore";
  * Copyright (C) 1998 Stephane Rehel.
  *
  * See the gltt Official Site at http://home.worldnet.fr/~rehel/gltt/gltt.html
+ * May have better luck at http://perso.libertysurf.fr/lassauge/en_xlock.htm
  *
  * My e-mail address changed to lassauge@mail.dotcom.fr
  * Web site at http://perso.libertysurf.fr/lassauge/
  *
  * Eric Lassauge  (October-28-1999)
  *
- * REVISION HISTORY:
- *       99/10/28: fixes from Jouk "I play with every mode" Jansen.
- *                 Option ttanimate added.
- *       99/06/02: patches for initialization errors of GLTT library.
- *                 Thanks to Jouk Jansen and Scott <mcmillan@cambridge.com>.
- *                 text3d updates for fortunes thanks to Jouk Jansen
- *                 <joukj@hrem.stm.tudelft.nl>
- *                 Option no_split added.
- *
- *       98/08/23: add better handling of "faulty" fontfile and randomize
- *                 fontfile if '-ttfont' value is a directory.
- *                 Minor changes for AIX from Jouk Jansen (joukj@hrem.stm.tudelft.nl).
+ * Revision History:
+ * 01-Nov-2000: Allocation checks
+ * 28-Oct-1999: fixes from Jouk "I play with every mode" Jansen.
+ *               Option ttanimate added.
+ * 02-Jun-1999: patches for initialization errors of GLTT library.
+ *              Thanks to Jouk Jansen and Scott <mcmillan@cambridge.com>.
+ *              text3d updates for fortunes thanks to Jouk Jansen
+ *              <joukj@hrem.stm.tudelft.nl>
+ *              Option no_split added.
+ * 23-Aug-1998: add better handling of "faulty" fontfile and randomize
+ *              fontfile if '-ttfont' value is a directory.
+ *              Minor changes for AIX from Jouk Jansen
+ *              (joukj@hrem.stm.tudelft.nl).
  *
  * TODO :
  *       Need more animation functions. Help welcome !!
  *       Light problem with some letters (don't know why they "reflect" more):
  *       is the problem in gltt or Mesa ???
  *       SPEED !!!!
- *
  */
 
 #ifdef STANDALONE
@@ -192,6 +193,10 @@ extern "C" {
 typedef void (*t3dAnimProc) (text3dstruct * tp);
 }
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 static void t3d_anim_fullrandom(text3dstruct * tp);
 static void t3d_anim_default(text3dstruct * tp);
 static void t3d_anim_default2(text3dstruct * tp);
@@ -204,6 +209,10 @@ static void t3d_anim_rotateyz(text3dstruct * tp);
 static void t3d_anim_frequency(text3dstruct * tp);
 static void t3d_anim_amplitude(text3dstruct * tp);
 
+#ifdef __cplusplus
+}
+#endif
+
 static t3dAnimProc anim_array[] =
 {
 	t3d_anim_fullrandom,
@@ -221,18 +230,18 @@ static t3dAnimProc anim_array[] =
 
 static char * anim_names[] =
 {
-	"Random",
-	"FullRandom",
-	"Default",
-	"Default2",
-	"None",
-	"Crazy",
-	"UpDown",
-	"Extrude",
-	"RotateXY",
-	"RotateYZ",
-	"Frequency",	/* needs -rot_frequency /= 0.0 */
-	"Amplitude",	/* and   -rot_amplitude /= 0.0 */
+	(char *) "Random",
+	(char *) "FullRandom",
+	(char *) "Default",
+	(char *) "Default2",
+	(char *) "None",
+	(char *) "Crazy",
+	(char *) "UpDown",
+	(char *) "Extrude",
+	(char *) "RotateXY",
+	(char *) "RotateYZ",
+	(char *) "Frequency",	/* needs -rot_frequency /= 0.0 */
+	(char *) "Amplitude",	/* and   -rot_amplitude /= 0.0 */
 	NULL
 };
 
@@ -267,7 +276,7 @@ static int
 {
     FILE *fp;
 
-    if ((fp = my_fopen(filename, "r")) == NULL)
+    if ((fp = my_fopen(filename, (char *) "r")) == NULL)
 	return False;
     (void) fclose(fp);
     return True;
@@ -459,6 +468,10 @@ public:
 
 #define FAC_RAND	25
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*-------------------------------------------------------------*/
 static void
  t3d_anim_default(text3dstruct * tp)
@@ -643,6 +656,11 @@ static void
     anim_array[tp->animation](tp);
 }
 
+#ifdef __cplusplus
+}
+#endif
+
+
 /*-------------------------------------------------------------*/
 /*-------------------------------------------------------------*/
 /*
@@ -682,7 +700,7 @@ static void
 
     if (!nosplit)
     {
-	text_length = index_dir(tp->words, " ");
+	text_length = index_dir(tp->words, (char *) " ");
 	if (text_length == 0)
 	    text_length = strlen(tp->words);
 	c_text = (char *) malloc(text_length);
@@ -774,7 +792,7 @@ static void
 	free(c_text);
     if (size_x == 0.0)
     {
-	fprintf(stderr, "Please give something to draw !\n");
+	(void) fprintf(stderr, "Please give something to draw !\n");
 	return;
     }
 
@@ -1077,10 +1095,9 @@ static void
 void
  init_text3d(ModeInfo * mi)
 {
-    int screen = MI_SCREEN(mi);
-    text3dstruct *tp;
     char *fontfile = (char *) NULL;
     int i;
+    text3dstruct *tp;
 
     if (text3d == NULL)
     {
@@ -1088,7 +1105,7 @@ void
 					      sizeof(text3dstruct))) == NULL)
 	    return;
     }
-    tp = &text3d[screen];
+    tp = &text3d[MI_SCREEN(mi)];
     tp->wire = MI_IS_WIREFRAME(mi);
     tp->extrusion = extrusion;
     tp->rampl = rampl;
@@ -1124,7 +1141,7 @@ void
     tp->face = new FTFace;
     if (!tp->face->open(fontfile))
     {
-	fprintf(stderr, "%s: unable to open True Type font %s !\n", MI_NAME(mi), fontfile);
+	(void) fprintf(stderr, "%s: unable to open True Type font %s !\n", MI_NAME(mi), fontfile);
 	delete tp->face;
 
 	(void) free((void *) text3d);
@@ -1209,29 +1226,25 @@ void
 void
  draw_text3d(ModeInfo * mi)
 {
-    text3dstruct *tp = (text3dstruct *) NULL;
-
     Display *display = MI_DISPLAY(mi);
     Window window = MI_WINDOW(mi);
+    text3dstruct *tp;
+
+    if (text3d == NULL) {
+#ifdef USE_BLANK
+	  draw_blank(mi);
+#endif
+	  return;
+	}
+    tp = &text3d[MI_SCREEN(mi)];
 
     MI_IS_DRAWN(mi) = True;
-
-    if (text3d)
-	tp = &text3d[MI_SCREEN(mi)];
-#ifdef USE_BLANK
-    if (!tp)
-    {
-	draw_blank(mi);
+    if (!tp->glx_context)
 	return;
-    }
-#else
-    if (!tp || !tp->glx_context)
-	return;
-#endif
     tp->counter = tp->counter + 1;
     if (tp->counter > MI_CYCLES(mi) & !nosplit)
     {
-	int text_length = index_dir(tp->words, " ");
+	int text_length = index_dir(tp->words, (char *) " ");
 
 	/* Every now and then, get a new word */
 	if (text_length == 0)
@@ -1280,11 +1293,9 @@ void
 void
  release_text3d(ModeInfo * mi)
 {
-    int screen;
-
     if (text3d != NULL)
     {
-	for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+	for (int screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 	{
 	    text3dstruct *tp = &text3d[screen];
 
@@ -1310,22 +1321,18 @@ void
 void
  change_text3d(ModeInfo * mi)
 {
-    text3dstruct *tp = (text3dstruct *) NULL;
-
-    if (text3d)
-	tp = &text3d[MI_SCREEN(mi)];
+    text3dstruct *tp;
 
+    if (text3d == NULL) {
 #ifdef USE_BLANK
-    if (!tp)
-    {
-	refresh_blank(mi);
-	return;
-    }
-#else
-    if (!tp || !tp->glx_context)
-	return;
+	  refresh_blank(mi);
 #endif
+	  return;
+	}
+    tp = &text3d[MI_SCREEN(mi)];
 
+    if (!tp->glx_context)
+	return;
     glDrawBuffer(GL_BACK);
     glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(tp->glx_context));
 }
