@@ -2,7 +2,7 @@
 /* grav --- planets spinning around a pulsar */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)grav.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)grav.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,9 +22,10 @@ static const char sccsid[] = "@(#)grav.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Compatible with xscreensaver
- * 11-Jul-94: color version
- * 06-Oct-93: Written by Greg Bowering <greg@smug.student.adelaide.edu.au>
+ * 01-Nov-2000: Compatible with xscreensaver
+ * 10-May-1997: Compatible with xscreensaver
+ * 11-Jul-1994: color version
+ * 06-Oct-1993: Written by Greg Bowering <greg@smug.student.adelaide.edu.au>
  */
 
 #ifdef STANDALONE
@@ -236,8 +237,8 @@ init_grav(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
-	gravstruct *gp;
 	unsigned char ball;
+	gravstruct *gp;
 
 	if (gravs == NULL) {
 		if ((gravs = (gravstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -259,8 +260,11 @@ init_grav(ModeInfo * mi)
 		}
 		gp->nplanets = NRAND(-gp->nplanets) + 1;	/* Add 1 so its not too boring */
 	}
-	if (!gp->planets)
-		gp->planets = (planetstruct *) calloc(gp->nplanets, sizeof (planetstruct));
+	if (gp->planets == NULL) {
+		if ((gp->planets = (planetstruct *) calloc(gp->nplanets,
+				sizeof (planetstruct))) == NULL)
+			return;
+	}
 
 	MI_CLEARWINDOW(mi);
 
@@ -283,11 +287,16 @@ draw_grav(ModeInfo * mi)
 	Display    *display = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	gravstruct *gp = &gravs[MI_SCREEN(mi)];
 	register unsigned char ball;
+	gravstruct *gp;
+
+	if (gravs == NULL)
+			return;
+	gp = &gravs[MI_SCREEN(mi)];
+	if (gp->planets == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	/* Mask centrepoint */
 	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	XDrawArc(display, window, gc,
