@@ -2,7 +2,7 @@
 /* ico --- bouncing polyhedra */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ico.c	4.17 2000/03/30 xlockmore";
+static const char sccsid[] = "@(#)ico.c	5.00 2000/11/01 xlockmore";
 
 #endif
 /*-
@@ -21,12 +21,13 @@ static const char sccsid[] = "@(#)ico.c	4.17 2000/03/30 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 01-Nov-2000: Allocation checks
  * 27-Mar-2000: Added double buffering for drawings
  * 10-May-1997: Compatible with xscreensaver
- * 25-Mar-1997:  David Bagley <bagleyd@tux.org>
- *             Took ico from the X11R6 distribution.  Stripped out
- *             anything complicated... to be added back in later.
- *             added dodecahedron, tetrahedron, and star octahedron.
+ * 25-Mar-1997: David Bagley <bagleyd@tux.org>
+ *              Took ico from the X11R6 distribution.  Stripped out
+ *              anything complicated... to be added back in later.
+ *              added dodecahedron, tetrahedron, and star octahedron.
  * $XConsortium: ico.c,v 1.47 94/04/17 20:45:15 gildea Exp $
  */
 
@@ -483,8 +484,9 @@ static int  polysize = sizeof (polygons) / sizeof (polygons[0]);
 #define POLYSIZE 5  /* Only the 5 Platonic solids work, why is that? */
 
 #define POLYBITS(n,w,h)\
-  ip->pixmaps[ip->init_bits++]=\
-  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
+  if ((ip->pixmaps[ip->init_bits]=\
+  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1))==None){\
+  free_ico(display,ip);return;} else {ip->init_bits++;}
 
 typedef double Transform3D[4][4];
 
@@ -909,13 +911,38 @@ drawPoly(ModeInfo * mi, Polyinfo * poly, GC gc,
 	XFlush(display);
 }
 
+static void
+free_ico(Display *display, icostruct *ip)
+{
+	int shade;
+
+	if (ip->stippledGC != None) {
+		XFreeGC(display, ip->stippledGC);
+		ip->stippledGC = None;
+	}
+	for (shade = 0; shade < ip->init_bits; shade++) {
+		if (ip->pixmaps[shade] != None) {
+			XFreePixmap(display, ip->pixmaps[shade]);
+			ip->pixmaps[shade] = None;
+		}
+	}
+	if (ip->dbuf != None) {
+		XFreePixmap(display, ip->dbuf);
+		ip->dbuf = None;
+	}
+	if (ip->dbuf_gc != None) {
+		XFreeGC(display, ip->dbuf_gc);
+		ip->dbuf_gc = None;
+	}
+}
+
 void
 init_ico(ModeInfo * mi)
 {
 	Display * display = MI_DISPLAY(mi);
 	Window window = MI_WINDOW(mi);
-	icostruct  *ip;
 	int         size = MI_SIZE(mi);
+	icostruct  *ip;
 
 	if (icos == NULL) {
 		if ((icos = (icostruct *) calloc(MI_NUM_SCREENS(mi),
@@ -933,14 +960,18 @@ init_ico(ModeInfo * mi)
 			XGCValues   gcv;
 
 			gcv.fill_style = FillOpaqueStippled;
-			ip->stippledGC = XCreateGC(display, window, GCFillStyle,
-				 &gcv);
+			if ((ip->stippledGC = XCreateGC(display, window, GCFillStyle,
+					 &gcv)) == NULL) {
+				free_ico(display, ip);
+				return;
+			}
 		}
 		if (ip->init_bits == 0) {
 			int i;
 
-			for (i = 1; i < NUMSTIPPLES; i++)
+			for (i = 1; i < NUMSTIPPLES; i++) {
 				POLYBITS(stipples[i], STIPPLESIZE, STIPPLESIZE);
+			}
 		}
 	}
 
@@ -993,19 +1024,16 @@ init_ico(ModeInfo * mi)
 	else if (ip->faces && ip->edges)
 		ip->color = (LRAND() & 1) ? MI_WHITE_PIXEL(mi) : MI_BLACK_PIXEL(mi);
 
-	if (ip->dbuf)
+	ip->color_offset = NRAND(MI_NPIXELS(mi));
+#ifndef NO_DBUF
+	if (ip->dbuf != None)
 		XFreePixmap(display, ip->dbuf);
-#ifdef NO_DBUF
-	ip->dbuf = NULL;
-#else
 	ip->dbuf = XCreatePixmap(display, window,
 		ip->polyW + ip->deltaWidth + ip->linewidth,
 		ip->polyH + ip->deltaHeight + ip->linewidth,
-		MI_DEPTH(mi));
-#endif
-	ip->color_offset = NRAND(MI_NPIXELS(mi));
-
-	if (ip->dbuf) {
+	 	MI_DEPTH(mi));
+	/* Allocation checked */
+	if (ip->dbuf != None) {
 		XGCValues   gcv;
 
 		gcv.foreground = 0;
@@ -1013,17 +1041,22 @@ init_ico(ModeInfo * mi)
 		gcv.graphics_exposures = False;
 		gcv.function = GXcopy;
 
-		if (ip->dbuf_gc)
+		if (ip->dbuf_gc != None)
 			XFreeGC(display, ip->dbuf_gc);
-		ip->dbuf_gc = XCreateGC(display, ip->dbuf,
-			GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
-			&gcv);
-		XFillRectangle(display, ip->dbuf, ip->dbuf_gc,
-			0, 0, ip->polyW + ip->deltaWidth + ip->linewidth,
-			ip->polyH + ip->deltaHeight + ip->linewidth);
-		XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
-		XSetFunction(display, MI_GC(mi), GXcopy);
+		if ((ip->dbuf_gc = XCreateGC(display, ip->dbuf,
+				GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
+				&gcv)) == None) {
+			XFreePixmap(display, ip->dbuf);
+			ip->dbuf = None;
+		} else {
+			XFillRectangle(display, ip->dbuf, ip->dbuf_gc,
+				0, 0, ip->polyW + ip->deltaWidth + ip->linewidth,
+				ip->polyH + ip->deltaHeight + ip->linewidth);
+			XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
+			XSetFunction(display, MI_GC(mi), GXcopy);
+		}
 	}
+#endif
 
 	MI_CLEARWINDOW(mi);
 
@@ -1037,10 +1070,15 @@ init_ico(ModeInfo * mi)
 void
 draw_ico(ModeInfo * mi)
 {
-	icostruct  *ip = &icos[MI_SCREEN(mi)];
+	icostruct  *ip;
+
+	if (icos == NULL)
+		return;
+	ip = &icos[MI_SCREEN(mi)];
+	if ((MI_NPIXELS(mi) <= 2) && (ip->stippledGC == None))
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (++ip->loopcount > MI_CYCLES(mi))
 		init_ico(mi);
 
@@ -1076,20 +1114,9 @@ release_ico(ModeInfo * mi)
 {
 	if (icos != NULL) {
 		int screen;
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			icostruct  *ip = &icos[screen];
-			int shade;
 
-                        if (ip->stippledGC != None) {
-                                XFreeGC(MI_DISPLAY(mi), ip->stippledGC);
-                        }
-                        for (shade = 0; shade < ip->init_bits; shade++)
-                                XFreePixmap(MI_DISPLAY(mi), ip->pixmaps[shade]);
-			if (ip->dbuf)
-				XFreePixmap(MI_DISPLAY(mi), ip->dbuf);
-			if (ip->dbuf_gc)
-				XFreeGC(MI_DISPLAY(mi), ip->dbuf_gc);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_ico(MI_DISPLAY(mi), &icos[screen]);
 		(void) free((void *) icos);
 		icos = NULL;
 	}
@@ -1098,7 +1125,13 @@ release_ico(ModeInfo * mi)
 void
 change_ico(ModeInfo * mi)
 {
-	icostruct  *ip = &icos[MI_SCREEN(mi)];
+	icostruct  *ip;
+
+	if (icos == NULL)
+		return;
+	ip = &icos[MI_SCREEN(mi)];
+	if (MI_NPIXELS(mi) <= 2 && ip->stippledGC == None)
+		return;
 
 	if (MI_COUNT(mi) <= 0 || MI_COUNT(mi) > POLYSIZE) {
 		ip->object = (ip->object + 1) % (POLYSIZE);
