@@ -2,7 +2,7 @@
 /* juggle */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)juggle.c 1.0 99/10/16 xlockmore";
+static const char sccsid[] = "@(#)juggle.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -21,6 +21,9 @@ static const char sccsid[] = "@(#)juggle.c 1.0 99/10/16 xlockmore";
  * event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
  *
+ * Revision History
+ * 01-Nov-2000: Allocation checks
+ * 1996: Written
  */
 
 /*-
@@ -115,37 +118,53 @@ so the answer is to do a single 661 and then drop straight down to (5).
 #ifdef MODE_juggle
 
 #define DEF_PATTERN "" /* All patterns */
-#define DEF_PATTERNTRACE "0" /* No trace */
-#define DEF_SOLID "0" /* Not solid */
+#define DEF_TRAIL "0" /* No trace */
+#ifdef UNI
+#define DEF_UNI "FALSE" /* No unicycle */ /* Not implemented yet */
+#endif
+#define DEF_SOLID "FALSE" /* Not solid */
 
 static char *pattern;
-static int patterntrace;
+static int trail;
+#ifdef UNI
+static Bool uni;
+#endif
 static Bool solid;
 
 static XrmOptionDescRec opts[] =
 {
   {(char* ) "-pattern", (char *) ".juggle.pattern",
    XrmoptionSepArg, (caddr_t) NULL},
-  {(char* ) "-patterntrace", (char *) ".juggle.patterntrace",
+  {(char* ) "-trail", (char *) ".juggle.trail",
    XrmoptionSepArg, (caddr_t) NULL},
+#ifdef UNI
+  {(char *) "-uni", (char *) ".juggle.uni", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+uni", (char *) ".juggle.uni", XrmoptionNoArg, (caddr_t) "off"},
+#endif
   {(char *) "-solid", (char *) ".juggle.solid", XrmoptionNoArg, (caddr_t) "on"},
-  {(char *) "+solid", (char *) ".juggle.solid", XrmoptionNoArg, (caddr_t) "off"}, 
+  {(char *) "+solid", (char *) ".juggle.solid", XrmoptionNoArg, (caddr_t) "off"}
 };
 static argtype vars[] =
 {
   {(caddr_t *) &pattern, (char *) "pattern",
    (char *) "Pattern", (char *) DEF_PATTERN, t_String},
-  {(caddr_t *) &patterntrace, (char *) "patterntrace",
-   (char *) "PatternTrace", (char *) DEF_PATTERNTRACE, t_Int},
+  {(caddr_t *) &trail, (char *) "trail",
+   (char *) "Trail", (char *) DEF_TRAIL, t_Int},
+#ifdef UNI
+  {(caddr_t *) &uni, (char *) "uni",
+   (char *) "Uni", (char *) DEF_UNI, t_Bool},
+#endif
   {(caddr_t *) &solid, (char *) "solid",
    (char *) "Solid", (char *) DEF_SOLID, t_Bool}
-
 };
 static OptionStruct desc[] =
 {
   {(char *) "-pattern string", (char *) "Cambridge Juggling Pattern"},
-  {(char *) "-patterntrace num", (char *) "Trace Juggling Patterns"},
-  {(char *) "-/+solid", (char *) "solid color (else its a 4 panel look (half white))"},
+  {(char *) "-trail num", (char *) "Trace Juggling Patterns"},
+#ifdef UNI
+  {(char *) "-/+uni", (char *) "Unicycle"},
+#endif
+  {(char *) "-/+solid", (char *) "solid color (else its a 4 panel look (half white))"}
 };
 
 ModeSpecOpt juggle_opts =
@@ -272,7 +291,7 @@ typedef struct {
   int         count;
   time_t      begintime; /* seconds */
   int         time; /* millisecond timer */
-  Bool        solid;	
+  Bool        solid, uni;	
 } jugglestruct;
 
 static jugglestruct *juggles = NULL;
@@ -294,44 +313,44 @@ static PatternIndex* patternindex = NULL;
 
 /* List of popular patterns, in any order */
 static patternstruct portfolio[] = {
-  {"[+2 1]", "+3 1, Typical 2 ball juggler"},
-  {"[2 0]", "4 0, 2 balls 1 hand"},
-  {"[2 0 1]", "5 0 1"},
-  {"[+2 0 +2 0 0]", "+5 0 +5 0 0"},
-  {"[3]", "3, cascade"},
-  {"[+3]", "+3, reverse cascade"},
-  {"[=3]", "=3, cascade under arm"},
-  {"[&3]", "&3, cascade catching under arm"},
-  {"[_3]", "_3, bouncing cascade"},
-  {"[+3 x3 =3]", "+3 x3 =3, Mill's mess"},
-  {"[3 2 1]", "5 3 1"},
-  {"[3 3 1]", "4 4 1"},
-  {"[3 1 2]",  "6 1 2, See-saw"},
-  {"[=3 3 1 2]", "=4 5 1 2"},
-  {"[=3 2 2 3 1 2]", "=6 2 2 5 1 2, =4 5 1 2 stretched"},
-  {"[+3 3 1 3]",  "+4 4 1 3, anemic shower box"},
-  {"[3 3 1]",  "4 4 1"},
-  {"[+3 2 3]",  "+4 2 3"},
-  {"[+3 1]",  "+5 1, 3 shower"},
-  {"[_3 1]",  "_5 1, bouncing 3 shower"},
-  {"[3 0 3 0 3]", "5 0 5 0 5, shake 3 out of 5"},
-  {"[3 3 3 0 0]",  "5 5 5 0 0, flash 3 out of 5"},
-  {"[3 3 0]", "4 5 0, complete waste of a 5 ball juggler"},
-  {"[3 3 3 0 0 0 0]", "7 7 7 0 0 0 0, 3 flash"},
-  {"[+3 0 +3 0 +3 0 0]", "+7 0 +7 0 +7 0 0"},
-  {"[4]", "4, 4 cascade"},
-  {"[+4 3]", "+5 3, 4 ball half shower"},
-  {"[4 4 2]", "5 5 2"},
-  {"[+4 4 4 +4]", "+4 4 4 +4, 4 columns"},
-  {"[4 3 +4]", "5 3 +4"},
-  {"[+4 1]", "+7 1, 4 shower"},
-  {"[4 4 4 4 0]", "5 5 5 5 0, learning 5"},
-  {"[5]", "5, 5 cascade"},
-  {"[_5 _5 _5 _5 _5 5 5 5 5 5]", "_5 _5 _5 _5 _5 5 5 5 5 5, jump rope"},
-  {"[+5 x5 =5]", "+5 x5 =5, Mill's mess for 5"},
-  {"[6]", "6, 6 cascade"},
-  {"[7]", "7, 7 cascade"},
-  {"[_7]", "_7, bouncing 7 cascade"},
+  {(char *) "[+2 1]", (char *) "+3 1, Typical 2 ball juggler"},
+  {(char *) "[2 0]", (char *) "4 0, 2 balls 1 hand"},
+  {(char *) "[2 0 1]", (char *) "5 0 1"},
+  {(char *) "[+2 0 +2 0 0]", (char *) "+5 0 +5 0 0"},
+  {(char *) "[3]", (char *) "3, cascade"},
+  {(char *) "[+3]", (char *) "+3, reverse cascade"},
+  {(char *) "[=3]", (char *) "=3, cascade under arm"},
+  {(char *) "[&3]", (char *) "&3, cascade catching under arm"},
+  {(char *) "[_3]", (char *) "_3, bouncing cascade"},
+  {(char *) "[+3 x3 =3]", (char *) "+3 x3 =3, Mill's mess"},
+  {(char *) "[3 2 1]", (char *) "5 3 1"},
+  {(char *) "[3 3 1]", (char *) "4 4 1"},
+  {(char *) "[3 1 2]", (char *) "6 1 2, See-saw"},
+  {(char *) "[=3 3 1 2]", (char *) "=4 5 1 2"},
+  {(char *) "[=3 2 2 3 1 2]", (char *) "=6 2 2 5 1 2, =4 5 1 2 stretched"},
+  {(char *) "[+3 3 1 3]", (char *) "+4 4 1 3, anemic shower box"},
+  {(char *) "[3 3 1]", (char *) "4 4 1"},
+  {(char *) "[+3 2 3]", (char *) "+4 2 3"},
+  {(char *) "[+3 1]", (char *) "+5 1, 3 shower"},
+  {(char *) "[_3 1]", (char *) "_5 1, bouncing 3 shower"},
+  {(char *) "[3 0 3 0 3]", (char *) "5 0 5 0 5, shake 3 out of 5"},
+  {(char *) "[3 3 3 0 0]", (char *) "5 5 5 0 0, flash 3 out of 5"},
+  {(char *) "[3 3 0]", (char *) "4 5 0, complete waste of a 5 ball juggler"},
+  {(char *) "[3 3 3 0 0 0 0]", (char *) "7 7 7 0 0 0 0, 3 flash"},
+  {(char *) "[+3 0 +3 0 +3 0 0]", (char *) "+7 0 +7 0 +7 0 0"},
+  {(char *) "[4]", (char *) "4, 4 cascade"},
+  {(char *) "[+4 3]", (char *) "+5 3, 4 ball half shower"},
+  {(char *) "[4 4 2]", (char *) "5 5 2"},
+  {(char *) "[+4 4 4 +4]", (char *) "+4 4 4 +4, 4 columns"},
+  {(char *) "[4 3 +4]", (char *) "5 3 +4"},
+  {(char *) "[+4 1]", (char *) "+7 1, 4 shower"},
+  {(char *) "[4 4 4 4 0]", (char *) "5 5 5 5 0, learning 5"},
+  {(char *) "[5]", (char *) "5, 5 cascade"},
+  {(char *) "[_5 _5 _5 _5 _5 5 5 5 5 5]", (char *) "_5 _5 _5 _5 _5 5 5 5 5 5, jump rope"},
+  {(char *) "[+5 x5 =5]", (char *) "+5 x5 =5, Mill's mess for 5"},
+  {(char *) "[6]", (char *) "6, 6 cascade"},
+  {(char *) "[7]", (char *) "7, 7 cascade"},
+  {(char *) "[_7]", (char *) "_7, bouncing 7 cascade"},
 };
 
 /* Private Functions */
@@ -340,9 +359,9 @@ static patternstruct portfolio[] = {
 
 /* t receives trajectory to be created.  ot must point to an existing
    trajectory or be identical to t to start a new list. */
-#define INSERT_AFTER(t, ot)					\
+#define INSERT_AFTER(t, ot, ret)					\
   if ((t = (Trajectory *)calloc(1, sizeof(Trajectory))) == NULL) \
-    (void) fprintf(stderr, "Memory problems in juggle mode\n");	\
+    {free_juggle(sp); return ret;}				\
   (t)->next = (ot)->next;					\
   (t)->prev = (ot);						\
   (ot)->next = (t);						\
@@ -356,11 +375,34 @@ static patternstruct portfolio[] = {
   (void) free((void *) t)
 
 static void
+free_pattern(jugglestruct *sp) {
+	if (sp->head != NULL) {
+		while (sp->head->next != sp->head) {
+			Trajectory *t = sp->head->next;
+
+			REMOVE(t); /* don't eliminate t */
+		}
+		(void) free((void *) sp->head);
+  		sp->head = NULL;
+	}
+}
+
+static void
+free_juggle(jugglestruct *sp)
+{
+	if (sp->trace != NULL) {
+		(void) free((void *) sp->trace);
+		sp->trace = NULL;
+	}
+	free_pattern(sp);
+}
+
+static Bool
 add_throw(jugglestruct *sp, char type, int h, Notation n)
 {
   Trajectory *t;
 
-  INSERT_AFTER(t, sp->head->prev);
+  INSERT_AFTER(t, sp->head->prev, False);
   t->posn = type;
   if (n == ADAM) {
 	t->adam = h;
@@ -369,23 +411,11 @@ add_throw(jugglestruct *sp, char type, int h, Notation n)
 	t->height = h;
 	t->status = THRATCH;
   }
-}
-
-static void
-free_pattern(jugglestruct *sp) {
-  if (sp->head) {
-	while (sp->head->next != sp->head) {
-	  Trajectory *t = sp->head->next;
-
-	  REMOVE(t); /* don't eliminate t */
-	}
-	(void) free((void *) sp->head);
-  }
-  sp->head = NULL;
+  return True;
 }
 
 /* add a Thratch to the performance */
-static void
+static Bool
 program(ModeInfo *mi, const char *patn, int repeat)
 {
   jugglestruct *sp = &juggles[MI_SCREEN(mi)];
@@ -395,7 +425,7 @@ program(ModeInfo *mi, const char *patn, int repeat)
   char type;
 
   if (MI_IS_VERBOSE(mi)) {
-	fprintf(stderr, "%s x %d\n", patn, repeat);
+	(void) fprintf(stderr, "%s x %d\n", patn, repeat);
   }
 
   for(i=0; i < repeat; i++) {
@@ -430,7 +460,8 @@ program(ModeInfo *mi, const char *patn, int repeat)
 		  /* fall through */
 		case ' ':
 		  if (seen) {
-			add_throw(sp, type, h, notation);
+			if (!add_throw(sp, type, h, notation))
+				return False;
 			type=' ';
 			h = 0;
 			seen = 0;
@@ -444,9 +475,11 @@ program(ModeInfo *mi, const char *patn, int repeat)
 	  }
 	}
 	if (seen) {
-	  add_throw(sp, type, h, notation);
+	  if (!add_throw(sp, type, h, notation))
+		return False;
 	}
   }
+	return True;
 }
 
 /*
@@ -494,7 +527,7 @@ adam(jugglestruct *sp)
 /* ..n1.. -> .. LTn RT1 LC RC .. */
 /* ..nm.. -> .. LTn LC RTm RC .. */
 
-static void
+static Bool
 part(jugglestruct *sp)
 {
   Trajectory *t, *nt, *p;
@@ -534,7 +567,7 @@ part(jugglestruct *sp)
 		p = p->prev; /* early throw */
 	  }
 	  t->action = CATCH;
-	  INSERT_AFTER(nt, p);
+	  INSERT_AFTER(nt, p, False);
 	  nt->status = ACTION;
 	  nt->action = THROW;
 	  nt->height = t->height;
@@ -542,6 +575,7 @@ part(jugglestruct *sp)
 	  nt->posn = posn;
 	}
   }
+  return True;
 }
 
 /* Connnect up throws and catches to figure out which ball goes where.
@@ -709,7 +743,7 @@ makeSplinePair(Spline *s1, Spline *s2,
 }
 
 /* Turn abstract timings into physically appropriate ball trajectories. */
-static void
+static Bool
 projectile(jugglestruct *sp)
 {
   Trajectory *t, *n;
@@ -739,7 +773,7 @@ projectile(jugglestruct *sp)
 		  for(i = tc / 2; i > 0; i/=2){
 			double dy, dt, yt;
 			if(tb == 0){
-			  fprintf(stderr, "div by zero!\n");
+			  (void) fprintf(stderr, "div by zero!\n");
 			  break;
 			}
 			dy = (yf - y0)/tb + 0.5*sp->Gr*tb;
@@ -760,7 +794,7 @@ projectile(jugglestruct *sp)
 			t->dx = (t->balllink->x - t->x) / tc;
 
 			/* ball follows parabola down */
-			INSERT_AFTER(n, t->prev);
+			INSERT_AFTER(n, t->prev, False);
 			n->start = t->start;
 			n->finish = (int) (t->start + tb);
 			n->type = Ball;
@@ -785,7 +819,7 @@ projectile(jugglestruct *sp)
 
 
 			/* ball follows parabola up */
-			INSERT_AFTER(n, t->prev);
+			INSERT_AFTER(n, t->prev, False);
 			n->start = (int) (t0 + tb);
 			n->finish = (int) (t0 + tc);
 			n->type = Ball;
@@ -816,7 +850,7 @@ projectile(jugglestruct *sp)
 		  double t0, dt;
 
 		  /* ball follows parabola */
-		  INSERT_AFTER(n, t->prev);
+		  INSERT_AFTER(n, t->prev, False);
 		  n->start = t->start;
 		  n->finish = t->balllink->start;
 		  n->type = Ball;
@@ -850,6 +884,7 @@ projectile(jugglestruct *sp)
 	  }
 	}
   }
+  return True;
 }
 
 /* Turn abstract hand motions into cubic splines. */
@@ -929,6 +964,8 @@ draw_arm(ModeInfo * mi, Hand side, int *x, int *y)
 
   int sig = (side == LEFT) ? 1 : -1;
 
+  XSetLineAttributes(dpy, gc,
+		ARMWIDTH, LineSolid, CapRound, JoinRound);
   if (sp->arm[side][0].x != *x || sp->arm[side][0].y != *y) {
 	XPoint h, e;
 	XSetForeground(dpy, gc, MI_BLACK_PIXEL(mi));
@@ -941,6 +978,8 @@ draw_arm(ModeInfo * mi, Hand side, int *x, int *y)
   }
   XSetForeground(dpy, gc, MI_WHITE_PIXEL(mi));
   XDrawLines(dpy, win, gc, sp->arm[side], 3, CoordModeOrigin);
+  XSetLineAttributes(dpy, gc,
+		1, LineSolid, CapNotLast, JoinRound);
 }
 
 static void
@@ -951,12 +990,15 @@ draw_figure(ModeInfo * mi)
   GC gc = MI_GC(mi);
   jugglestruct *sp = &juggles[MI_SCREEN(mi)];
 
+  XSetLineAttributes(dpy, gc,
+		ARMWIDTH, LineSolid, CapRound, JoinRound);
   XSetForeground(dpy, gc, MI_WHITE_PIXEL(mi));
-  XDrawLines(dpy, win, gc, sp->figure_path, FIGURE1,
-			 CoordModeOrigin);
+  XDrawLines(dpy, win, gc, sp->figure_path, FIGURE1, CoordModeOrigin);
   XDrawSegments(dpy, win, gc, sp->figure_segs, FIGURE2);
   XDrawArc(dpy, win, gc,
-		   sp->cx - HED/2, sp->cy + NEY - HED, HED, HED, 0, 64*360);
+	 sp->cx - HED/2, sp->cy + NEY - HED, HED, HED, 0, 64*360);
+  XSetLineAttributes(dpy, gc,
+		1, LineSolid, CapNotLast, JoinRound);
 }
 
 
@@ -1071,11 +1113,8 @@ init_juggle(ModeInfo * mi)
 	  /* last pattern has most balls */
 	  maxballs = get_num_balls(portfolio[nelements - 1].pattern);
 	  /* allocate index */
-	  if ((patternindex =
-		   (PatternIndex *) calloc(maxballs + 1,
+	  if ((patternindex = (PatternIndex *) calloc(maxballs + 1,
 				sizeof (PatternIndex))) == NULL) {
-    			(void) fprintf(stderr,
-			  "Memory problems for patternindex in juggle mode\n");
 		return;
 	  }
 
@@ -1086,7 +1125,7 @@ init_juggle(ModeInfo * mi)
 		int b = get_num_balls(portfolio[i].pattern);
 		if (b > maxballs) {
 		  if (MI_IS_VERBOSE(mi)) {
-			fprintf(stderr, "%d %d ball pattern%s\n",
+			(void) fprintf(stderr, "%d %d ball pattern%s\n",
 					numpat, maxballs, (numpat == 1) ? "" : "s");
 		  }
 		  patternindex[maxballs].number = numpat;
@@ -1098,7 +1137,7 @@ init_juggle(ModeInfo * mi)
 		}
 	  }
 	  if (MI_IS_VERBOSE(mi)) {
-		fprintf(stderr, "%d %d ball pattern%s\n",
+		(void) fprintf(stderr, "%d %d ball pattern%s\n",
 				numpat, maxballs, (numpat == 1) ? "" : "s");
 	  }
 	  patternindex[maxballs].number = numpat;
@@ -1107,8 +1146,7 @@ init_juggle(ModeInfo * mi)
 	if (juggles == NULL) { /* allocate jugglestruct */
 		if ((juggles = (jugglestruct *) calloc(MI_NUM_SCREENS(mi),
 			       sizeof (jugglestruct))) == NULL) {
-    		  	(void) fprintf(stderr,
-			  "Memory problems for jugglestruct in juggle mode\n");
+			release_juggle(mi);
 			return;
 		}
 	}
@@ -1118,8 +1156,14 @@ init_juggle(ModeInfo * mi)
 
 	if (MI_IS_FULLRANDOM(mi)) {
 		sp->solid = (Bool) (LRAND() & 1);
+#ifdef UNI
+		sp->uni = (Bool) (LRAND() & 1);
+#endif
 	} else {
 		sp->solid = solid;
+#ifdef UNI
+		sp->uni = uni;
+#endif
 	}
 
 	sp->width = MI_WIDTH(mi);
@@ -1128,7 +1172,7 @@ init_juggle(ModeInfo * mi)
 	sp->scale = sp->height / 480.0;
 	/* vary x a little so the juggler does not burn the screen */
 	sp->cx = sp->width / 2 + RFX + NRAND(LFX - RFX + 1);
-	sp->cy = sp->height - FY;
+	sp->cy = sp->height - FY - ((int) sp->uni) * FY / 3; /* raise higher */
 	/* "7" hits top of screen */
 	sp->Gr = GRAVITY(sp->cy, 7 * THROW_CATCH_INTERVAL);
 
@@ -1165,14 +1209,15 @@ init_juggle(ModeInfo * mi)
 	sp->arm[RIGHT][2].y = sp->cy + SY;
 
 	if (sp->trace == NULL) {
-	  sp->trace = (XPoint *)calloc(patterntrace, sizeof(XPoint));
+	  if ((sp->trace = (XPoint *)calloc(trail, sizeof(XPoint))) == NULL) {
+		free_juggle(sp);
+ 		return;
+	  }
 	}
 
 	/* Clear the background. */
 	MI_CLEARWINDOW(mi);
 
-  	XSetLineAttributes(MI_DISPLAY(mi), MI_GC(mi),
-		ARMWIDTH, LineSolid, CapRound, JoinRound);
 	draw_figure(mi);
 
 	/* record start time */
@@ -1181,7 +1226,7 @@ init_juggle(ModeInfo * mi)
 	free_pattern(sp);
 
 	/* create circular list */
-	INSERT_AFTER(sp->head, sp->head);
+	INSERT_AFTER(sp->head, sp->head, /**/);
 
 	/* generate pattern */
 	if (pattern == NULL) {
@@ -1219,7 +1264,8 @@ init_juggle(ModeInfo * mi)
 			new_balls -= 2;
 		  }
 		  if (new_balls < num_balls) {
-			program(mi, "[*]", 1); /* lose ball */
+			if (!program(mi, "[*]", 1)) /* lose ball */
+				return;
 		  }
 		  num_balls = new_balls;
 		}
@@ -1227,10 +1273,11 @@ init_juggle(ModeInfo * mi)
 
 		if (NRAND(2) && patternindex[num_balls].number) {
 		  /* Pick from PortFolio */
-		  program(mi,
+		  if (!program(mi,
 			  portfolio[patternindex[num_balls].start +
 				  NRAND(patternindex[num_balls].number)].pattern,
-				  t);
+				  t))
+			return;
 		} else {
 		  /* Invent a new pattern */
 		  *b++='[';
@@ -1266,23 +1313,27 @@ init_juggle(ModeInfo * mi)
 		  *b++ = ']';
 		  *b = '\0';
 		  if (maxseen) {
-			program(mi, buf, t);
+			if (!program(mi, buf, t))
+				return;
 		  }
 		}
 	  }
 	} else { /* pattern supplied in height or 'a' notation */
-	  program(mi, pattern, MI_CYCLES(mi));
+	  if (!program(mi, pattern, MI_CYCLES(mi)))
+		return;
 	}
 
 	adam(sp);
 
-	part(sp);
+	if (!part(sp))
+		return;
 
  	lob(mi);
 
 	positions(sp);
 
-	projectile(sp);
+	if (!projectile(sp))
+		return;
 
 	hands(sp);
 
@@ -1313,7 +1364,7 @@ init_juggle(ModeInfo * mi)
 #endif
 
 static void
-draw_juggle_ball(ModeInfo *mi, long color, int x, int y, int degree_offset, int divisions)
+draw_juggle_ball(ModeInfo *mi, unsigned long color, int x, int y, int degree_offset, int divisions)
 {
 	Display    *dpy = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
@@ -1322,7 +1373,8 @@ draw_juggle_ball(ModeInfo *mi, long color, int x, int y, int degree_offset, int
 	int offset;
 
 	XSetForeground(dpy, gc, color);
-	if (color == MI_WHITE_PIXEL(mi) || (divisions != 2 && divisions != 4) || sp->solid) {
+	if ((color == MI_WHITE_PIXEL(mi)) ||
+	    ((divisions != 2) && (divisions != 4)) || sp->solid) {
 		XFillArc(dpy, window, gc,
 			x - BALLRADIUS, y - BALLRADIUS,
 			2*BALLRADIUS, 2*BALLRADIUS,
@@ -1368,10 +1420,16 @@ draw_juggle(ModeInfo * mi)
 	Display    *dpy = MI_DISPLAY(mi);
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
-	jugglestruct *sp = &juggles[MI_SCREEN(mi)];
 	Trajectory *traj;
 	int future = 0;
 	int length = 0;
+	jugglestruct *sp;
+
+	if (juggles == NULL)
+		return;
+	sp = &juggles[MI_SCREEN(mi)];
+	if (sp->trace == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
 
@@ -1395,7 +1453,7 @@ draw_juggle(ModeInfo * mi)
 	  } else if (sp->time < traj->finish) { /* working */
 		int x = (int) CUBIC(traj->xp, sp->time);
 		int y = (int) CUBIC(traj->yp, sp->time);
-		long color;
+		unsigned long color;
 
 		if (MI_NPIXELS(mi) > 2) {
 		  color = MI_PIXEL(mi, traj->color);
@@ -1406,12 +1464,12 @@ draw_juggle(ModeInfo * mi)
 		  draw_arm(mi, traj->hand, &x, &y);
 		}
 		if (traj->type == Ball || traj->type == Full) {
-		  if(patterntrace > 0) {
+		  if(trail > 0) {
 			ERASE_BALL(sp->trace[sp->traceindex].x,
 				sp->trace[sp->traceindex].y);
 			sp->trace[sp->traceindex].x = traj->x;
 			sp->trace[sp->traceindex].y = traj->y;
-			if (++sp->traceindex >= patterntrace) {
+			if (++sp->traceindex >= trail) {
 			  sp->traceindex = 0;
 			}
 		  } else {
@@ -1447,22 +1505,18 @@ draw_juggle(ModeInfo * mi)
 void
 release_juggle(ModeInfo * mi)
 {
-  if (juggles != NULL) {
-	int screen;
+	if (juggles != NULL) {
+		int screen;
 
-	for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-	  jugglestruct *sp = &juggles[screen];
-
-	 if (sp->trace != NULL) {
-           (void) free((void *) sp->trace);
-         }
-	  free_pattern(sp);
-          XSetLineAttributes(MI_DISPLAY(mi), MI_GC(mi),
-	   1, LineSolid, CapNotLast, JoinRound);
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_juggle(&juggles[screen]);
+		(void) free((void *) juggles);
+		juggles = NULL;
+	}
+	if (patternindex != NULL) {
+		(void) free((void *) patternindex);
+		patternindex = NULL;
 	}
-	(void) free((void *) juggles);
-	juggles = NULL;
-  }
 }
 
 #endif /* MODE_juggle */
