@@ -2,7 +2,7 @@
 /* loop --- Chris Langton's self-producing loops */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)loop.c	4.13 98/10/18 xlockmore";
+static const char sccsid[] = "@(#)loop.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,22 +22,28 @@ static const char sccsid[] = "@(#)loop.c	4.13 98/10/18 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 01-Nov-2000: Allocation checks
  * 16-Jun-2000: Fully coded the hexagonal rules.  (Rules that end up in
- *   state zero were not bothered with since a calloc was used to set
- *   non-explicit rules to zero.  This allows the compile-time option
- *   RAND_RULES to work here (at least to generation 540).)
- *   Also added compile-time option DELAYDEBUGLOOP for debugging life form.
- *   This also turns off the random initial direction of the loop.
- *   Set DELAYDEBUGLOOP to be 10 and use with something like this:
- *   xlock -mode loop -neighbors 6 -size 5 -delay 1 -count 540 -nolock
+ *              state zero were not bothered with since a calloc was used
+ *              to set non-explicit rules to zero.  This allows the
+ *              compile-time option RAND_RULES to work here (at least to
+ *              generation 540).)
+ *              Also added compile-time option DELAYDEBUGLOOP for debugging
+ *              life form.  This also turns off the random initial direction
+ *              of the loop.  Set DELAYDEBUGLOOP to be 10 and use with
+ *              something like this:
+ *       xlock -mode loop -neighbors 6 -size 5 -delay 1 -count 540 -nolock
  * 18-Oct-1998: Started creating a hexagon version.
- *   It proved not that difficult because I used Langton's Loop
- *   as a guide, hexagon have more neighbors so there is more
- *   freedom to program, and the loop will has six sides to
- *   store its genes (data).
+ *              It proved not that difficult because I used Langton's Loop
+ *              as a guide, hexagon have more neighbors so there is more
+ *              freedom to program, and the loop will has six sides to
+ *              store its genes (data).
+ *              (Soon after this a triangular version with neighbors = 6 but
+ *              was unsuccessful).
  * 10-May-1997: Compatible with xscreensaver
  * 15-Nov-1995: Coded from Chris Langton's Self-Reproduction in Cellular
- *   Automata Physica 10D 135-144 1984, also used wire.c as a guide.
+ *              Automata Physica 10D 135-144 1984, also used wire.c as a
+ *              guide.
  */
 
 /*-
@@ -126,8 +132,9 @@ ModStruct   loop_description =
 #endif
 
 #define LOOPBITS(n,w,h)\
-  lp->pixmaps[lp->init_bits++]=\
-  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1)
+  if ((lp->pixmaps[lp->init_bits]=\
+  XCreatePixmapFromBitmapData(display,window,(char *)n,w,h,1,0,1))==None){\
+  free_loop(display,lp); return;} else {lp->init_bits++;}
 
 static int  local_neighbors = 0;
 
@@ -211,6 +218,7 @@ typedef struct {
 static loopstruct *loops = NULL;
 
 #define TRANSITION(TT,V) V=TT&7;TT>>=3
+#define FINALTRANSITION(TT,V) V=TT&7
 #define TABLE(R,T,L,B) (table[((B)<<9)|((L)<<6)|((T)<<3)|(R)])
 #define HEX_TABLE(R,T,t,l,b,B) (table[((B)<<15)|((b)<<12)|((l)<<9)|((t)<<6)|((T)<<3)|(R)])
 
@@ -859,23 +867,6 @@ drawcell(ModeInfo * mi, int col, int row, int state)
 	fillcell(mi, gc, col, row);
 }
 
-static void
-addtolist(ModeInfo * mi, int col, int row, unsigned char state)
-{
-	loopstruct *lp = &loops[MI_SCREEN(mi)];
-	CellList   *current = lp->cellList[state];
-
-	lp->cellList[state] = NULL;
-	if ((lp->cellList[state] = (CellList *) malloc(sizeof (CellList))) == NULL) {
-		lp->cellList[state] = current;
-		return;
-	}
-	lp->cellList[state]->pt.x = col;
-	lp->cellList[state]->pt.y = row;
-	lp->cellList[state]->next = current;
-	lp->ncells[state]++;
-}
-
 #ifdef DEBUG
 static void
 print_state(ModeInfo * mi, int state)
@@ -909,21 +900,75 @@ free_state(loopstruct * lp, int state)
 }
 
 static void
+free_list(loopstruct * lp)
+{
+	int         state;
+
+	for (state = 0; state < COLORS; state++)
+		free_state(lp, state);
+}
+
+static void
+free_loop(Display *display, loopstruct * lp)
+{
+	int         shade;
+
+	for (shade = 0; shade < lp->init_bits; shade++)
+		if (lp->pixmaps[shade] != None) {
+			XFreePixmap(display, lp->pixmaps[shade]);
+			lp->pixmaps[shade] = None;
+		}
+	if (lp->stippledGC != None) {
+		XFreeGC(display, lp->stippledGC);
+		lp->stippledGC = None;
+	}
+	if (lp->oldcells != NULL) {
+		(void) free((void *) lp->oldcells);
+		lp->oldcells = NULL;
+	}
+	if (lp->newcells != NULL) {
+		(void) free((void *) lp->newcells);
+		lp->newcells = NULL;
+	}
+	free_list(lp);
+}
+
+static Bool
+addtolist(ModeInfo * mi, int col, int row, unsigned char state)
+{
+	loopstruct *lp = &loops[MI_SCREEN(mi)];
+	CellList   *current = lp->cellList[state];
+
+	if ((lp->cellList[state] = (CellList *) malloc(sizeof (CellList))) ==
+			NULL) {
+		lp->cellList[state] = current;
+		free_loop(MI_DISPLAY(mi), lp);
+		return False;
+	}
+	lp->cellList[state]->pt.x = col;
+	lp->cellList[state]->pt.y = row;
+	lp->cellList[state]->next = current;
+	lp->ncells[state]++;
+	return True;
+}
+
+static Bool
 draw_state(ModeInfo * mi, int state)
 {
 	loopstruct *lp = &loops[MI_SCREEN(mi)];
+	Display    *display = MI_DISPLAY(mi);
 	GC          gc;
 	XGCValues   gcv;
 	CellList   *current = lp->cellList[state];
 
 	if (MI_NPIXELS(mi) >= COLORS) {
 		gc = MI_GC(mi);
-		XSetForeground(MI_DISPLAY(mi), gc, lp->colors[state]);
+		XSetForeground(display, gc, lp->colors[state]);
 	} else {
 		gcv.stipple = lp->pixmaps[state];
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
-		XChangeGC(MI_DISPLAY(mi), lp->stippledGC,
+		XChangeGC(display, lp->stippledGC,
 			  GCStipple | GCForeground | GCBackground, &gcv);
 		gc = lp->stippledGC;
 	}
@@ -938,21 +983,22 @@ draw_state(ModeInfo * mi, int state)
 			lp->shape.hexagon[0].x = lp->xb + ccol * lp->xs;
 			lp->shape.hexagon[0].y = lp->yb + crow * lp->ys;
 			if (lp->xs == 1 && lp->ys == 1)
-				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				XDrawPoint(display, MI_WINDOW(mi), gc,
 					lp->shape.hexagon[0].x, lp->shape.hexagon[0].y);
 			else
-				XFillPolygon(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
+				XFillPolygon(display, MI_WINDOW(mi), gc,
 				  	lp->shape.hexagon, 6, Convex, CoordModePrevious);
 			current = current->next;
 		}
 	} else {
 		/* Take advantage of XFillRectangles */
-		XRectangle *rects = NULL;
+		XRectangle *rects;
 		int         nrects = 0;
 
 		/* Create Rectangle list from part of the cellList */
-		if ((rects = (XRectangle *) malloc(lp->ncells[state] * sizeof (XRectangle))) == NULL) {
-			return;
+		if ((rects = (XRectangle *) malloc(lp->ncells[state] *
+				sizeof (XRectangle))) == NULL) {
+			return False;
 		}
 
 		while (current) {
@@ -964,15 +1010,16 @@ draw_state(ModeInfo * mi, int state)
 			nrects++;
 		}
 		/* Finally get to draw */
-		XFillRectangles(MI_DISPLAY(mi), MI_WINDOW(mi), gc, rects, nrects);
+		XFillRectangles(display, MI_WINDOW(mi), gc, rects, nrects);
 		/* Free up rects list and the appropriate part of the cellList */
 		(void) free((void *) rects);
 	}
 	free_state(lp, state);
-	XFlush(MI_DISPLAY(mi));
+	XFlush(display);
+	return True;
 }
 
-static int
+static Bool
 init_table()
 {
 	if (table == NULL) {
@@ -988,7 +1035,7 @@ init_table()
 			mult *= 8;
 
 		if ((table = (unsigned int *) calloc(mult, sizeof (unsigned int))) == NULL) {
-			return 1;
+			return False;
 		}
 
 
@@ -1013,7 +1060,7 @@ init_table()
 				for (k = 0; k < local_neighbors; k++) {
 					TRANSITION(tt, n[k]);
 				}
-				TRANSITION(tt, c);
+				FINALTRANSITION(tt, c);
 				HEX_TABLE_IN(c, n[0], n[1], n[2], n[3], n[4], n[5], i);
 				HEX_TABLE_IN(c, n[1], n[2], n[3], n[4], n[5], n[0], i);
 				HEX_TABLE_IN(c, n[2], n[3], n[4], n[5], n[0], n[1], i);
@@ -1028,7 +1075,7 @@ init_table()
 				for (k = 0; k < local_neighbors; k++) {
 					TRANSITION(tt, n[k]);
 				}
-				TRANSITION(tt, c);
+				FINALTRANSITION(tt, c);
 				TABLE_IN(c, n[0], n[1], n[2], n[3], i);
 				TABLE_IN(c, n[1], n[2], n[3], n[0], i);
 				TABLE_IN(c, n[2], n[3], n[0], n[1], i);
@@ -1036,7 +1083,7 @@ init_table()
 			}
 		}
 	}
-	return 0;
+	return True;
 }
 
 static void
@@ -1269,36 +1316,14 @@ do_gen(loopstruct * lp)
 	}
 }
 
-static void
-free_list(loopstruct * lp)
-{
-	int         state;
-
-	for (state = 0; state < COLORS; state++)
-		free_state(lp, state);
-}
-
 void
 release_loop(ModeInfo * mi)
 {
 	if (loops != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			loopstruct *lp = &loops[screen];
-			int         shade;
-
-			for (shade = 0; shade < lp->init_bits; shade++)
-				if (lp->pixmaps[shade] != None)
-					XFreePixmap(MI_DISPLAY(mi), lp->pixmaps[shade]);
-			if (lp->stippledGC != None)
-				XFreeGC(MI_DISPLAY(mi), lp->stippledGC);
-			if (lp->oldcells != NULL)
-				(void) free((void *) lp->oldcells);
-			if (lp->newcells != NULL)
-				(void) free((void *) lp->newcells);
-			free_list(lp);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_loop(MI_DISPLAY(mi), &loops[screen]);
 		(void) free((void *) loops);
 		loops = NULL;
 	}
@@ -1329,9 +1354,9 @@ init_loop(ModeInfo * mi)
 	if ((MI_NPIXELS(mi) < COLORS) && (lp->init_bits == 0)) {
 		if (lp->stippledGC == None) {
 			gcv.fill_style = FillOpaqueStippled;
-			lp->stippledGC = XCreateGC(display, window, GCFillStyle, &gcv);
-			if (lp->stippledGC == None) {
-				release_loop(mi);
+			if ((lp->stippledGC = XCreateGC(display, window,
+				 GCFillStyle, &gcv)) == None) {
+				free_loop(display, lp);
 				return;
 			}
 		}
@@ -1343,10 +1368,6 @@ init_loop(ModeInfo * mi)
 		LOOPBITS(stipples[7], STIPPLESIZE, STIPPLESIZE);
 		LOOPBITS(stipples[8], STIPPLESIZE, STIPPLESIZE);
 		LOOPBITS(stipples[10], STIPPLESIZE, STIPPLESIZE);
-  	if (lp->pixmaps[COLORS - 1] == None) {
-			release_loop(mi);
-			return;
-		}
 	}
 	if (MI_NPIXELS(mi) >= COLORS) {
 		/* Maybe these colors should be randomized */
@@ -1441,19 +1462,21 @@ init_loop(ModeInfo * mi)
 		(void) free((void *) lp->oldcells);
 		lp->oldcells = NULL;
 	}
-	if ((lp->oldcells = (unsigned char *) calloc(lp->bncols * lp->bnrows, sizeof (unsigned char))) == NULL) {
-		release_loop(mi);
+	if ((lp->oldcells = (unsigned char *) calloc(lp->bncols * lp->bnrows,
+			sizeof (unsigned char))) == NULL) {
+		free_loop(display, lp);
 		return;
 	}
 	if (lp->newcells != NULL) {
 		(void) free((void *) lp->newcells);
 		lp->newcells = NULL;
 	}
-	if ((lp->newcells = (unsigned char *) calloc(lp->bncols * lp->bnrows, sizeof (unsigned char))) == NULL) {
-		release_loop(mi);
+	if ((lp->newcells = (unsigned char *) calloc(lp->bncols * lp->bnrows,
+			sizeof (unsigned char))) == NULL) {
+		free_loop(display, lp);
 		return;
 	}
-	if (init_table()) {
+	if (!init_table()) {
 		release_loop(mi);
 		return;
 	}
@@ -1463,14 +1486,16 @@ init_loop(ModeInfo * mi)
 void
 draw_loop(ModeInfo * mi)
 {
-	loopstruct *lp = &loops[MI_SCREEN(mi)];
 	int         offset, i, j;
 	unsigned char *z, *znew;
+	loopstruct *lp;
 
-	if (loops == NULL) {
-		init_loop(mi);
+	if (loops == NULL)
 		return;
-	}
+	lp = &loops[MI_SCREEN(mi)];
+	if (lp->newcells == NULL)
+		return;
+
 	MI_IS_DRAWN(mi) = True;
 	lp->dead = True;
 #ifdef DELAYDEBUGLOOP
@@ -1487,7 +1512,8 @@ draw_loop(ModeInfo * mi)
 			if (*z != *znew) {
 				lp->dead = False;
 				*z = *znew;
-				addtolist(mi, i - lp->bx, j - lp->by, *znew);
+				if (!addtolist(mi, i - lp->bx, j - lp->by, *znew))
+					return;
 				if (i == lp->mincol && i > lp->bx)
 					lp->mincol--;
 				if (j == lp->minrow && j > lp->by)
@@ -1500,7 +1526,10 @@ draw_loop(ModeInfo * mi)
 		}
 	}
 	for (i = 0; i < COLORS; i++)
-		draw_state(mi, i);
+		if (!draw_state(mi, i)) {
+			free_loop(MI_DISPLAY(mi), lp);
+			return;
+		}
 	if (++lp->generation > MI_CYCLES(mi) || lp->dead) {
 		init_loop(mi);
 		return;
@@ -1525,12 +1554,12 @@ draw_loop(ModeInfo * mi)
 void
 refresh_loop(ModeInfo * mi)
 {
-	loopstruct *lp = &loops[MI_SCREEN(mi)];
+	loopstruct *lp;
 
-	if (loops == NULL) {
-		init_loop(mi);
+	if (loops == NULL)
 		return;
-	}
+	lp = &loops[MI_SCREEN(mi)];
+
 	MI_CLEARWINDOW(mi);
 	lp->redrawing = 1;
 	lp->redrawpos = lp->by * lp->ncols + lp->bx;
