@@ -2,7 +2,7 @@
 /* random --- run random modes for a certain duration */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)random.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)random.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -20,16 +20,17 @@ static const char sccsid[] = "@(#)random.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Made more compatible with xscreensaver :)
- * 18-Mar-96: Ron Hitchens <ron@idiom.com>
- *		Re-coded for the ModeInfo calling scheme.  Added the
- *		change hook.  Get ready for 3.8 release.
- * 23-Dec-95: Ron Hitchens <ron@idiom.com>
- *		Re-coded pickMode() to keep track of the modes, so
- *		that all modes are tried before there are any repeats.
- *		Also prevent a mode from being picked twice in a row
- *		(could happen as first pick after refreshing the list).
- * 04-Sep-95: Written by Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Made more compatible with xscreensaver :)
+ * 18-Mar-1996: Ron Hitchens <ron@idiom.com>
+ *              Re-coded for the ModeInfo calling scheme.  Added the
+ *              change hook.  Get ready for 3.8 release.
+ * 23-Dec-1995: Ron Hitchens <ron@idiom.com>
+ *              Re-coded pickMode() to keep track of the modes, so
+ *              that all modes are tried before there are any repeats.
+ *              Also prevent a mode from being picked twice in a row
+ *              (could happen as first pick after refreshing the list).
+ * 04-Sep-1995: Written by Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
  *
  */
 
@@ -144,10 +145,13 @@ static char gl_modes[][MAXMODECHARS] =
 	"bubble3d",
 #endif
 #ifdef MODE_cage
-  "cage",
+	"cage",
+#endif
+#ifdef MODE_fire
+	"fire",
 #endif
 #ifdef MODE_gears
-  "gears",
+	"gears",
 #endif
 #ifdef MODE_invert
 	"invert",
@@ -167,6 +171,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_rubik
 	"rubik",
 #endif
+#ifdef MODE_sballs
+	"sballs",
+#endif
 #ifdef MODE_skewb
 	"skewb",
 #endif
@@ -197,9 +204,6 @@ static char xpm_modes[][MAXMODECHARS] =
 #ifdef MODE_bat
 	"bat",
 #endif
-#ifdef MODE_cartoon
-	"cartoon",
-#endif
 #ifdef MODE_image
 	"image",
 #endif
@@ -367,7 +371,7 @@ static char nice_modes[][MAXMODECHARS] =
 	"sierpinski",
 #endif
 #ifdef MODE_solitare
-	"spline",
+	"solitare",
 #endif
 #ifdef MODE_spline
 	"spline",
@@ -432,17 +436,28 @@ static char use3d_modes[][MAXMODECHARS] =
 
 static char mouse_modes[][MAXMODECHARS] =
 {
+#ifndef DISABLE_INTERACTIVE
+#ifdef MODE_solitare
+	"solitare",
+#endif
+#ifdef MODE_tetris
+	"tetris",
+#endif
+#endif
 #ifdef MODE_eyes
 	"eyes",
 #endif
+#ifdef MODE_fire
+	"fire",
+#endif
 #ifdef MODE_julia
 	"julia",
 #endif
-#ifdef MODE_solitare
-	"solitare",
+#ifdef MODE_swarm
+	"swarm",
 #endif
 #ifdef MODE_swarm
-	"swarm"
+	"t3d"
 #endif
 };
 
@@ -583,6 +598,9 @@ static char geometry_modes[][MAXMODECHARS] =
 #ifdef MODE_penrose
 	"penrose",
 #endif
+#ifdef MODE_pentominoes
+	"pentominoes",
+#endif
 #ifdef MODE_petal
 	"petal",
 #endif
@@ -652,8 +670,8 @@ pickMode(void)
 	int         mode, i;
 
 	if (mode_indexes == NULL) {
-		if ((mode_indexes = (int *) calloc(nmodes, sizeof (int))) ==
-		    NULL) {
+		if ((mode_indexes = (int *) calloc(nmodes,
+				sizeof (int))) == NULL) {
 			if (sequential)
 				return modes[0];
 			else
@@ -684,8 +702,7 @@ pickMode(void)
 	return (last_mode = mode);	/* remember last mode picked */
 }
 
-static
-char       *
+static char *
 strpmtok(int *sign, char *str)
 {
 	static int  nextsign = 0;
@@ -748,15 +765,16 @@ strpmtok(int *sign, char *str)
 	return r;
 }
 
-static void
+static Bool
 parsemodelist(ModeInfo * mi)
 {
 	int         i, sign = 1, j, found;
 	char       *p;
 
-	modes = (int *) calloc(numprocs - 1, sizeof (int));
-
-	p = strpmtok(&sign, (modelist) ? modelist : "");
+	if ((modes = (int *) calloc(numprocs - 1, sizeof (int))) == NULL) {
+			return False;
+	}
+	p = strpmtok(&sign, (modelist) ? modelist : (char *) "");
 
 	while (p) {
 		if (!strcmp(p, "all")) {
@@ -855,10 +873,10 @@ parsemodelist(ModeInfo * mi)
 			(void) fprintf(stderr, "%d ", modes[i]);
 		(void) fprintf(stderr, "\n");
 	}
+	return True;
 }
 
-static
-void
+static void
 setMode(ModeInfo * mi, int newmode)
 {
 	randomstruct *rp = &randoms[MI_SCREEN(mi)];
@@ -901,8 +919,8 @@ init_random(ModeInfo * mi)
 
 	MI_SET_FLAG_STATE(mi, WI_FLAG_FULLRANDOM, fullrandom);
 	if (currentmode < 0) {
-		parsemodelist(mi);
-
+		if (!parsemodelist(mi))
+			return;
 		for (i = startscreen; i < MI_NUM_SCREENS(mi); i++) {
 			(void) XGetGCValues(MI_DISPLAY(mi), MI_GC(mi),
 					    GC_SAVE_VALUES, &(rp->gcvs));
@@ -925,11 +943,17 @@ void
 draw_random(ModeInfo * mi)
 {
 	int         scrn = MI_SCREEN(mi);
-	randomstruct *rp = &randoms[scrn];
 	int         newmode;
 	unsigned long now = seconds();
 	int         has_run = (duration == 0) ? 0 : (int) (now - starttime);
 	static int  do_init = 0;
+	randomstruct *rp;
+
+	if (randoms == NULL)
+		return;
+	rp = &randoms[scrn];
+	if (currentmode < 0)
+		return;
 
 	if ((scrn == startscreen) && do_init) {
 		do_init = 0;
@@ -959,15 +983,18 @@ draw_random(ModeInfo * mi)
 void
 refresh_random(ModeInfo * mi)
 {
+	if (currentmode < 0)
+		return;
 	call_refresh_hook(&LockProcs[currentmode], mi);
 }
 
 void
 change_random(ModeInfo * mi)
 {
+	if (currentmode < 0)
+		return;
 	if (MI_SCREEN(mi) == startscreen)
 		change_now = True;	/* force a change on next draw callback */
-
 	draw_random(mi);
 }
 
