@@ -2,7 +2,7 @@
 /* rotor --- a swirly rotor */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)rotor.c	4.07 97/11/24 xlockmore";
+static const char sccsid[] = "@(#)rotor.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -22,12 +22,14 @@ static const char sccsid[] = "@(#)rotor.c	4.07 97/11/24 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
- * 10-May-97: Compatible with xscreensaver
- * 08-Mar-95: CAT stuff for ## was tripping up some C compilers.  Removed.
- * 01-Dec-93: added patch for AIXV3 from Tom McConnell
- *            <tmcconne@sedona.intel.com>
- * 11-Nov-90: put into xlock by Steve Zellers <zellers@sun.com>
- * 16-Oct-90: Received from Tom Lawrence (tcl@cs.brown.edu: 'flight' simulator)
+ * 01-Nov-2000: Allocation checks
+ * 10-May-1997: Compatible with xscreensaver
+ * 08-Mar-1995: CAT stuff for ## was tripping up some C compilers.  Removed.
+ * 01-Dec-1993: added patch for AIXV3 from Tom McConnell
+ *              <tmcconne@sedona.intel.com>
+ * 11-Nov-1990: put into xlock by Steve Zellers <zellers@sun.com>
+ * 16-Oct-1990: Received from Tom Lawrence (tcl@cs.brown.edu: 'flight'
+ *               simulator)
  */
 
 #ifdef STANDALONE
@@ -35,7 +37,7 @@ static const char sccsid[] = "@(#)rotor.c	4.07 97/11/24 xlockmore";
 #define HACK_INIT init_rotor
 #define HACK_DRAW draw_rotor
 #define rotor_opts xlockmore_opts
-#define DEFAULTS "*delay: 500 \n" \
+#define DEFAULTS "*delay: 100 \n" \
  "*count: 4 \n" \
  "*cycles: 100 \n" \
  "*size: -6 \n" \
@@ -55,7 +57,7 @@ ModeSpecOpt rotor_opts =
 ModStruct   rotor_description =
 {"rotor", "init_rotor", "draw_rotor", "release_rotor",
  "refresh_rotor", "init_rotor", NULL, &rotor_opts,
- 500, 4, 100, -6, 64, 0.3, "",
+ 100, 4, 100, -6, 64, 0.3, "",
  "Shows Tom's Roto-Rooter", 0, NULL};
 
 #endif
@@ -104,13 +106,26 @@ typedef struct {
 
 static rotorstruct *rotors = NULL;
 
+static void
+free_rotor(rotorstruct *rp)
+{
+	if (rp->elements != NULL) {
+		(void) free((void *) rp->elements);
+		rp->elements = NULL;
+	}
+	if (rp->save != NULL) {
+		(void) free((void *) rp->save);
+		rp->save = NULL;
+	}
+}
+
 void
 init_rotor(ModeInfo * mi)
 {
-	rotorstruct *rp;
 	int         x;
 	elem       *pelem;
 	unsigned char wasiconified;
+	rotorstruct *rp;
 
 	if (rotors == NULL) {
 		if ((rotors = (rotorstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -158,12 +173,20 @@ init_rotor(ModeInfo * mi)
 			}
 		}
 		if (rp->elements == NULL)
-			rp->elements = (elem *) calloc(rp->num, sizeof (elem));
+			if ((rp->elements = (elem *) calloc(rp->num,
+					sizeof (elem))) == NULL) {
+				free_rotor(rp);
+				return;
+			}
 		rp->nsave = MI_CYCLES(mi);
 		if (rp->nsave <= 1)
 			rp->nsave = 2;
 		if (rp->save == NULL)
-			rp->save = (XPoint *) malloc(rp->nsave * sizeof (XPoint));
+			if ((rp->save = (XPoint *) malloc(rp->nsave *
+					sizeof (XPoint))) == NULL) {
+				free_rotor(rp);
+				return;
+			}
 		for (x = 0; x < rp->nsave; x++) {
 			rp->save[x].x = rp->centerx;
 			rp->save[x].y = rp->centery;
@@ -206,14 +229,19 @@ draw_rotor(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
 	GC          gc = MI_GC(mi);
-	register rotorstruct *rp = &rotors[MI_SCREEN(mi)];
 	register elem *pelem;
 	int         thisx, thisy;
 	int         i;
 	int         x_1, y_1, x_2, y_2;
+	rotorstruct *rp;
+
+	if (rotors == NULL)
+		return;
+	rp = &rotors[MI_SCREEN(mi)];
+	if (rp->elements == NULL)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
-
 	if (!rp->iconifiedscreen) {
 		thisx = rp->centerx;
 		thisy = rp->centery;
@@ -343,14 +371,8 @@ release_rotor(ModeInfo * mi)
 	if (rotors != NULL) {
 		int         screen;
 
-		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			rotorstruct *rp = &rotors[screen];
-
-			if (rp->elements != NULL)
-				(void) free((void *) rp->elements);
-			if (rp->save != NULL)
-				(void) free((void *) rp->save);
-		}
+		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+			free_rotor(&rotors[screen]);
 		(void) free((void *) rotors);
 		rotors = NULL;
 	}
@@ -359,7 +381,11 @@ release_rotor(ModeInfo * mi)
 void
 refresh_rotor(ModeInfo * mi)
 {
-	rotorstruct *rp = &rotors[MI_SCREEN(mi)];
+	rotorstruct *rp;
+
+	if (rotors == NULL)
+		return;
+	rp = &rotors[MI_SCREEN(mi)];
 
 	MI_CLEARWINDOW(mi);
 	rp->redrawing = 1;
