@@ -2,7 +2,7 @@
 /* solitare --- Shows a pointless game, used in the Mandarin Candidate */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)solitare.cc	1.0 99/12/10 xlockmore";
+static const char sccsid[] = "@(#)solitare.cc	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -33,6 +33,7 @@ static const char sccsid[] = "@(#)solitare.cc	1.0 99/12/10 xlockmore";
  *   http://wildsav.idv.uni-linz.ac.at/mfx/psol-cardsets/index.html
  *
  * Revision History:
+ * 01-Nov-2000: Allocation checks
  * 17-Jan-2000: autoplay, trackmouse, resizing, release
  * 10-Dec-1999: wanted to do some C++, may end up using GL but not for now.
  *
@@ -84,22 +85,30 @@ enum Suits {spade, diamond, club, heart};
 #ifdef FR
 // Ace => As, Jack => Valet,
 // Queen = Reine (Dame), King = Roi, Joker = Joker
-static char *RankNames[] = {NULL, "A", "2", "3", "4", "5", "6", "7",
-	"8", "9", "10", "V", "D", "R"};
+static char *RankNames[] = {NULL, (char *) "A",
+	(char *) "2", (char *) "3", (char *) "4", (char *) "5", (char *) "6",
+	(char *) "7", (char *) "8", (char *) "9", (char *) "10",
+	(char *) "V", (char *) "D", (char *) "R"};
 #else
 #ifdef NL
 // Ace => Aas, Jack => Boer (Farmer),
 // Queen = Vrouw (Lady), King = Heer (Gentleman), Joker = Joker
-static char *RankNames[] = {NULL, "A", "2", "3", "4", "5", "6", "7",
-	"8", "9", "10", "B", "V", "H"};
+static char *RankNames[] = {NULL, (char *) "A",
+	(char *) "2", (char *) "3", (char *) "4", (char *) "5", (char *) "6",
+	(char *) "7", (char *) "8", (char *) "9", (char *) "10",
+	(char *) "B", (char *) "V", (char *) "H"};
 #else
 #ifdef RU
 // Some may be approximations to Cyrillic
-static char *RankNames[] = {NULL, "T", "2", "3", "4", "5", "6", "7",
-	"8", "9", "10", "B", "Q", "K"};
+static char *RankNames[] = {NULL, (char *) "T",
+	(char *) "2", (char *) "3", (char *) "4", (char *) "5", (char *) "6",
+	(char *) "7", (char *) "8", (char *) "9", (char *) "10",
+	(char *) "B", (char *) "Q", (char *) "K"};
 #else
-static char *RankNames[] = {NULL, "A", "2", "3", "4", "5", "6", "7",
-	"8", "9", "10", "J", "Q", "K"};
+static char *RankNames[] = {NULL, (char *) "A",
+	(char *) "2", (char *) "3", (char *) "4", (char *) "5", (char *) "6",
+	(char *) "7", (char *) "8", (char *) "9", (char *) "10",
+	(char *) "J", (char *) "Q", (char *) "K"};
 #endif
 #endif
 #endif
@@ -265,7 +274,9 @@ CardView::CardView(ModeInfo * a_mi, Card * a_card)
 CardView::CardView(ModeInfo * a_mi, Suits suit, int rank)
 {
 	this->mi = a_mi;
-	this->card = new Card(suit, rank);
+	if ((this->card = new Card(suit, rank)) == NULL) {
+		return;
+	}
 	faceUp = False;
 	locationX = locationY = 0;
 }
@@ -407,7 +418,7 @@ public:
 	virtual Bool canTake(Card *);
 	Bool contains(int, int); // mouse interaction
 	virtual void displayPile();
-	virtual void initialize();
+	virtual Bool initialize();
 	virtual void cleanup();
 	CardLink * removeCard();
 	virtual Bool select(int, int); // mouse interaction
@@ -428,7 +439,7 @@ class DealPile : public CardPile
 public:
 	DealPile(ModeInfo *);
 	virtual void cleanup();
-	void shuffle(CardPile *);
+	Bool shuffle(CardPile *);
 };
 inline DealPile::DealPile(ModeInfo * a_mi) : CardPile(a_mi) { ; }
 
@@ -453,7 +464,7 @@ public:
 	virtual void displayPile();
 	virtual Bool select(int, int);
 	virtual Bool select(Bool);
-	virtual void initialize();
+	virtual Bool initialize();
 private:
 	int pile;
 };
@@ -464,7 +475,7 @@ public:
 	DeckPile(ModeInfo * a_mi) : CardPile(a_mi) { ; }
 	virtual void updateLocation(ModeInfo *);
 	virtual void addCard(CardLink *);
-	virtual void initialize();
+	virtual Bool initialize();
 	virtual Bool select(int, int);
 	virtual Bool select(Bool);
 };
@@ -485,7 +496,7 @@ class GameTable
 public:
 	GameTable(ModeInfo *);
 	~GameTable();
-	void newGame(ModeInfo *);
+	Bool newGame(ModeInfo *);
 	Bool suitCanAdd(Card *);
 	CardPile * suitAddPile(Card *);
 	Bool alternateCanAdd(Card *);
@@ -576,20 +587,21 @@ void CardPile::displayPile()
 	}
 }
 
-void CardPile::initialize()
+Bool CardPile::initialize()
 {
 	top = nilLink;
+	return True;
 }
 
 void CardPile::cleanup()
 {
 	CardLink *p;
 
-  while (top != nilLink) {
-    p = top;
+	while (top != nilLink) {
+  		p = top;
 		top = top->nextCard();
-    delete p;
-   }
+		delete p;
+	}
 }
 
 CardLink * CardPile::removeCard()
@@ -627,7 +639,7 @@ void DealPile::cleanup()
    }
 }
 
-void DealPile::shuffle(CardPile * pile)
+Bool DealPile::shuffle(CardPile * pile)
 {
 	CardLink *p, *q;
 	int max, limit, i;
@@ -648,10 +660,13 @@ void DealPile::shuffle(CardPile * pile)
 			if (i <= limit)
 				p = p->nextCard();
 		}
-		q = new CardLink(mi, p->thisCard());
+		if ((q = new CardLink(mi, p->thisCard())) == NULL) {
+			return False;
+		}
 		pile->addCard(q);
 		p->flip();
 	}
+	return True;
 }
 
 void DeckPile::updateLocation(ModeInfo * a_mi)
@@ -670,12 +685,14 @@ void DeckPile::updateLocation(ModeInfo * a_mi)
 	}
 }
 
-void DeckPile::initialize()
+Bool DeckPile::initialize()
 {
 	solitarestruct *bp = &solitare[MI_SCREEN(mi)];
 
 	CardPile::initialize();
-	bp->game->dealPile->shuffle(this);
+	if (!bp->game->dealPile->shuffle(this))
+    	return False;
+	return True;
 }
 
 void DeckPile::addCard(CardLink *c)
@@ -816,9 +833,9 @@ void AlternatePile::updateLocation(ModeInfo * a_mi)
 		if (p->isFaceUp())
 			bf = p;
 	}
-  if (bf != nilLink) {
+	if (bf != nilLink) {
 		// ok but this is wrong.  Since we only have a singly link
-		// we will do this slightly ineffiently...
+		// we will do this slightly inefficiently...
 		ty = y;
 		while (bf != top) {
 			CardLink *sp;
@@ -830,19 +847,20 @@ void AlternatePile::updateLocation(ModeInfo * a_mi)
 	}
 }
 
-void AlternatePile::initialize()
+Bool AlternatePile::initialize()
 {
 	solitarestruct *bp = &solitare[MI_SCREEN(mi)];
 	int a_pile;
 
 	//put the right number of cards on the alternate
-	CardPile::initialize();
+	(void) CardPile::initialize();
 	for (a_pile = 0; a_pile <= this->pile; a_pile++)
 		addCard((bp->game->deckPile)->removeCard());
 	// flip the last one
 	if (top != nilLink) {
 		top->flip();
 	}
+	return True;
 }
 
 void AlternatePile::addCard(CardLink * card)
@@ -1011,28 +1029,51 @@ GameTable::GameTable(ModeInfo *mi)
 	bp->cardwidth = bp->width / (MAXALTERNATEPILES + 1);
 	bp->cardheight = bp->height / 5;
 	// Create the original (unshuffled) deck (with no cards) ... dah daaah!
-	dealPile = new DealPile(mi);
+	if ((dealPile = new DealPile(mi)) == NULL) {
+		return;
+	}
 	for (suit = 0; suit < MAXSUITS; suit++)
 		for (rank = 1; rank <= MAXRANK; rank++) {
+            CardLink * deal;
 			// 52 pickup, creating new cards and creating links
-			dealPile->addCard(new CardLink(mi, Suits(suit), rank));
+			if ((deal = new CardLink(mi, Suits(suit), rank)) == NULL) {
+				dealPile->cleanup();
+				delete dealPile;
+				return;
+			}
+			dealPile->addCard(deal);
 		}
 	// create 2 piles the deck and discard piles
-	allPiles[0] = deckPile = new DeckPile(mi);
+	if ((allPiles[0] = deckPile = new DeckPile(mi)) == NULL) {
+		dealPile->cleanup();
+		delete dealPile;
+		return;
+	}
 	deckPile->updateLocation(mi);
-	allPiles[1] = discardPile = new DiscardPile(mi);
+	if ((allPiles[1] = discardPile = new DiscardPile(mi)) == NULL) {
+		dealPile->cleanup();
+		delete dealPile;
+		delete deckPile;
+		return;
+	}
 	discardPile->updateLocation(mi);
 
 	// create suit piles
 	for (suit = 0; suit < MAXSUITS; suit++) {
-		allPiles[suit + 2] = suitPiles[Suits(suit)] = new SuitPile(mi, suit);
+		if ((allPiles[suit + 2] = suitPiles[Suits(suit)] = new SuitPile(mi, suit)) == NULL) {
+			delete this;
+			return;
+		}
 		(suitPiles[Suits(suit)])->updateLocation(mi);
 	}
 
 	// create alternate piles
 	for (pile = 0; pile < MAXALTERNATEPILES; pile++) {
-		allPiles[pile + 2 + MAXSUITS] = alternatePiles[pile] =
-			new AlternatePile(mi, pile);
+		if ((allPiles[pile + 2 + MAXSUITS] = alternatePiles[pile] =
+			new AlternatePile(mi, pile)) == NULL) {
+			delete this;
+			return;
+		}
 		(alternatePiles[pile])->updateLocation(mi);
 	}
 }
@@ -1073,15 +1114,17 @@ void GameTable::Resize(ModeInfo *mi)
 	}
 }
 
-void GameTable::newGame(ModeInfo * mi)
+Bool GameTable::newGame(ModeInfo * mi)
 {
 	int pile;
 
 	// initialize all the piles
 	for (pile = 0; pile < MAXPILES; pile++)
-		allPiles[pile]->initialize();
+		if (!allPiles[pile]->initialize())
+			return False;
 	// redraw the game window
 	Redraw(mi);
+	return True;
 }
 
 void GameTable::Redraw(ModeInfo * mi)
@@ -1254,7 +1297,8 @@ init_solitare(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 #ifdef DOFONT
 	Display    *display = MI_DISPLAY(mi);
-	 XGCValues gcv;
+	XGCValues gcv;
+
 	 if (mode_font == None)
 		mode_font = getFont(display);
 	  if (mode_font != None) {
@@ -1263,8 +1307,11 @@ init_solitare(ModeInfo * mi)
 			gcv.graphics_exposures = False;
 			gcv.foreground = MI_WHITE_PIXEL(mi);
 			gcv.background = MI_BLACK_PIXEL(mi);
-			mp->gc = XCreateGC(display, MI_WINDOW(mi),
-			  GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
+			if ((mp->gc = XCreateGC(display, MI_WINDOW(mi),
+			  GCForeground | GCBackground | GCGraphicsExposures | GCFont,
+					&gcv)) == None) {
+				return;
+			}
 			mp->ascent = mode_font->ascent;
 			mp->height = font_height(mode_font);
 			for (i = 0; i < 256; i++)
@@ -1275,15 +1322,19 @@ init_solitare(ModeInfo * mi)
 		if (bp->width != MI_WIDTH(mi) || bp->height != MI_HEIGHT(mi)) {
 			/* Let us not be creative then... */
 			bp->game->Resize(mi);
-	    bp->painted = True;
+			bp->painted = True;
 			refresh_solitare(mi);
 			return;
 		}
 		delete bp->game;
 	}
 	bp->painted = False;
-	bp->game = new GameTable(mi);
-	bp->game->newGame(mi);
+	if ((bp->game = new GameTable(mi)) == NULL) {
+		return;
+	}
+	if (!bp->game->newGame(mi)) {
+		delete bp->game;
+	}
 }
 
 /*
@@ -1294,7 +1345,13 @@ init_solitare(ModeInfo * mi)
 void
 draw_solitare(ModeInfo * mi)
 {
-	solitarestruct *bp = &solitare[MI_SCREEN(mi)];
+	solitarestruct *bp;
+
+	if (solitare == NULL)
+		return;
+	bp = &solitare[MI_SCREEN(mi)];
+	if (!bp->game)
+		return;
 
 	MI_IS_DRAWN(mi) = True;
 	bp->painted = True;
@@ -1306,8 +1363,9 @@ draw_solitare(ModeInfo * mi)
 		if (!bp->game->HandleMouse(mi)) {
 			bp->showend++;
 		}
-	} else if (!bp->game->HandleGenerate())
-			bp->showend++;
+	} else if (!bp->game->HandleGenerate()) {
+		bp->showend++;
+	}
 }
 
 /*
@@ -1323,8 +1381,7 @@ release_solitare(ModeInfo * mi)
 {
 	if (solitare != NULL) {
 		for (int screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-			Display    *display = MI_DISPLAY(mi);
-	    solitarestruct *bp = &solitare[screen];
+			solitarestruct *bp = &solitare[screen];
 
 			if (bp->game)
 				delete bp->game;
@@ -1337,7 +1394,13 @@ release_solitare(ModeInfo * mi)
 void
 refresh_solitare(ModeInfo * mi)
 {
-	solitarestruct *bp = &solitare[MI_SCREEN(mi)];
+	solitarestruct *bp;
+
+	if (solitare == NULL)
+		return;
+	bp = &solitare[MI_SCREEN(mi)];
+	if (!bp->game)
+		return;
 
 	if (bp->painted) {
 		bp->game->Redraw(mi);
