@@ -2,7 +2,7 @@
 /* tetris --- autoplaying tetris game */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)tetris.c  4.13 98/10/01 xlockmore";
+static const char sccsid[] = "@(#)tetris.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -54,10 +54,13 @@ static const char sccsid[] = "@(#)tetris.c  4.13 98/10/01 xlockmore";
  *       on tetris
  *
  * Revision History:
+ * 01-Dec-2000: Lifted code from pentominoes variable size pieces
+ *              Copyright (c) 2000 by Stephen Montgomery-Smith
+ * 01-Nov-2000: Allocation checks
  * 02-Aug-2000: alwelltris trackmouse implemented
  * 27-Jul-2000: alwelltris all but trackmouse
  * 22-Jun-2000: trackmouse added, row clearing fixed, bonus pieces,
- * 01-Nov-1998: altetris additions
+ * 01-Nov-1998: altetris additions by David Bagley
  * 01-Oct-1998: Created
  */
 
@@ -134,53 +137,14 @@ ModeSpecOpt tetris_opts =
 ModStruct   tetris_description =
 {"tetris", "init_tetris", "draw_tetris", "release_tetris",
  "refresh_tetris", "change_tetris", NULL, &tetris_opts,
- 600000, -40, 200, 0, 64, 1.0, "",
+ 600000, -40, 200, -100, 64, 1.0, "",
  "Shows an autoplaying tetris game", 0, NULL};
 
 #endif
 
-#if !defined(BM_3D) && !defined(BM_PLAIN)
-#define BM_3D
-#endif
-#ifdef BM_3D
-#include  "bitmaps/rot00.xbm"
-#include  "bitmaps/rot01.xbm"
-#include  "bitmaps/rot02.xbm"
-#include  "bitmaps/rot03.xbm"
-#include  "bitmaps/rot04.xbm"
-#include  "bitmaps/rot05.xbm"
-#include  "bitmaps/rot06.xbm"
-#include  "bitmaps/rot07.xbm"
-#include  "bitmaps/rot08.xbm"
-#include  "bitmaps/rot09.xbm"
-#include  "bitmaps/rot10.xbm"
-#include  "bitmaps/rot11.xbm"
-#include  "bitmaps/rot12.xbm"
-#include  "bitmaps/rot13.xbm"
-#include  "bitmaps/rot14.xbm"
-#include  "bitmaps/rot15.xbm"
-#else
-#ifdef BM_PLAIN
-#include  "bitmaps/rop00.xbm"
-#include  "bitmaps/rop01.xbm"
-#include  "bitmaps/rop02.xbm"
-#include  "bitmaps/rop03.xbm"
-#include  "bitmaps/rop04.xbm"
-#include  "bitmaps/rop05.xbm"
-#include  "bitmaps/rop06.xbm"
-#include  "bitmaps/rop07.xbm"
-#include  "bitmaps/rop08.xbm"
-#include  "bitmaps/rop09.xbm"
-#include  "bitmaps/rop10.xbm"
-#include  "bitmaps/rop11.xbm"
-#include  "bitmaps/rop12.xbm"
-#include  "bitmaps/rop13.xbm"
-#include  "bitmaps/rop14.xbm"
-#include  "bitmaps/rop15.xbm"
-#endif
-#endif
 #define MINGRIDSIZE 10 
-#define MINSIZE 2
+#define MAXPIXELSIZE 13 
+#define MINSIZE 6 
 #define MAX_SIDES 4
 
 #define NUM_FLASHES 16
@@ -199,47 +163,6 @@ ModStruct   tetris_description =
 #define CHECKDOWN(x) (((x)/4)*4+(x)%2)
 #define EMPTY (-1)
 
-static XImage logo[BITMAPS] =
-{
-#ifdef BM_3D
-  {0, 0, 0, XYBitmap, (char *) rot00_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot01_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot02_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot03_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot04_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot05_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot06_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot07_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot08_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot09_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot10_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot11_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot12_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot13_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot14_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rot15_bits, LSBFirst, 8, LSBFirst, 8, 1}
-#else
-#ifdef BM_PLAIN
-  {0, 0, 0, XYBitmap, (char *) rop00_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop01_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop02_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop03_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop04_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop05_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop06_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop07_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop08_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop09_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop10_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop11_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop12_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop13_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop14_bits, LSBFirst, 8, LSBFirst, 8, 1},
-  {0, 0, 0, XYBitmap, (char *) rop15_bits, LSBFirst, 8, LSBFirst, 8, 1}
-#endif
-#endif
-};
-
 typedef enum {FALL, DROP, LEFT, RIGHT, ROTATE, REFLECT} move_t;
 
 typedef struct {
@@ -291,6 +214,7 @@ typedef struct {
   int   xs, ys;
   int   width, height;
   int   pixelmode;
+  XImage *images[BITMAPS];
   XColor     *colors;
   int   ncolors;
   int   ncols, nrows;
@@ -307,10 +231,24 @@ typedef struct {
   fieldstruct wall[WELL_DEPTH+WELL_WIDTH][WELL_PERIMETER];
   fieldstruct base[WELL_WIDTH][WELL_WIDTH];
 
+  ModeInfo *mi;
 } trisstruct;
 
+#define ROUND8(n) ((((n)+7)/8)*8)
+#define BITNO(x,y) ((x)+(y)*ROUND8(tp->ys))
+#define SETBIT(x,y) {data[BITNO(x,y)/8] |= 1<<(BITNO(x,y)%8);}
+#define RESBIT(x,y) {data[BITNO(x,y)/8] &= ~(1<<(BITNO(x,y)%8));}
+#define HALFBIT(x,y) {if ((x%2)^(y%2)) RESBIT(x,y) else SETBIT(x,y)}
+#define YESSET(b,x,y) {if (b) SETBIT(x,y) else HALFBIT(x,y)}
+#define LEFT(n) ((n)&1)
+#define RIGHT(n) ((n)&2)
+#define UP(n) ((n)&4)
+#define DOWN(n) ((n)&8)
+#define ARR(x,y) (((x)<0||(x)>=tp->width||(y)<0||(y)>=tp->height)?-1:tp->array[x][y])
+ /* Map from Q. Alex Zhao notation to Stephen Montgomery-Smith notation */
+#define Z2MS(z) ((!((z)&1))+((!((z)&2))<<3)+((!((z)&4))<<1)+((!((z)&8))<<2))
+
 static trisstruct *triss = NULL;
-static int icon_width, icon_height;
 
 /* Setup: Normal, Bonus */
 static int area_kinds[] = {3, 3};
@@ -462,10 +400,10 @@ leadingEmptyHeight(Polyominoes * polyo)
   polyo->leadingEmptyHeight = polyo->size;
 }
 
-static void
-readPolyominoes(trisstruct * tp)
+static Bool
+readPolyominoes(void)
 {
-  int   size = 0, n = 0, index = 0, sum = 0;
+  int   size, n, index = 0, sum = 0;
   int   counter, start_counter;
   int   polyomino, kinds, i, j, bonustype;
 
@@ -476,12 +414,12 @@ readPolyominoes(trisstruct * tp)
       size = ominos[index++];
       if (size > omino_squares[bonustype]) {
         (void) printf("tetris corruption: size %d\n", size);
-        return;
+        return False;
       }
       n = ominos[index++];
       if (sum > max_ominoes[bonustype]) {
         (void) printf("tetris corruption: n %d\n", n);
-        return;
+        return False;
       }
       for (polyomino = 0; polyomino < n; polyomino++) {
         if (polyomino + counter < max_ominoes[bonustype]) {
@@ -489,12 +427,12 @@ readPolyominoes(trisstruct * tp)
         } else {
           (void) printf("tetris corruption: bonustype %d, polyomino %d, counter %d\n",
             bonustype, polyomino, counter);
-          return;
+          return False;
         }
         if (polyomino != ominos[index]) {
           (void) printf("tetris corruption: polyomino %d, ominos[index] %d\n",
             polyomino, ominos[index]);
-          return;
+          return False;
         }
         index++;  /* This is only there to "read" it */
         polytris[bonustype].polyomino[sum].rotation =
@@ -508,12 +446,15 @@ readPolyominoes(trisstruct * tp)
           } else {
             (void) printf("tetris corruption: bonustype %d, polyomino %d, start_counter %d\n",
               bonustype, polyomino, start_counter);
-            return;
+            return False;
           }
         }
         polytris[bonustype].polyomino[sum].size = size;
-        polytris[bonustype].polyomino[sum].shape = (int *) malloc(size *
-    size * sizeof(int));
+        if ((polytris[bonustype].polyomino[sum].shape = (int *) malloc(size *
+            size * sizeof(int))) == NULL) {
+          (void) printf("tetris out of memory\n");
+          return False;
+	}
         for (j = 0; j < size; j++) {
           for (i = 0; i < size; i++) {
             polytris[bonustype].polyomino[sum].shape[j * size + i] =
@@ -526,6 +467,7 @@ readPolyominoes(trisstruct * tp)
       counter += n;
     }
   }
+  return True;
 }
 
 #ifdef DEBUG
@@ -560,7 +502,7 @@ writePolyominoes(trisstruct * tp)
 #endif
 
 static void
-setColour(ModeInfo * mi, int cid, int exor) 
+setColour(ModeInfo * mi, int cid /* , int exor */) 
 {
   Display    *display = MI_DISPLAY(mi);
   trisstruct *tp = &triss[MI_SCREEN(mi)];
@@ -589,22 +531,22 @@ drawSquare(ModeInfo * mi, int pmid, int cid, int col, int row)
   Display    *display = MI_DISPLAY(mi);
   Window      window = MI_WINDOW(mi);
   trisstruct *tp = &triss[MI_SCREEN(mi)];
-  long colour;
 
-  setColour(mi, cid, 0);
+  setColour(mi, cid /* , 0 */);
   if (tp->well)
     XFillRectangle(display, window, tp->gc,
       tp->xb + col * tp->xs + 1, tp->yb + row * tp->ys + 1, tp->xs - 2, tp->ys - 2);
   else if (tp->pixelmode)
     XFillRectangle(display, window, tp->gc,
       tp->xb + col * tp->xs, tp->yb + row * tp->ys, tp->xs, tp->ys);
-  else
+  else {
 /*-
  * PURIFY on SunOS4 and on Solaris 2 reports a 120 byte memory leak on
  * the next line */
-    (void) XPutImage(display, window, tp->gc, &logo[pmid], 0, 0,
-      tp->xb + icon_width * col, tp->yb + icon_height * row,
-      icon_width, icon_height);
+    (void) XPutImage(display, window, tp->gc, tp->images[Z2MS(pmid)], 0, 0,
+      tp->xb + tp->xs * col, tp->yb + tp->ys * row,
+      tp->xs, tp->ys);
+  }
 }
 
 static void
@@ -699,7 +641,6 @@ static void
 clearTrapazoid(ModeInfo * mi, int i, int j)
 {
   Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
   XSetForeground(display, tp->gc, tp->blackpixel);
@@ -707,31 +648,33 @@ clearTrapazoid(ModeInfo * mi, int i, int j)
 }
 
 static void
-drawTrapazoid(ModeInfo * mi, int pmid, int cid, int i, int j)
+drawTrapazoid(ModeInfo * mi, /* int pmid, */ int cid, int i, int j)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
-  setColour(mi, cid, 0);
+  setColour(mi, cid /* , 0 */ );
   drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
 }
 
+#if 0
 static void
-xorTrapazoid(ModeInfo * mi, int pmid, int cid, int i, int j)
+xorTrapazoid(ModeInfo * mi, /* int pmid, */ int cid, int i, int j)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
-  setColour(mi, cid, 1);
+  setColour(mi, cid /* , 1 */ );
   XSetFunction(MI_DISPLAY(mi), tp->gc, GXxor);
   drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
   XSetFunction(MI_DISPLAY(mi), tp->gc, GXcopy);
 }
+#endif
 
 static void
 freezeTrapazoid(ModeInfo * mi, int i, int j)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
 
-  setColour(mi, -1, 1);
+  setColour(mi, -1 /* , 1 */);
   XSetFunction(MI_DISPLAY(mi), tp->gc, GXxor);
   drawWallSquare(mi, tp->gc, i, j); /* pmid not used yet */
   XSetFunction(MI_DISPLAY(mi), tp->gc, GXcopy);
@@ -773,8 +716,6 @@ static void
 dropWall(ModeInfo *mi, int w)
 {
   Display    *display = MI_DISPLAY(mi);
-  Window      window = MI_WINDOW(mi);
-  GC          gc = MI_GC(mi);
   trisstruct *tp = &triss[MI_SCREEN(mi)];
   int i, j, k, l, lines, base_x, base_y;
   Bool fits;
@@ -816,7 +757,7 @@ dropWall(ModeInfo *mi, int w)
               tp->wall[l + j][k] = tp->wall[l][k];
               tp->wall[l][k].pmid = EMPTY;
               clearTrapazoid(mi, k, l);
-              drawTrapazoid(mi, tp->wall[l][k].pmid, tp->wall[l][k].cid, k, l + j);
+              drawTrapazoid(mi, /* tp->wall[l][k].pmid, */ tp->wall[l][k].cid, k, l + j);
             }
             XFlush(display);
           }
@@ -827,7 +768,6 @@ dropWall(ModeInfo *mi, int w)
 static void
 freezeWall(ModeInfo * mi, int w)
 {
-  trisstruct *tp = &triss[MI_SCREEN(mi)];
   int i, j;
 
   for (j = 0; j < WELL_DEPTH; j++)
@@ -837,7 +777,7 @@ freezeWall(ModeInfo * mi, int w)
   XFlush(MI_DISPLAY(mi));
 }
 
-Bool
+static Bool
 allFrozen(ModeInfo * mi)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
@@ -854,7 +794,7 @@ allFrozen(ModeInfo * mi)
   return True;
 }
 
-void
+static void
 checkFreeze(ModeInfo *mi)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
@@ -884,7 +824,7 @@ static void
 drawBox(ModeInfo * mi, int pmid, int cid, int i, int j)
 {
   if (j < WELL_DEPTH) {
-    drawTrapazoid(mi, pmid, cid, i, j);
+    drawTrapazoid(mi, /* pmid, */ cid, i, j);
   } else {
     int base_i, base_j;
 
@@ -961,7 +901,7 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
     for (j = 0; j < tp->curPolyomino.size; j++)
       if ((tp->curPolyomino.ypos + j >= 0) &&
         polytris[(int) tp->bonusNow].polyomino
-          [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i])
+          [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
         if (tp->well) {
           int xi, yj;
 
@@ -975,6 +915,7 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
             [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i],
             tp->curPolyomino.color_number, tp->curPolyomino.xpos + i, tp->curPolyomino.ypos + j);
         }
+      }
 
   for (i = 0; i < tp->curPolyomino.size; i++)
     for (j = 0; j < tp->curPolyomino.size; j++) {
@@ -984,7 +925,7 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
         ((ox < 0) || (ox >= tp->curPolyomino.size) ||
         (oy < 0) || (oy >= tp->curPolyomino.size) ||
         !polytris[(int) tp->bonusNow].polyomino
-          [tp->curPolyomino.polyomino_number].shape[oy * tp->curPolyomino.size + ox]))
+          [tp->curPolyomino.polyomino_number].shape[oy * tp->curPolyomino.size + ox])) {
         if (tp->well) {
           int xi, yj;
 
@@ -994,6 +935,7 @@ drawPolyominoDiff(ModeInfo *mi, thing_t * old)
         } else {
           clearSquare(mi, (old->xpos + i), (old->ypos + j));
         }
+      }
     }
 }
 
@@ -1207,30 +1149,30 @@ atBasePartially(ModeInfo * mi)
 
   for (i = 0; i < tp->curPolyomino.size; i++)
     for (j = 0; j < tp->curPolyomino.size; j++)
-        if (tp->curPolyomino.ypos + j >= WELL_DEPTH &&
+        if ((tp->curPolyomino.ypos + j >= WELL_DEPTH) &&
             polytris[(int) tp->bonusNow].polyomino
               [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i])
           return True;
-     return False;
+  return False;
 }
 
 static Bool
-wallChange(ModeInfo * mi, thing_t old, thing_t new)
+wallChange(ModeInfo *mi, thing_t oldthing, thing_t newthing)
 {
   trisstruct *tp = &triss[MI_SCREEN(mi)];
   int             w = -1, i, j;
   int             x = tp->curPolyomino.xpos;
 
-  for (i = 0; i < old.size; i++)
-    for (j = 0; j < old.size; j++)
-  if (polytris[(int) tp->bonusNow].polyomino[old.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
+  for (i = 0; i < oldthing.size; i++)
+    for (j = 0; j < oldthing.size; j++)
+  if (polytris[(int) tp->bonusNow].polyomino[oldthing.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
     if (w == -1)
       w = ((x + i) % WELL_PERIMETER) / WELL_WIDTH;
     else if (w != ((x + i) % WELL_PERIMETER) / WELL_WIDTH)
       return True;
   }
-  for (i = 0; i < new.size; i++)
-    for (j = 0; j < new.size; j++)
+  for (i = 0; i < newthing.size; i++)
+    for (j = 0; j < newthing.size; j++)
       if (polytris[(int) tp->bonusNow].polyomino
                 [tp->curPolyomino.polyomino_number].shape[j * tp->curPolyomino.size + i]) {
           if (w != ((x + i) % WELL_PERIMETER) / WELL_WIDTH)
@@ -1311,7 +1253,7 @@ tryMove(ModeInfo * mi, move_t move)
   }
 }
 
-void
+static void
 fillLines(ModeInfo *mi)
 {
   Display    *display = MI_DISPLAY(mi);
@@ -1342,6 +1284,53 @@ fillLines(ModeInfo *mi)
   }
 }
 
+static void
+free_images(trisstruct *tp)
+{
+  int i;
+
+  for (i = 0; i < BITMAPS; i++) {
+    if (tp->images[i] != None) {
+      (void) XDestroyImage(tp->images[i]);
+      tp->images[i] = None;
+    }
+  }
+}
+
+static void
+free_tetris(Display *display, trisstruct *tp)
+{
+  ModeInfo *mi = tp->mi;
+
+  if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
+    MI_WHITE_PIXEL(mi) = tp->whitepixel;
+    MI_BLACK_PIXEL(mi) = tp->blackpixel;
+#ifndef STANDALONE
+    MI_FG_PIXEL(mi) = tp->fg;
+    MI_BG_PIXEL(mi) = tp->bg;
+#endif
+    if (tp->colors != NULL) {
+      if (tp->ncolors && !tp->no_colors)
+        free_colors(display, tp->cmap, tp->colors, tp->ncolors);
+      (void) free((void *) tp->colors);
+      tp->colors = NULL;
+    }
+    if (tp->cmap != None) {
+      XFreeColormap(display, tp->cmap);
+      tp->cmap = None;
+    }
+  }
+  if (tp->gc != NULL) {
+     XFreeGC(display, tp->gc);
+     tp->gc = NULL;
+  }
+  if (tp->field != NULL) {
+     (void) free((void *) tp->field);
+     tp->field = NULL;
+  }
+  free_images(tp);
+}
+
 static int
 checkLines(ModeInfo *mi)
 {
@@ -1442,7 +1431,10 @@ checkLines(ModeInfo *mi)
     int      *lSet, nset = 0;
     int       i, j, y;
 
-    lSet = (int *) calloc(tp->nrows, sizeof (int));
+    if ((lSet = (int *) calloc(tp->nrows, sizeof (int))) == NULL) {
+        free_tetris(display, tp);
+        return -1; /* error */
+    }
     for (j = 0; j < tp->nrows; j++) {
       for (i = 0; i < tp->ncols; i++)
         if (tp->field[j * tp-> ncols + i].pmid >= 0)
@@ -1450,7 +1442,6 @@ checkLines(ModeInfo *mi)
       if (lSet[j] == tp->ncols)
         nset++;
     }
-
     if (nset) {
       for (i = 0; i < ((NUM_FLASHES / nset) % 2) * 2; i++) {
         for (j = 0; j < tp->nrows; j++) {
@@ -1464,8 +1455,8 @@ checkLines(ModeInfo *mi)
         if (lSet[j] == tp->ncols) {
           for (y = j; y > 0; y--)
             for (i = 0; i < tp->ncols; i++)
-          tp->field[y * tp->ncols + i] =
-             tp->field[(y - 1) * tp->ncols + i];
+              tp->field[y * tp->ncols + i] =
+                 tp->field[(y - 1) * tp->ncols + i];
           for (i = 0; i < tp->ncols; i++)
             tp->field[i].pmid = EMPTY;
 
@@ -1473,8 +1464,7 @@ checkLines(ModeInfo *mi)
             tp->xb, tp->yb,
             tp->xs * tp->ncols, j * tp->ys,
             tp->xb, tp->yb + tp->ys);
-          XSetBackground(display, tp->gc,
-            MI_BLACK_PIXEL(mi));
+          XSetForeground(display, tp->gc, tp->blackpixel);
           XFillRectangle(display, window, tp->gc,
             tp->xb, tp->yb, tp->xs * tp->ncols, tp->ys);
 
@@ -1526,10 +1516,113 @@ gameOver(ModeInfo *mi)
   fillLines(mi);
 }
 
+static Bool
+create_images(ModeInfo *mi, trisstruct *tp)
+{
+  int x, y, n;
+  unsigned char * data;
+
+  for (n = 0; n < BITMAPS; n++) {
+    if ((data = (unsigned char *) malloc(sizeof(unsigned char)*
+           (tp->ys*ROUND8(tp->xs)/8))) == NULL) {
+        return False;  /* no biggie, will not use images */
+    }
+
+    for (y=0;y<tp->ys;y++) for (x=0;x<tp->xs;x++) {
+      if ((x>=y && x<=tp->ys-y-1 && x<=tp->ys/2)
+          || (x<=y && x<=tp->ys-y-1 && y<=tp->ys/2)
+          || (UP(n)     && x>=y && x<=tp->ys-y-1 && x> tp->ys/2)
+          || (!DOWN(n)  && x<=y && x>=tp->ys-y-1 && x<=tp->ys/2)
+          || (LEFT(n)   && x<=y && x<=tp->ys-y-1 && y> tp->ys/2)
+          || (!RIGHT(n) && x>=y && x>=tp->ys-y-1 && y<=tp->ys/2))
+        SETBIT(x,y)
+      else
+        HALFBIT(x,y)
+    }
+
+#define T 2
+#define G 1
+    if (LEFT(n))
+      for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
+        SETBIT(x,y)
+    if (RIGHT(n))
+      for (y=0;y<tp->ys;y++) for (x=G;x<G+T;x++)
+        SETBIT(tp->ys-1-x,y)
+    if (UP(n))
+      for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
+        SETBIT(x,y)
+    if (DOWN(n))
+      for (x=0;x<tp->ys;x++) for (y=G;y<G+T;y++)
+        SETBIT(x,tp->ys-1-y)
+    if (LEFT(n))
+      for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
+        RESBIT(x,y)
+    if (RIGHT(n))
+      for (y=0;y<tp->ys;y++) for (x=0;x<G;x++)
+        RESBIT(tp->ys-1-x,y)
+    if (UP(n))
+      for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
+        RESBIT(x,y)
+    if (DOWN(n))
+      for (x=0;x<tp->ys;x++) for (y=0;y<G;y++)
+        RESBIT(x,tp->ys-1-y)
+
+#define R 6
+#define RT 3
+    if (LEFT(n) && UP(n))
+      for (x=G;x<=G+R;x++)
+        for (y=G;y<=R+2*G-x;y++) {
+          if (x+y>R+2*G-RT)
+            SETBIT(x,y)
+          else
+            RESBIT(x,y)
+		}
+    if (LEFT(n) && DOWN(n))
+      for (x=G;x<=G+R;x++)
+        for (y=G;y<=R+2*G-x;y++) {
+          if (x+y>R+2*G-RT)
+            SETBIT(x,tp->ys-1-y)
+          else
+            RESBIT(x,tp->ys-1-y)
+        }
+    if (RIGHT(n) && UP(n))
+      for (x=G;x<=G+R;x++)
+        for (y=G;y<=R+2*G-x;y++) {
+          if (x+y>R+2*G-RT)
+            SETBIT(tp->ys-1-x,y)
+          else
+            RESBIT(tp->ys-1-x,y)
+        }
+    if (RIGHT(n) && DOWN(n))
+      for (x=G;x<=G+R;x++)
+        for (y=G;y<=R+2*G-x;y++) {
+          if (x+y>R+2*G-RT)
+            SETBIT(tp->ys-1-x,tp->ys-1-y)
+          else
+            RESBIT(tp->ys-1-x,tp->ys-1-y)
+        }
+    if ((tp->images[n] = XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi),
+             1, XYBitmap, 0, (char *) data, tp->xs, tp->ys, 8, 0)) == None) {
+       (void) free((void *) data);
+       return False;
+    }
+    tp->images[n]->byte_order = MSBFirst;
+    tp->images[n]->bitmap_unit = 8;
+    tp->images[n]->bitmap_bit_order = LSBFirst;
+  }
+  return True;
+}
+
 void
 refresh_tetris(ModeInfo * mi)
 {
-  trisstruct *tp = &triss[MI_SCREEN(mi)];
+  trisstruct *tp;
+
+  if (triss == NULL)
+    return;
+  tp = &triss[MI_SCREEN(mi)];
+  if (!tp->well && tp->field == NULL)
+    return;
 
   if (!tp->painted)
     return;
@@ -1540,7 +1633,7 @@ refresh_tetris(ModeInfo * mi)
     for (i = 0; i < WELL_PERIMETER; i++) {
       for (j = 0; j < WELL_DEPTH; j++) {
         if (tp->wall[j][i].pmid >= 0) {
-          drawTrapazoid(mi,  tp->wall[j][i].pmid, tp->wall[j][i].cid, i, j);
+          drawTrapazoid(mi, /*  tp->wall[j][i].pmid, */ tp->wall[j][i].cid, i, j);
         }
         if (tp->frozen_wall[i / WELL_WIDTH]) {
           freezeTrapazoid(mi, i, j);
@@ -1570,49 +1663,42 @@ refresh_tetris(ModeInfo * mi)
 
 void
 release_tetris(ModeInfo * mi) {
-  Display *   display = MI_DISPLAY(mi);
   int bonustype, number;
 
   if (triss != NULL) {
     int   screen;
 
-    for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-      trisstruct *tp = &triss[screen];
-
-      if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
-        MI_WHITE_PIXEL(mi) = tp->whitepixel;
-        MI_BLACK_PIXEL(mi) = tp->blackpixel;
-#ifndef STANDALONE
-        MI_FG_PIXEL(mi) = tp->fg;
-        MI_BG_PIXEL(mi) = tp->bg;
-#endif
-        if (tp->colors && tp->ncolors && !tp->no_colors)
-          free_colors(display, tp->cmap, tp->colors, tp->ncolors);
-        if (tp->colors)
-          (void) free((void *) tp->colors);
-        if (tp->cmap)
-          XFreeColormap(display, tp->cmap);
-      }
-      if (tp->gc != NULL)
-        XFreeGC(display, tp->gc);
-      if (tp->field)
-        (void) free((void *) tp->field);
-    }
+    for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+       free_tetris(MI_DISPLAY(mi), &triss[screen]);
     (void) free((void *) triss);
     triss = NULL;
     for (bonustype = 0; bonustype < 2; bonustype++) {
-      for (number = 0; number < max_ominoes[bonustype]; number++) {
-        if (polytris[bonustype].polyomino[number].shape)
-          (void) free((void *) polytris[bonustype].polyomino[number].shape);
+      if (polytris[bonustype].polyomino != NULL) {
+        for (number = 0; number < max_ominoes[bonustype]; number++) {
+          if (polytris[bonustype].polyomino[number].shape != NULL)
+            (void) free((void *) polytris[bonustype].polyomino[number].shape);
+        }
+        (void) free((void *) polytris[bonustype].polyomino);
+        polytris[bonustype].polyomino = NULL;
+      }
+      if (polytris[bonustype].mode.start != NULL) {
+        (void) free((void *) polytris[bonustype].mode.start);
+        polytris[bonustype].mode.start = NULL;
       }
-      (void) free((void *) polytris[bonustype].polyomino);
-      (void) free((void *) polytris[bonustype].mode.start);
     }
   }
 }
 
 void
 change_tetris(ModeInfo * mi) {
+  trisstruct *tp;
+
+  if (triss == NULL)
+    return;
+  tp = &triss[MI_SCREEN(mi)];
+  if (!tp->well && tp->field == NULL)
+    return;
+
   tryMove(mi, ROTATE);
 }
 
@@ -1620,26 +1706,33 @@ void
 init_tetris(ModeInfo * mi) {
   Display *   display = MI_DISPLAY(mi);
   Window      window = MI_WINDOW(mi);
-  XGCValues       gcv;
-  trisstruct *tp;
   int size = MI_SIZE(mi);
-  int   i, j, first = 0, bonustype;
+  int   i, j, bonustype;
+  trisstruct *tp;
 
   if (triss == NULL) {
     if ((triss = (trisstruct *) calloc(MI_NUM_SCREENS(mi),
                sizeof (trisstruct))) == NULL)
       return;
     for (bonustype = 0; bonustype < 2; bonustype++) {
-      polytris[bonustype].mode.start = (int *) malloc(start_ominoes[bonustype] * sizeof(int));
-      polytris[bonustype].polyomino = (Polyominoes *) malloc(max_ominoes[bonustype] * sizeof(Polyominoes));
+      if (((polytris[bonustype].mode.start = (int *) malloc(start_ominoes[bonustype] *
+            sizeof(int))) == NULL) ||
+          ((polytris[bonustype].polyomino = (Polyominoes *) malloc(max_ominoes[bonustype] *
+            sizeof(Polyominoes))) == NULL)) {
+        release_tetris(mi);
+        return;
+      }
+    }
+    if (!readPolyominoes()) {
+      release_tetris(mi);
+      return;
     }
-    tp = &triss[MI_SCREEN(mi)];
-    readPolyominoes(tp);
 #ifdef DEBUG
     writePolyominoes(tp);
 #endif
-  } else
-    tp = &triss[MI_SCREEN(mi)];
+  }
+   tp = &triss[MI_SCREEN(mi)];
+  tp->mi = mi;
 
   if (MI_IS_FULLRANDOM(mi)) {
     tp->bonus = (Bool) (LRAND() & 1);
@@ -1649,27 +1742,7 @@ init_tetris(ModeInfo * mi) {
     tp->well = well;
   }
 
-#define UNDER_CONSTRUCTION
-#ifndef UNDER_CONSTRUCTION
-  if (trackmouse)
-    tp->well = False;
-#endif 
-  if (!tp->gc) {
-#ifdef BM_3D
-    icon_width = rot00_width;
-    icon_height = rot00_height;
-#else
-#ifdef BM_PLAIN
-    icon_width = rop00_width;
-    icon_height = rop00_height;
-#endif
-#endif
-    for (i = 0; i < BITMAPS; i++) {
-      logo[i].width = icon_width;
-      logo[i].height = icon_height;
-      logo[i].bytes_per_line = (icon_width + 7) / 8;
-    }
-
+  if (tp->gc == None) {
     tp->blackpixel = MI_BLACK_PIXEL(mi);
     if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
       XColor      color;
@@ -1682,8 +1755,11 @@ init_tetris(ModeInfo * mi) {
       tp->bg = MI_BG_PIXEL(mi);
 #endif
       tp->whitepixel = MI_WHITE_PIXEL(mi);
-      tp->cmap = XCreateColormap(display, window,
-                MI_VISUAL(mi), AllocNone);
+      if ((tp->cmap = XCreateColormap(display, window,
+          MI_VISUAL(mi), AllocNone)) == None) {
+        free_tetris(display, tp);
+        return;
+      }
       XSetWindowColormap(display, window, tp->cmap);
       (void) XParseColor(display, tp->cmap, "black", &color);
       (void) XAllocColor(display, tp->cmap, &color);
@@ -1699,16 +1775,22 @@ init_tetris(ModeInfo * mi) {
       (void) XAllocColor(display, tp->cmap, &color);
       MI_FG_PIXEL(mi) = color.pixel;
 #endif
-      tp->colors = 0;
+      tp->colors = NULL;
       tp->ncolors = 0;
     }
     if ((tp->gc = XCreateGC(display, MI_WINDOW(mi),
-           (unsigned long) 0, (XGCValues *) NULL)) == None) {
+        (unsigned long) 0, (XGCValues *) NULL)) == None) {
+      free_tetris(display, tp);
       return;
     }
-    gcv.function = GXxor;
-    gcv.foreground = tp->fg;
-    gcv.background = tp->bg;
+#if 0
+    {
+      XGCValues       gcv;
+      gcv.function = GXxor;
+      gcv.foreground = tp->fg;
+      gcv.background = tp->bg;
+    }
+#endif
   }
   MI_CLEARWINDOW(mi);
   tp->painted = False;
@@ -1716,24 +1798,21 @@ init_tetris(ModeInfo * mi) {
   tp->width = MI_WIDTH(mi);
   tp->height = MI_HEIGHT(mi);
 
+  free_images(tp);
   if (tp->width < 2 + 2 * tp->well)
     tp->width = 2 + 2 * tp->well;
   if (tp->height < 2 + 2 * tp->well)
     tp->height = 2 + 2 * tp->well;
-  if (size == 0 ||
-    MINGRIDSIZE * size > tp->width || MINGRIDSIZE * size > tp->height) {
-      if (!tp->well && tp->width > MINGRIDSIZE * icon_width &&
-        tp->height > MINGRIDSIZE * icon_height) {
-        tp->pixelmode = False;
-        tp->xs = icon_width;
-        tp->ys = icon_height;
+  if (size > MAXPIXELSIZE  &&
+        (MINGRIDSIZE * size < tp->width && MINGRIDSIZE * size < tp->height)) {
+      if (!tp->well) {
+        tp->xs = size;
+        tp->ys = size;
       } else {
-        tp->pixelmode = True;
         tp->xs = tp->ys = MAX(MINSIZE, MIN(tp->width, tp->height) /
             ((tp->well) ? 3 * MINGRIDSIZE : MINGRIDSIZE));
       }
   } else {
-    tp->pixelmode = True;
     if (size < -MINSIZE)
       tp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(tp->width, tp->height) /
         ((tp->well) ? 3 * MINGRIDSIZE : MINGRIDSIZE))) - MINSIZE + 1) + MINSIZE;
@@ -1744,7 +1823,18 @@ init_tetris(ModeInfo * mi) {
         ((tp->well) ? 3 * MINGRIDSIZE : MINGRIDSIZE)));
     tp->xs = tp->ys;
   }
-
+  if (tp->ys > MAXPIXELSIZE && !tp->well) {
+    if (tp->ys & 1) {
+      tp->ys--;
+      tp->xs = tp->ys;
+    }
+    if (!create_images(mi, tp)) {
+      free_images(tp);
+      tp->pixelmode = True;
+    } else
+      tp->pixelmode = False;
+  } else
+    tp->pixelmode = True;
   if (tp->well) {
     tp->ncols = MAX_SIDES * WELL_WIDTH;
     tp->nrows = WELL_DEPTH;
@@ -1770,9 +1860,13 @@ init_tetris(ModeInfo * mi) {
     tp->nrows = MAX(tp->height / tp->ys, 2);
     tp->xb = (tp->width - tp->xs * tp->ncols) / 2;
     tp->yb = (tp->height - tp->ys * tp->nrows) / 2;
-    if (tp->field)
+    if (tp->field != NULL)
       (void) free((void *) tp->field);
-    tp->field = (fieldstruct *) malloc(tp->ncols * tp->nrows * sizeof (fieldstruct));
+    if ((tp->field = (fieldstruct *) malloc(tp->ncols * tp->nrows *
+        sizeof (fieldstruct))) == NULL) {
+      free_tetris(display, tp);
+      return;
+    }
     for (i = 0; i < tp->ncols * tp->nrows; i++) {
       tp->field[i].pmid = EMPTY;
       tp->field[i].cid = 0;
@@ -1785,11 +1879,12 @@ init_tetris(ModeInfo * mi) {
   /* Set up colour map */
   tp->direction = (LRAND() & 1) ? 1 : -1;
   if (MI_IS_INSTALL(mi) && MI_NPIXELS(mi) > 2) {
-    if (tp->colors && tp->ncolors && !tp->no_colors)
-      free_colors(display, tp->cmap, tp->colors, tp->ncolors);
-    if (tp->colors)
+    if (tp->colors != NULL) {
+      if (tp->ncolors && !tp->no_colors)
+        free_colors(display, tp->cmap, tp->colors, tp->ncolors);
       (void) free((void *) tp->colors);
-    tp->colors = 0;
+      tp->colors = NULL;
+    }
     tp->ncolors = MI_NCOLORS(mi);
     if (tp->ncolors < 2)
       tp->ncolors = 2;
@@ -1799,9 +1894,13 @@ init_tetris(ModeInfo * mi) {
       tp->mono_p = False;
 
     if (tp->mono_p)
-      tp->colors = 0;
+      tp->colors = NULL;
     else
-      tp->colors = (XColor *) malloc(sizeof (*tp->colors) * (tp->ncolors + 1));
+      if ((tp->colors = (XColor *) malloc(sizeof (*tp->colors) *
+          (tp->ncolors + 1))) == NULL) {
+        free_tetris(display, tp);
+        return;
+      }
     tp->cycle_p = has_writable_cells(mi);
     if (tp->cycle_p) {
       if (MI_IS_FULLRANDOM(mi)) {
@@ -1817,7 +1916,7 @@ init_tetris(ModeInfo * mi) {
       if (!(LRAND() % 10))
         make_random_colormap(
 #if STANDALONE
-            MI_DISPLAY(mi), MI_WINDOW(mi),
+            display, MI_WINDOW(mi),
 #else
             mi,
 #endif
@@ -1826,7 +1925,7 @@ init_tetris(ModeInfo * mi) {
       else if (!(LRAND() % 2))
         make_uniform_colormap(
 #if STANDALONE
-            MI_DISPLAY(mi), MI_WINDOW(mi),
+            display, MI_WINDOW(mi),
 #else
             mi,
 #endif
@@ -1835,7 +1934,7 @@ init_tetris(ModeInfo * mi) {
       else
         make_smooth_colormap(
 #if STANDALONE
-            MI_DISPLAY(mi), MI_WINDOW(mi),
+            display, MI_WINDOW(mi),
 #else
             mi,
 #endif
@@ -1887,7 +1986,7 @@ moveOne(ModeInfo *mi, move_t move)
             if (cy == 0 && cx == 0)
               wall = 0.0;
             else
-              wall = 2.0 * (atan2(cy, cx) + 3 * M_PI / 4) / M_PI;
+              wall = 2.0 * (atan2((double) cy, (double) cx) + 3 * M_PI / 4) / M_PI;
             if (wall < 0.0)
               wall += 4.0;
             mousepos = (int) (wall * WELL_WIDTH);
@@ -1930,7 +2029,9 @@ moveOne(ModeInfo *mi, move_t move)
     {
       int       lines;
 
-      lines = checkLines(mi);
+      if ((lines = checkLines(mi)) < 0) {
+        return False;
+      }
       tp->rows += lines;
       if (tp->rows > THRESHOLD(tp->level)) {
         tp->level++;
@@ -1969,9 +2070,16 @@ moveOne(ModeInfo *mi, move_t move)
 }
 
 void
-draw_tetris(ModeInfo * mi) {
+draw_tetris(ModeInfo * mi)
+{
   Display *   display = MI_DISPLAY(mi);
-  trisstruct *tp = &triss[MI_SCREEN(mi)];
+  trisstruct *tp;
+
+  if (triss == NULL)
+    return;
+  tp = &triss[MI_SCREEN(mi)];
+  if (!tp->well && tp->field == NULL)
+    return;
 
   if (tp->no_colors) {
     init_tetris(mi);
@@ -1983,7 +2091,7 @@ draw_tetris(ModeInfo * mi) {
   /* Rotate colours */
   if (tp->cycle_p) {
     rotate_colors(display, tp->cmap, tp->colors, tp->ncolors,
-            tp->direction);
+      tp->direction);
     if (!(LRAND() % 1000))
       tp->direction = -tp->direction;
   }
