@@ -15,7 +15,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xcl.c 0.5pl0  2000/07/25";
+static const char sccsid[] = "@(#)xcl.c	5.00 2000/11/01 xlockmore";
 #endif
 
 #define RAND(x)	(((my_random() % x )- x/2))  /* random number around 0 */
@@ -114,7 +114,7 @@ ModeSpecOpt xcl_opts =
 #ifdef USE_MODULES    /* for xlockmore */
 ModStruct   xcl_description =
 {"xcl", "init_xcl", "draw_xcl", "release_xcl",
- "refresh_xcl", "init_xcl", NULL, &xcl_opts ,
+ "draw_xcl", "init_xcl", NULL, &xcl_opts ,
  20000, -3, 1, 1, 64, 1.0, "",
  "Shows a control line combat model race", 0, NULL};
 #endif
@@ -159,14 +159,14 @@ typedef struct
 
 static xclstruct *xcls = NULL;
 
-int my_random(void)  /* not really good, but it works */
+static int my_random(void)  /* not really good, but it works */
 {
   static int number;
   number -= random_pid;
   return ( 0x0fff & (number));
 }
 
-int countlines(void)
+static int countlines(void)
 {
   int array;
   int count = 0;
@@ -177,7 +177,7 @@ int countlines(void)
   return(count);
 }
 
-void um2(int X, int Y, int Z, xclstruct *dp)
+static void um2(int X, int Y, int Z, xclstruct *dp)
 {
   float X2,X3,X4,Y1,Y3,Y4,Z1,Z2,Z4;
 
@@ -195,7 +195,7 @@ void um2(int X, int Y, int Z, xclstruct *dp)
   dp->By = dp->mid_y - (-Z4 / Y4 * dp->G);
 }
 
-void view_3d(XSegment *xseg, xclstruct *dp)
+static void view_3d(XSegment *xseg, xclstruct *dp)
 {
   int count = 0;
   int I,J;
@@ -228,7 +228,7 @@ void view_3d(XSegment *xseg, xclstruct *dp)
     }
 }
 
-long get_color(Display *dpy,char *color,XColor *final_color)
+static long get_color(Display *dpy,char *color,XColor *final_color)
 {
   XColor cdef;
   Colormap cmap;
@@ -237,7 +237,7 @@ long get_color(Display *dpy,char *color,XColor *final_color)
   if (!XParseColor(dpy, cmap, color, &cdef) ||
       !XAllocColor(dpy, cmap, &cdef))
     {
-      fprintf(stderr, "Color \"%s\" wasn't found\n", color);
+      (void) fprintf(stderr, "Color \"%s\" wasn't found\n", color);
     }
   
   if (final_color != NULL) *final_color = cdef;/* Copy the final color. */
@@ -245,7 +245,7 @@ long get_color(Display *dpy,char *color,XColor *final_color)
   return(cdef.pixel);
 }
 
-void get_GC(Display *dpy,Window win,GC *gc,long color)
+static Bool get_GC(Display *dpy,Window win,GC *gc,long color)
 {
   unsigned long valuemask = 0; /*ignore XGCvalues and use defaults */
   XGCValues values;
@@ -254,38 +254,46 @@ void get_GC(Display *dpy,Window win,GC *gc,long color)
   int cap_style = CapRound;
   int join_style = JoinRound;
 
-  *gc = XCreateGC(dpy, win, valuemask , &values);
-
+  if ((*gc = XCreateGC(dpy, win, valuemask , &values)) == None)
+    return False;
   XSetForeground(dpy, *gc, color);
 
   XSetLineAttributes(dpy, *gc, line_width, line_style,
 		     cap_style, join_style);
+  return True;
 }
 
-void refresh_xcl(ModeInfo * mi)
+static void
+free_xcl(Display *display, xclstruct  *dp)
 {
-  draw_xcl(mi);
+    int plane;
+      
+    for (plane = 0; plane < dp->planes; plane++) {
+      if (dp->xseg[plane] != NULL) {
+	    (void) free((void *) dp->xseg[plane]);
+        dp->xseg[plane] = NULL;
+      }
+	  if (dp->xseg_old[plane] != NULL) {
+	    free(dp->xseg_old[plane]);
+        dp->xseg_old[plane] = NULL;
+      }
+	  if (dp->gc[plane] != None) {
+	    XFreeGC(display, dp->gc[plane]);
+        dp->gc[plane] = None;
+      }
+    }
+    if (dp->erase_gc != None) {
+	  XFreeGC(display, dp->erase_gc);
+      dp->erase_gc = None;
+    }
 }
 
 void release_xcl(ModeInfo * mi)
 {
   if (xcls != NULL) {
     int screen;
-    int i;
-    for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++) {
-      xclstruct  *dp = &xcls[screen];
-      
-      for(i=0;i< dp->planes; i++) {
-	if(dp->xseg[i] != NULL)
-	  free(dp->xseg[i]);
-	if(dp->xseg_old[i] != NULL)
-	  free(dp->xseg_old[i]);
-	if(dp->gc[i] != NULL)
-	  XFreeGC(MI_DISPLAY(mi), dp->gc[i]);
-      }
-      if(dp->erase_gc != NULL)
-	XFreeGC(MI_DISPLAY(mi), dp->erase_gc);
-    }
+    for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
+      free_xcl(MI_DISPLAY(mi), &xcls[screen]);
     (void) free((void *) xcls);
     xcls = NULL;
   }
@@ -293,8 +301,9 @@ void release_xcl(ModeInfo * mi)
 
 void init_xcl(ModeInfo * mi)
 {
-  xclstruct *dp;
+  Display *display = MI_DISPLAY(mi);
   int i;            /* scratch */
+  xclstruct *dp;
   
   if (xcls == NULL) {
     if ((xcls = (xclstruct *) calloc(MI_NUM_SCREENS(mi),
@@ -372,9 +381,9 @@ void init_xcl(ModeInfo * mi)
 	  switch(i)
 	    {
 	    case 0:
-	      dp->planecolor[0] = get_color(MI_DISPLAY(mi),"yellow",NULL);
+	      dp->planecolor[0] = get_color(display, (char *) "yellow",NULL);
 	    case 1:
-	      dp->planecolor[1] = get_color(MI_DISPLAY(mi),"red",NULL);
+	      dp->planecolor[1] = get_color(display, (char *) "red",NULL);
 	    default:
 	      dp->planecolor[i] = MI_PIXEL(mi, NRAND(MI_NPIXELS(mi)));
 	    }
@@ -383,42 +392,58 @@ void init_xcl(ModeInfo * mi)
     }
 
     if(dp->erase_gc == NULL)
-      get_GC(MI_DISPLAY(mi), MI_WINDOW(mi), &(dp->erase_gc),dp->bg);
+      if (!get_GC(display, MI_WINDOW(mi), &(dp->erase_gc),dp->bg)) {
+        free_xcl(display, dp);
+        return;
+      }
 
     dp->lines = countlines();
 
     for(i=0;i< dp->planes; i++) { 
       if(dp->gc[i] == NULL)
-	get_GC(MI_DISPLAY(mi), MI_WINDOW(mi), &(dp->gc[i]),
-	       dp->planecolor[i]);
+        if (!get_GC(display, MI_WINDOW(mi), &(dp->gc[i]),
+	        dp->planecolor[i])) {
+          free_xcl(display, dp);
+          return;
+        }
       dp->omega_const[i] = speed[i]/3.6 /line_length*1000.0;
       
       if(dp->xseg[i] == NULL)
-	dp-> xseg[i] = (XSegment *) malloc(sizeof(XSegment) * dp->lines);
+        if ((dp-> xseg[i] = (XSegment *) malloc(sizeof(XSegment) *
+		    dp->lines)) == NULL) {
+          free_xcl(display, dp);
+          return;
+        }
       if(dp->xseg_old[i] == NULL)
-	dp->xseg_old[i] = (XSegment *) malloc(sizeof(XSegment) * dp->lines);
+        if ((dp->xseg_old[i] = (XSegment *) malloc(sizeof(XSegment) *
+            dp->lines)) == NULL) {
+          free_xcl(display, dp);
+          return;
+        }
     }
     
     if(MI_IS_VERBOSE(mi)) {
-      printf("X control line combat in a box\n");
-      printf("Version: %s\n",sccsid);
-      printf("Line length: %gm\n",line_length/1000.0);
-      printf("Speed %g km/h  \n",speed[0]);
-      printf("Lines per plane: %d\n",dp->lines);
-      printf("Spectator at %gm\n",spectator/1000.0);
-      printf("Try %g frames per Second (frametime: %dus)\n",
+      (void) printf("X control line combat in a box\n");
+#if !defined( lint ) && !defined( SABER )
+      (void) printf("Version: %s\n",sccsid);
+#endif
+      (void) printf("Line length: %gm\n",line_length/1000.0);
+      (void) printf("Speed %g km/h  \n",speed[0]);
+      (void) printf("Lines per plane: %d\n",dp->lines);
+      (void) printf("Spectator at %gm\n",spectator/1000.0);
+      (void) printf("Try %g frames per Second (frametime: %dus)\n",
 	     1000000.0/frametime,frametime);
-      printf("Calibration at %d frames\n",REGULATE);
+      (void) printf("Calibration at %d frames\n",REGULATE);
     }
   }
 
   /* clear the screen */
   
   MI_CLEARWINDOW(mi);
-  /* XFillRectangle(MI_DISPLAY(mi), MI_WINDOW(mi),dp->erase_gc,0,0,MI_WIDTH(mi),MI_HEIGHT(mi));    */
+  /* XFillRectangle(display, MI_WINDOW(mi),dp->erase_gc,0,0,MI_WIDTH(mi),MI_HEIGHT(mi));    */
   
   
-  gettimeofday(&(dp->tv1),0);
+  (void) gettimeofday(&(dp->tv1),0);
   dp->time1 = (double)dp->tv1.tv_sec + (double)dp->tv1.tv_usec/(double)1000000;
   
   dp->xcldelay = frametime;
@@ -428,7 +453,13 @@ void draw_xcl(ModeInfo * mi)
 {
   static int count = 0;
   int i;
-  xclstruct *dp = &xcls[MI_SCREEN(mi)];
+  xclstruct *dp;
+  
+  if (xcls == NULL)
+    return;
+  dp = &xcls[MI_SCREEN(mi)];
+  if (dp->erase_gc == None)
+    return;
   
   if(viewmodel == True)
     {
@@ -450,9 +481,9 @@ void draw_xcl(ModeInfo * mi)
 			dp->xseg_old[0],dp->lines);
 	  XDrawSegments(MI_DISPLAY(mi),MI_WINDOW(mi),dp->gc[0],
 			dp->xseg[0],dp->lines);
-	  usleep(ROTATEDELAY * 2);
+	  (void) usleep(ROTATEDELAY * 2);
 	}
-      memcpy(dp->xseg_old[0],dp->xseg[0],sizeof(XSegment)*dp->lines);
+      (void) memcpy(dp->xseg_old[0],dp->xseg[0],sizeof(XSegment)*dp->lines);
       view_3d(dp->xseg[0],dp);
       count ++;
     }
@@ -463,7 +494,7 @@ void draw_xcl(ModeInfo * mi)
       
       for(i=0;i<dp->planes;i++) {
 	
-	memcpy(dp->xseg_old[i],dp->xseg[i],sizeof(XSegment)*dp->lines);
+	(void) memcpy(dp->xseg_old[i],dp->xseg[i],sizeof(XSegment)*dp->lines);
 	
 	dp->Alpha = - dp->alpha[i];
 	dp->Beta = - dp->el[i];
@@ -526,10 +557,10 @@ void draw_xcl(ModeInfo * mi)
 
       count++;
 
-      usleep(dp->xcldelay);
+      (void) usleep(dp->xcldelay);
       if((count % REGULATE) == 0)
 	{
-	  gettimeofday(&(dp->tv1),0);
+	  (void) gettimeofday(&(dp->tv1),0);
 	  dp->time2 = (double)dp->tv1.tv_sec + (double)dp->tv1.tv_usec/
 	    (double)1000000;
 	  dp->time3 = dp->time2 - dp->time1;
@@ -540,7 +571,7 @@ void draw_xcl(ModeInfo * mi)
 	    dp->xcldelay = 10;  /* 1 is possible xor xscreensaver mode */
 	  }
 	  if(debug == True)
-	    printf("t_draw: %d, t_delay: %d\n",dp->drawtime,dp->xcldelay);
+	    (void) printf("t_draw: %d, t_delay: %d\n",dp->drawtime,dp->xcldelay);
 	}
     }
 }
