@@ -2,7 +2,7 @@
 /* xjack -- Jack having one of those days */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)xjack.c	4.13 98/10/10 xlockmore";
+static const char sccsid[] = "@(#)xjack.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -25,7 +25,8 @@ static const char sccsid[] = "@(#)xjack.c	4.13 98/10/10 xlockmore";
  * don't come in!  Now, do you think you can handle that?
  *
  * Revision History:
- * 10-Oct-98: revision history edited.
+ * 01-Nov-2000: Allocation checks
+ * 10-Oct-1998: revision history edited.
  */
 
 #ifdef STANDALONE
@@ -106,15 +107,21 @@ init_xjack(ModeInfo * mi)
 	}
 	jp = &jacks[MI_SCREEN(mi)];
 	if (mode_font == None)
-		mode_font = getFont(display);
-	if (jp->gc == NULL && mode_font != None) {
+		if ((mode_font = getFont(display)) == None) {
+			release_xjack(mi);
+		}
+
+	if (jp->gc == NULL) {
 		gcv.font = mode_font->fid;
 		XSetFont(display, MI_GC(mi), mode_font->fid);
 		gcv.graphics_exposures = False;
 		gcv.foreground = MI_WHITE_PIXEL(mi);
 		gcv.background = MI_BLACK_PIXEL(mi);
-		jp->gc = XCreateGC(display, MI_WINDOW(mi),
-					 GCForeground | GCBackground | GCGraphicsExposures | GCFont, &gcv);
+		if ((jp->gc = XCreateGC(display, MI_WINDOW(mi),
+				GCForeground | GCBackground | GCGraphicsExposures | GCFont,
+				&gcv)) == None) {
+			return;
+		}
 		jp->ascent = mode_font->ascent;
 		jp->height = font_height(mode_font);
 	}
@@ -160,8 +167,14 @@ draw_xjack(ModeInfo * mi)
 {
 	Display		*display = MI_DISPLAY(mi);
 	Window			window = MI_WINDOW(mi);
-	jackstruct *jp = &jacks[MI_SCREEN(mi)];
 	const char *s2;
+	jackstruct *jp;
+
+	if (jacks == NULL)
+		return;
+	jp = &jacks[MI_SCREEN(mi)];
+	if (jp->gc == None)
+		return;
 
 	jp->word_length = 0;
 	for (s2 = jp->s; *s2 && *s2 != ' '; s2++)
@@ -229,7 +242,7 @@ draw_xjack(ModeInfo * mi)
 #if 0
 					XFlush(display);
 					if (delay)
-						 usleep (delay * 10);
+						 (void) usleep (delay * 10);
 #endif
 				}
 				if (jp->y < 0)
@@ -309,7 +322,7 @@ draw_xjack(ModeInfo * mi)
 #if 0
 				XFlush(display);
 				if (delay)
-					 usleep (0xFFFF & (delay + (NRAND(delay * 10))));
+					 (void) usleep (0xFFFF & (delay + (NRAND(delay * 10))));
 #endif
 			}
 		}
@@ -372,12 +385,12 @@ draw_xjack(ModeInfo * mi)
 #if 0
 	XFlush(display);
 	if (delay) {
-		usleep (delay);
+		(void) usleep (delay);
 		if (!NRAND(3))
-			usleep(0xFFFFFF & ((NRAND(delay * 5)) + 1));
+			(void) usleep(0xFFFFFF & ((NRAND(delay * 5)) + 1));
 
 		if (jp->break_para)
-			usleep(0xFFFFFF & ((NRAND(delay * 15)) + 1));
+			(void) usleep(0xFFFFFF & ((NRAND(delay * 15)) + 1));
 	}
 #endif
 
@@ -405,7 +418,7 @@ draw_xjack(ModeInfo * mi)
 			}
 #if 0
 			XFlush(display);
-			usleep (5000000);
+			(void) usleep (5000000);
 #endif
 			while (*n2) {
 				(void) XDrawString (display, window, jp->gc,
@@ -420,7 +433,7 @@ draw_xjack(ModeInfo * mi)
 			jp->y++;
 #if 0
 			XFlush(display);
-			usleep (500000);
+			(void) usleep (500000);
 #endif
 		}
 	}
