@@ -147,8 +147,8 @@ fixColormap(ModeInfo * mi, int ncolors, float saturation,
 	    Bool mono, Bool install, Bool inroot, Bool inwindow, Bool verbose)
 {
 	Display    *display = MI_DISPLAY(mi);
-        Window     window = MI_WINDOW(mi);
-        Screen     *scr = MI_SCREENPTR(mi);
+	Window     window = MI_WINDOW(mi);
+	Screen     *scr = MI_SCREENPTR(mi);
 	Colormap    cmap = MI_COLORMAP(mi);
 	Colormap    dcmap = DefaultColormapOfScreen(scr);
 	XColor      xcolor;
@@ -169,16 +169,28 @@ fixColormap(ModeInfo * mi, int ncolors, float saturation,
 	if (mono || CellsOfScreen(scr) <= 2) {
 		if (MI_PIXELS(mi))
 			return;
-		MI_PIXELS(mi) =
-			(unsigned long *) calloc(2, sizeof (unsigned long));
+		if ((MI_PIXELS(mi) = (unsigned long *) calloc(2,
+				sizeof (unsigned long))) == NULL) {
+			(void) fprintf(stderr, "could not get the 2 colors for mono\n");
+		}
 
 		monoColormap(scr, MI_SCREENINFO(mi), foreground, background);
 		return;
 	}
 	colorcount = ncolors;
-	red = (unsigned char *) calloc(ncolors, sizeof (unsigned char));
-	green = (unsigned char *) calloc(ncolors, sizeof (unsigned char));
-	blue = (unsigned char *) calloc(ncolors, sizeof (unsigned char));
+	if (((  red = (unsigned char *) calloc(ncolors,
+			sizeof (unsigned char))) == NULL) ||
+	    ((green = (unsigned char *) calloc(ncolors,
+			sizeof (unsigned char))) == NULL) ||
+	    (( blue = (unsigned char *) calloc(ncolors,
+			sizeof (unsigned char))) == NULL)) {
+		(void) fprintf(stderr, "could not get the %d colors\n", ncolors);
+		if (red != NULL)
+			(void) free((void *) red);
+		if (green != NULL)
+			(void) free((void *) green);
+		return;
+	}
 
 	visualclass = MI_VISUALCLASS(mi);
 	fixed = (visualclass == StaticGray) || (visualclass == StaticColor) ||
@@ -199,21 +211,16 @@ fixColormap(ModeInfo * mi, int ncolors, float saturation,
 #endif
 		XFreeColors(display, cmap, &(MI_BG_PIXEL(mi)), 1, 0);
 		XFreeColors(display, cmap, &(MI_FG_PIXEL(mi)), 1, 0);
-		(void) free((void *) MI_PIXELS(mi));
-		MI_PIXELS(mi) =
-			(unsigned long *) calloc(ncolors, sizeof (unsigned long));
-	} else {
-		if (MI_PIXELS(mi)) {
-			(void) free((void *) MI_PIXELS(mi));
-			/* (void) printf("pixels: this case is possible?\n"); */
-		}
-		/* if (cmap) { (void) printf("cmap: this case is possible?\n");  } */
-		MI_PIXELS(mi) =
-			(unsigned long *) calloc(ncolors, sizeof (unsigned long));
-
-		/* "allocate" the black and white pixels, so that they
-		   will be included by XCopyColormapAndFree() if it gets called */
 	}
+	/* else if (cmap) { (void) printf("cmap: this case is possible?\n");  } */
+	if (MI_PIXELS(mi))
+		(void) free((void *) MI_PIXELS(mi));
+	if ((MI_PIXELS(mi) = (unsigned long *) calloc(ncolors,
+			sizeof (unsigned long))) == NULL) {
+		(void) fprintf(stderr, "could not get the %d colors\n", ncolors);
+	}
+	/* "allocate" the black and white pixels, so that they
+	   will be included by XCopyColormapAndFree() if it gets called */
 #ifdef COMPLIANT_COLORMAP
 	MI_BLACK_PIXEL(mi) = BlackPixelOfScreen(scr);
 	MI_WHITE_PIXEL(mi) = WhitePixelOfScreen(scr);
@@ -271,7 +278,7 @@ fixColormap(ModeInfo * mi, int ncolors, float saturation,
 		XFreeColors(display, cmap, &(MI_WHITE_PIXEL(mi) ), 1, 0);
 #endif
 		XFreeColors(display, cmap, &(MI_BG_PIXEL(mi)), 1, 0);
-		XFreeColors(display, cmap, &(MI_BG_PIXEL(mi)), 1, 0);
+		XFreeColors(display, cmap, &(MI_FG_PIXEL(mi)), 1, 0);
 #ifndef COMPLIANT_COLORMAP
 		if (retry) {
 			fixColormap(mi, 64, 1.0,
@@ -407,8 +414,13 @@ free_colors(Display * dpy, Colormap cmap, XColor * colors, int ncolors)
 	int         i;
 
 	if (ncolors > 0) {
-		unsigned long *pixels = (unsigned long *)
-		malloc(sizeof (unsigned long) * ncolors);
+		unsigned long *pixels;
+
+		if ((pixels = (unsigned long *) malloc(sizeof (unsigned long) *
+				ncolors)) == NULL) {
+			(void) fprintf(stderr, "could not free colors\n");
+			return;
+		}
 
 		for (i = 0; i < ncolors; i++)
 			pixels[i] = colors[i].pixel;
