@@ -14,6 +14,9 @@ static const char sccsid[] = "@(#)iostuff.c	4.10 98/10/23 xlockmore";
  * 23-Apr-98: Separated out of util.c
  *
  */
+#ifdef USE_MAGICK
+# include "magick.h"
+#endif
 
 #ifdef STANDALONE
 #include "utils.h"
@@ -137,7 +140,7 @@ get_dir(char *fullpath, char *dir, char *filename)
 #else
 	ln = fullpath;
 	ip = 0;
-	while ((ip_temp = index_dir(ln, "/"))) {
+	while ((ip_temp = index_dir(ln, (char *) "/"))) {
 		ip = ip + ip_temp;
 		ln = fullpath + ip;
 	}
@@ -196,7 +199,7 @@ sel_image(struct dirent *name)
 
 	if (numfrag == -1) {
 		++numfrag;
-		while ((ip = index_dir(filename_tmp, "*"))) {
+		while ((ip = index_dir(filename_tmp, (char *) "*"))) {
 			if (numfrag >= 64)
 				return 0;
 			frags[numfrag] = (char *) malloc(ip);
@@ -291,7 +294,6 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 	if (num_list_tmp && compare != NULL)
 		(void) qsort((void *) namelist_tmp, num_list_tmp,
 			     sizeof (struct dirent *), compare);
-
 	if (debug)
 		(void) printf("scan_dir number %d\n", num_list_tmp);
 	*namelist = namelist_tmp;
@@ -349,7 +351,9 @@ extern int  XbmReadFileToImage(char *filename,
 #include <X11/xpm.h>		/* Normal spot */
 #endif
 #endif
-#include "ras.h"
+#ifndef USE_MAGICK
+# include "ras.h"
+#endif
 
 static XImage blogo =
 {
@@ -415,7 +419,32 @@ getImage(ModeInfo * mi, XImage ** logo,
 #endif /* STANDALONE */
 	}
 	if (bitmap_local && strlen(bitmap_local)) {
-#ifndef STANDALONE
+#if defined( USE_MAGICK ) && !defined( STANDALONE )
+	   if ( readable( bitmap_local ) )
+	     {
+		if ( MI_NPIXELS( mi ) > 2 )
+		  {
+		     if ( MagickSuccess == MagickFileToImage( mi ,
+							     bitmap_local ,
+							     logo) )
+		       {
+			  *graphics_format = IS_MAGICKFILE;
+			  if ( !fixedColors( mi ) )
+			    SetImageColors( display , *ncm );
+			  *black = GetColor(mi, MI_BLACK_PIXEL(mi));
+			  (void) GetColor(mi, MI_WHITE_PIXEL(mi));
+			  (void) GetColor(mi, MI_BG_PIXEL(mi));
+			  (void) GetColor(mi, MI_FG_PIXEL(mi));
+		       }
+		  }
+	     }
+	   else
+	     {
+		(void) fprintf(stderr , "could not read file \"%s\"\n" ,
+			bitmap_local);
+	     }
+#else
+# ifndef STANDALONE
 		if (readable(bitmap_local)) {
 			if (MI_NPIXELS(mi) > 2) {
 				if (RasterSuccess == RasterFileToImage(mi, bitmap_local, logo)) {
@@ -462,7 +491,8 @@ getImage(ModeInfo * mi, XImage ** logo,
 				*logo = &blogo;
 			}
 		}
-		if (*graphics_format <= 0 && MI_IS_VERBOSE(mi))
+#endif
+	   if (*graphics_format <= 0 && MI_IS_VERBOSE(mi))
 			(void) fprintf(stderr,
 				       "\"%s\" is in an unrecognized format or not compatible with screen\n",
 				       bitmap_local);
@@ -492,7 +522,8 @@ getImage(ModeInfo * mi, XImage ** logo,
 	}
 #ifndef STANDALONE		/* Come back later */
 	if (*ncm != None && *graphics_format != IS_RASTERFILE &&
-	    *graphics_format != IS_XPMFILE && *graphics_format != IS_XPM) {
+	    *graphics_format != IS_XPMFILE && *graphics_format != IS_XPM &&
+	    *graphics_format != IS_MAGICKFILE) {
 		XFreeColormap(display, *ncm);
 		*ncm = None;
 	}
@@ -515,6 +546,7 @@ destroyImage(XImage ** logo, int *graphics_format)
 		case IS_XPM:
 		case IS_XPMFILE:
 		case IS_RASTERFILE:
+		case IS_MAGICKFILE:
 			if (logo && *logo)
 				(void) XDestroyImage(*logo);
 			break;
@@ -590,7 +622,7 @@ getPixmap(ModeInfo * mi, Drawable drawable,
 #define FROM_FILE    3
 #define FROM_RESRC   4
 
-static char *def_words = "I'm out running around.";
+static char *def_words = (char *) "I'm out running around.";
 static int  getwordsfrom;
 
 static void
@@ -665,7 +697,7 @@ getWords(int screen, int screens)
 					(void) memset((char *) progerr, 0, sizeof (progerr));
 					(void) strcpy(progerr, "sh: ");
 					strcat_firstword(progerr, (!program || !*program) ?
-						      DEF_PROGRAM : program);
+						      (char *) DEF_PROGRAM : program);
 					(void) strcat(progerr, ": not found\n");
 					if (!strcmp(&buf[screen * BUFSIZ], progerr))
 						switch (NRAND(12)) {
