@@ -484,15 +484,18 @@ extern void release_last_mode(ModeInfo *);
 /* comment out following defines to remove modes */
 #ifdef USE_GL
 #define MODE_cage
+#define MODE_fire
 #define MODE_gears
+#define MODE_lament
 #define MODE_moebius
 #define MODE_morph3d
 #define MODE_rubik
+#define MODE_sballs
 #define MODE_stairs
 #define MODE_superquadrics
 #define MODE_sproingies
 #ifdef USE_UNSTABLE
-#define MODE_skewb /* actively underdevelopment */
+#define MODE_skewb /* under development */
 #ifdef HAVE_CXX
 #if defined( HAVE_TTF ) && defined( HAVE_GLTT )
 #define MODE_text3d
@@ -508,12 +511,6 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_invert
 #endif
 #endif
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#define MODE_lament
-#endif
-#endif
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#define MODE_cartoon
 #endif
 #ifdef HAVE_CXX
 #define MODE_solitare
@@ -592,6 +589,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_nose
 #define MODE_pacman
 #define MODE_penrose
+#define MODE_pentominoes
 #define MODE_petal
 #define MODE_puzzle
 #define MODE_pyro
@@ -737,13 +735,6 @@ extern ModeHook change_cage;
 extern ModeSpecOpt cage_opts;
 #endif
 
-#ifdef MODE_cartoon
-extern ModeHook init_cartoon;
-extern ModeHook draw_cartoon;
-extern ModeHook release_cartoon;
-extern ModeSpecOpt cartoon_opts;
-#endif
-
 #ifdef MODE_clock
 extern ModeHook init_clock;
 extern ModeHook draw_clock;
@@ -857,6 +848,15 @@ extern ModeHook refresh_fadeplot;
 extern ModeSpecOpt fadeplot_opts;
 #endif
 
+#ifdef MODE_fire
+extern ModeHook init_fire;
+extern ModeHook draw_fire;
+extern ModeHook release_fire;
+extern ModeHook refresh_fire;
+extern ModeHook change_fire;
+extern ModeSpecOpt fire_opts;
+#endif
+
 #ifdef MODE_flag
 extern ModeHook init_flag;
 extern ModeHook draw_flag;
@@ -1004,7 +1004,6 @@ extern ModeSpecOpt kaleid_opts;
 extern ModeHook init_kumppa;
 extern ModeHook draw_kumppa;
 extern ModeHook release_kumppa;
-extern ModeHook refresh_kumppa;
 extern ModeSpecOpt kumppa_opts;
 #endif
 
@@ -1181,6 +1180,14 @@ extern ModeHook refresh_penrose;	/* Needed */
 extern ModeSpecOpt penrose_opts;
 #endif
 
+#ifdef MODE_pentominoes
+extern ModeHook init_pentominoes;
+extern ModeHook draw_pentominoes;
+extern ModeHook release_pentominoes;
+extern ModeHook refresh_pentominoes;
+extern ModeSpecOpt pentominoes_opts;
+#endif
+
 #ifdef MODE_petal
 extern ModeHook init_petal;
 extern ModeHook draw_petal;
@@ -1249,6 +1256,15 @@ extern ModeHook change_rubik;
 extern ModeSpecOpt rubik_opts;
 #endif
 
+#ifdef MODE_sballs
+extern ModeHook init_sballs;
+extern ModeHook draw_sballs;
+extern ModeHook release_sballs;
+extern ModeHook refresh_sballs;
+extern ModeHook change_sballs;
+extern ModeSpecOpt sballs_opts;
+#endif
+
 #ifdef MODE_shape
 extern ModeHook init_shape;
 extern ModeHook draw_shape;
@@ -1353,7 +1369,6 @@ extern ModeSpecOpt star_opts;
 extern ModeHook init_starfish;
 extern ModeHook draw_starfish;
 extern ModeHook release_starfish;
-extern ModeHook refresh_starfish;
 extern ModeSpecOpt starfish_opts;
 #endif
 
@@ -1507,7 +1522,6 @@ extern ModeSpecOpt worm_opts;
 extern ModeHook init_xcl;
 extern ModeHook draw_xcl;
 extern ModeHook release_xcl;
-extern ModeHook refresh_xcl;
 extern ModeSpecOpt xcl_opts;
 #endif
 
