@@ -1,5 +1,5 @@
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)passwd.c	4.02 97/04/01 xlockmore";
+static const char sccsid[] = "@(#)passwd.c	5.00 2000/11/01 xlockmore";
 
 #endif
 
@@ -11,26 +11,27 @@ static const char sccsid[] = "@(#)passwd.c	4.02 97/04/01 xlockmore";
  * Revision History:
  *
  * Changes maintained by David Bagley <bagleyd@tux.org>
- * 18-Feb-99: allowroot option no longer ignored for PAM authentication.
- *            Sven Paas <Sven.Paas@t-online.de>
- * 24-Jan-98: Updated PAM support and made it configure-able.
- *            Marc Ewing <marc@redhat.com>  Original PAM support from
- *            25-Jul-96 Michael K. Johnson <johnsonm@redhat.com>
- * 18-Jan-98: Kerberos corrections by RLS <sinkr@cbl.umces.edu>
- * 25-May-96: When xlock is compiled with shadow passwords it will still
- *            work on non shadowed systems.  Marek Michalkiewicz
- *            <marekm@i17linuxb.ists.pwr.wroc.pl>
- * 25-Feb-96: Lewis Muhlenkamp
- *            Added in ability for any of the root accounts to unlock
- *            screen.  Message now gets sent to syslog if root does the
- *            unlocking.
- * 23-Dec-95: Ron Hitchens <ron@idiom.com> reorganized.
- * 10-Dec-95: More context handling stuff for DCE thanks to
- *            Terje Marthinussen <terjem@cc.uit.no>.
- * 01-Sep-95: DCE code added thanks to Heath A. Kehoe
- *            <hakehoe@icaen.uiowa.edu>.
- * 24-Jun-95: Extracted from xlock.c, encrypted passwords are now fetched
- *            on start-up to ensure correct operation (except Ultrix).
+ * 01-Nov-2000: SIA passwording for Tru64 added
+ * 18-Feb-1999: allowroot option no longer ignored for PAM authentication.
+ *              Sven Paas <Sven.Paas@t-online.de>
+ * 24-Jan-1998: Updated PAM support and made it configure-able.
+ *              Marc Ewing <marc@redhat.com>  Original PAM support from
+ *              25-Jul-96 Michael K. Johnson <johnsonm@redhat.com>
+ * 18-Jan-1998: Kerberos corrections by RLS <sinkr@cbl.umces.edu>
+ * 25-May-1996: When xlock is compiled with shadow passwords it will still
+ *              work on non shadowed systems.  Marek Michalkiewicz
+ *              <marekm@i17linuxb.ists.pwr.wroc.pl>
+ * 25-Feb-1996: Lewis Muhlenkamp
+ *              Added in ability for any of the root accounts to unlock
+ *              screen.  Message now gets sent to syslog if root does the
+ *              unlocking.
+ * 23-Dec-1995: Ron Hitchens <ron@idiom.com> reorganized.
+ * 10-Dec-1995: More context handling stuff for DCE thanks to
+ *              Terje Marthinussen <terjem@cc.uit.no>.
+ * 01-Sep-1995: DCE code added thanks to Heath A. Kehoe
+ *              <hakehoe@icaen.uiowa.edu>.
+ * 24-Jun-1995: Extracted from xlock.c, encrypted passwords are now fetched
+ *              on start-up to ensure correct operation (except Ultrix).
  */
 
 #include "xlock.h"
@@ -255,6 +256,11 @@ struct itmlst {
 #include <afs/kautils.h>
 #endif /* AFS */
 
+#ifdef SIA
+#include<sia.h>
+#include<siad.h>
+#endif /* SIA */
+
 char        user[PASSLENGTH];
 #ifdef GLOBAL_UNLOCK
 char        global_user[PASSLENGTH];
@@ -386,13 +392,13 @@ my_passwd_entry(void)
 	uid = (int) getuid();
 #ifndef SUNOS_ADJUNCT_PASSWD
 	{
-		char       *logname = NULL;
+		char       *logname;
 
 		pw = 0;
 		logname = getenv("LOGNAME");
-		if (!logname)
+		if (logname == NULL)
 			logname = getenv("USER");
-		if (logname) {
+		if (logname != NULL) {
 			pw = getpwnam(logname);
 			if (pw && (pw->pw_uid != uid))
 				pw = 0;
@@ -522,7 +528,6 @@ getUserName(void)
 		(void) free((void *) buf);	/* Should never get here */
 	}
 	(void) strcpy(user, pw->ufld.fd_name);
-
 #else /* !OSF1_ENH_SEC */
 
 	struct passwd *pw;
@@ -654,7 +659,7 @@ gpass(void)
 #if !defined( ultrix ) && !defined( DCE_PASSWD ) && !defined( BSD_AUTH )  && !defined ( PAM )
 #ifndef USE_XLOCKRC
 
-#if defined( HAVE_SHADOW ) && !defined( AFS )
+#if defined( HAVE_SHADOW ) && !defined( AFS ) && !defined( SIA )
 static int
 passwd_invalid(char *passwd)
 {
@@ -775,7 +780,7 @@ getCryptedUserPasswd(void)
 	/* Program probably needs to be setuid to root.  */
 	(void) strcpy(userpass, pw->pw_passwd);
 #ifdef HAVE_SHADOW
-#ifndef AFS
+#if !defined ( AFS ) && !defined ( SIA )
 	if (passwd_invalid(pw->pw_passwd)) {
 		extern Bool verbose, debug;
 
@@ -795,7 +800,7 @@ getCryptedUserPasswd(void)
 		gpass();
 #endif
 	}
-#endif /* !AFS */
+#endif /* !AFS && !SIA */
 #endif /* HAVE_SHADOW */
 #if ( HAVE_FCNTL_H && defined( USE_MULTIPLE_USER ))
 	set_multiple(getuid());
@@ -1161,16 +1166,22 @@ checkPasswd(char *buffer)
 
 #else /* !BSD_AUTH */
 
+	/* check AFS & SIA passwd first, then local, then root */
 #ifdef AFS
 	{
 		char       *reason;
 
-		/* check afs passwd first, then local, then root */
 		if (!ka_UserAuthenticate(user, "", 0, buffer, 0, &reason))
 			return True;
-
 	}
-#endif /* !AFS */
+#endif /* AFS */
+#ifdef SIA
+	{
+		if ((sia_validate_user(NULL, "xlock", 1, NULL, user, NULL,
+				0, NULL, buffer)) == SIASUCCESS)
+			return True;
+	}
+#endif /* SIA */
 #if defined(HAVE_KRB4) || defined(HAVE_KRB5)
 	{
 		/* Somehow, buffer gets 'erased' after either doing a krb pw check
