@@ -143,11 +143,22 @@ matherr(register struct exception *x)
 
 	switch (x->type) {
 		case DOMAIN:
+			/* Suppress "asin: DOMAIN error" stderr message */
+			if (!strcmp(x->name, "asin")) {
+				x->retval = 0.0;
+				return ((debug) ? 0 : 1);	/* suppress message unless debugging */
+			}
+			/* Suppress "acos: DOMAIN error" stderr message */
+			if (!strcmp(x->name, "acos")) {
+				x->retval = 0.0;
+				return ((debug) ? 0 : 1);	/* suppress message unless debugging */
+			}
 			/* Suppress "atan2: DOMAIN error" stderr message */
 			if (!strcmp(x->name, "atan2")) {
 				x->retval = 0.0;
 				return ((debug) ? 0 : 1);	/* suppress message unless debugging */
 			}
+			/* change sqrt to return sqrt(-arg1), not NaN */
 			if (!strcmp(x->name, "sqrt")) {
 				x->retval = sqrt(-x->arg1);
 				/* x->retval = 0.0; */
