@@ -68,22 +68,22 @@ static struct visual_class_name {
 } VisualClassName[] = {
 
 	{
-		StaticGray, "StaticGray"
+		StaticGray, (char *) "StaticGray"
 	},
 	{
-		GrayScale, "GrayScale"
+		GrayScale, (char *) "GrayScale"
 	},
 	{
-		StaticColor, "StaticColor"
+		StaticColor, (char *) "StaticColor"
 	},
 	{
-		PseudoColor, "PseudoColor"
+		PseudoColor, (char *) "PseudoColor"
 	},
 	{
-		TrueColor, "TrueColor"
+		TrueColor, (char *) "TrueColor"
 	},
 	{
-		DirectColor, "DirectColor"
+		DirectColor, (char *) "DirectColor"
 	},
 	{
 		-1, NULL	/* Others are 0-5 respectively */
@@ -283,7 +283,7 @@ init_GL(ModeInfo * mi)
 	glGetBooleanv(GL_RGBA_MODE, &rgbaMode);
 	if (!rgbaMode) {
 		glIndexi(MI_WHITE_PIXEL(mi));
-		glClearIndex(MI_BLACK_PIXEL(mi));
+		glClearIndex((float) MI_BLACK_PIXEL(mi));
 	}
 #ifdef FX
 	resetSize(mi, True);
