@@ -146,6 +146,9 @@ XrmOptionDescRec options[100];
  * The resulting warnings are switched off here    */
 #pragma message disable nosimpint
 #endif
+#ifdef VMS
+#include "vms_x_fix.h"
+#endif
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
@@ -411,7 +414,7 @@ struct hostent {
 #define CLASS class
 #endif
 
-#if (!defined( AFS ) && defined( HAVE_SHADOW ) && !defined( OSF1_ENH_SEC ) && !defined( HP_PASSWDETC ) && !defined( VMS ) && !defined( PAM ))
+#if (!defined( AFS ) && !defined( SIA ) && defined( HAVE_SHADOW ) && !defined( OSF1_ENH_SEC ) && !defined( HP_PASSWDETC ) && !defined( VMS ) && !defined( PAM ))
 #define FALLBACK_XLOCKRC
 #endif
 
@@ -550,6 +553,10 @@ extern XFontSet fontset;
 #endif
 #endif
 
+#ifndef CLOCKS_PER_SEC
+#define	CLOCKS_PER_SEC		1000000
+#endif
+
 
 /* There is some overlap so it can be made more efficient */
 #define ERASE_IMAGE(d,w,g,x,y,xl,yl,xs,ys) \
