@@ -26,9 +26,6 @@ static LockStruct LockProcs[] =
 #ifdef USE_GL
 {"cage","Shows the Impossible Cage, an Escher-like GL scene"},
 #endif
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-{"cartoon","Shows bouncing cartoons"},
-#endif
 {"clock","Shows Packard's clock"},
 {"coral","Shows a coral reef"},
 {"crystal","Shows polygons in 2D plane groups"},
@@ -43,6 +40,9 @@ static LockStruct LockProcs[] =
 {"euler2d","Shows a simulation of 2D incompressible inviscid fluid"},
 {"eyes","Shows eyes following a bouncing grelb"},
 {"fadeplot","Shows a fading plot of sine squared"},
+#ifdef USE_GL
+{"fire","Shows a 3D fire-like image"},
+#endif
 {"flag","Shows a waving flag image"},
 {"flame","Shows cosmic flame fractals"},
 {"flow","Shows dynamic strange attractors"},
@@ -66,7 +66,7 @@ static LockStruct LockProcs[] =
 {"julia","Shows the Julia set"},
 {"kaleid","Shows a kaleidoscope"},
 {"kumppa","Shows kumppa"},
-#if defined( USE_GL ) && ( defined( USE_XPM ) || defined( USE_XPMINC ))
+#ifdef USE_GL
 {"lament","Shows Lemarchand's Box"},
 #endif
 {"laser","Shows spinning lasers"},
@@ -93,6 +93,7 @@ static LockStruct LockProcs[] =
 {"nose","Shows a man with a big nose runs around spewing out messages"},
 {"pacman","Shows Pacman(tm)"},
 {"penrose","Shows Penrose's quasiperiodic tilings"},
+{"pentominoes","Shows attempts to place pentominoes into a rectangle"},
 {"petal","Shows various GCD Flowers"},
 #ifdef USE_GL
 {"pipes","Shows a selfbuilding pipe system"},
@@ -105,6 +106,9 @@ static LockStruct LockProcs[] =
 #ifdef USE_GL
 {"rubik","Shows an auto-solving Rubik's Cube"},
 #endif
+#ifdef USE_GL
+{"sballs","Balls spinning like crazy in GL"},
+#endif
 {"shape","Shows stippled rectangles, ellipses, and triangles"},
 {"sierpinski","Shows Sierpinski's triangle"},
 #if defined(USE_GL) && defined( USE_UNSTABLE )
@@ -136,6 +140,9 @@ static LockStruct LockProcs[] =
 {"swirl","Shows animated swirling patterns"},
 {"t3d","Shows a Flying Balls Clock Demo"},
 {"tetris","Shows an autoplaying tetris game"},
+#if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_TTF ) && defined( HAVE_GLTT ) && defined( USE_UNSTABLE )
+{"text3d","Shows 3D text"},
+#endif
 {"thornbird","Shows an animated bird in a thorn bush fractal map"},
 {"tik_tak","Shows rotating polygons"},
 {"triangle","Shows a triangle mountain range"},
