@@ -38,6 +38,7 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #include <sys/wait.h>
 
 #ifdef VMS
+#include "vms_x_fix.h"
 #include <descrip.h>
 #include <lib$routines.h>
 #endif
@@ -60,7 +61,7 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #endif
 
 #ifdef SunCplusplus
-        extern int  kill(long, int);
+        extern int  kill(pid_t, int);
         extern pid_t  wait(int *);
 
 #else
@@ -95,8 +96,6 @@ extern Widget Menuoption;
 
 static pid_t numberprocess = -1;	/* PID of xlock */
 
-extern void Setup_Option(Widget MenuBar);
-
 /* Widget */
 Widget      toplevel;
 static Widget Labelxlock;
@@ -110,35 +109,35 @@ static Widget ScrolledListModes, PushButtons[PUSHBUTTONS], Toggles[TOGGLES];
 
 static char *r_PushButtons[PUSHBUTTONS] =
 {
-	"Launch",
-	"In Root",
-	"In Window",
-	"Exit"
+	(char *) "Launch",
+	(char *) "In Root",
+	(char *) "In Window",
+	(char *) "Exit"
 };
 
 static char *r_Toggles[TOGGLES] =
 {
-	"mono",
-	"nolock",
-	"remote",
-	"allowroot",
-	"enablesaver",
-	"allowaccess",
-	"grabmouse",
-	"echokeys",
-	"usefirst"
+	(char *) "mono",
+	(char *) "nolock",
+	(char *) "remote",
+	(char *) "allowroot",
+	(char *) "enablesaver",
+	(char *) "allowaccess",
+	(char *) "grabmouse",
+	(char *) "echokeys",
+	(char *) "usefirst"
 };
 
 char       *r_Options[OPTIONS] =
 {
-	"username",
-	"password",
-	"info",
-	"validate",
-	"invalid",
-	"geometry",
-	"font",
-	"program"
+	(char *) "username",
+	(char *) "password",
+	(char *) "info",
+	(char *) "validate",
+	(char *) "invalid",
+	(char *) "geometry",
+	(char *) "font",
+	(char *) "program"
 };
 
 static int  numberinlist = 0;
@@ -242,7 +241,7 @@ static void
 Setup_Widget(Widget father)
 {
 	Arg         args[15];
-	int         i, ac = 0;
+	int         i, ac;
 	Widget      Row, PushButtonRow, TogglesRow, ModeLabel, SwitchLabel;
 	char        string[160];
 	XmString    label_str;
@@ -255,7 +254,7 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
 	ac++;
-	SwitchLabel = XmCreateLabel(father, "Switches", args, ac);
+	SwitchLabel = XmCreateLabel(father, (char *) "Switches", args, ac);
 	XtManageChild(SwitchLabel);
 	ac = 0;
 	XtSetArg(args[ac], XmNheight, WINDOW_HEIGHT);
@@ -264,7 +263,7 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNleftAttachment, XmATTACH_FORM);
 	ac++;
-	ModeLabel = XmCreateLabel(father, "Modes", args, ac);
+	ModeLabel = XmCreateLabel(father, (char *) "Modes", args, ac);
 	XtManageChild(ModeLabel);
 	ac = 0;
 	XtSetArg(args[ac], XmNwidth, WINDOW_WIDTH);
@@ -277,7 +276,7 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNleftWidget, ModeLabel);
 	ac++;
-	Labelxlock = XmCreateLabel(father, "Window", args, ac);
+	Labelxlock = XmCreateLabel(father, (char *) "Window", args, ac);
 	XtManageChild(Labelxlock);
 
 /* buttons in the bottom */
@@ -290,9 +289,9 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNbottomAttachment, XmATTACH_FORM);
 	ac++;
-	PushButtonRow = XmCreateRowColumn(father, "PushButtonRow", args, ac);
+	PushButtonRow = XmCreateRowColumn(father, (char *) "PushButtonRow", args, ac);
 
-	Menuoption = XmCreateMenuBar(PushButtonRow, "MenuBar", NULL, 0);
+	Menuoption = XmCreateMenuBar(PushButtonRow, (char *) "MenuBar", NULL, 0);
 	XtManageChild(Menuoption);
 
 	for (i = 0; i < PUSHBUTTONS; i++) {
@@ -329,9 +328,9 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNorientation, XmHORIZONTAL);
 	ac++;
-	Row = XmCreateRowColumn(father, "Row", args, ac);
+	Row = XmCreateRowColumn(father, (char *) "Row", args, ac);
 
-	for (i = 0; i < numprocs; i++) {
+	for (i = 0; i < (int) numprocs; i++) {
 		(void) sprintf(string, "%-14s%s", LockProcs[i].cmdline_arg,
 			       LockProcs[i].desc);
 		TabXmStr[i] = XmStringCreate(string, XmSTRING_DEFAULT_CHARSET);
@@ -343,13 +342,13 @@ Setup_Widget(Widget father)
 	ac++;
 	XtSetArg(args[ac], XmNvisibleItemCount, 10);
 	ac++;
-	ScrolledListModes = XmCreateScrolledList(Row, "ScrolledListModes",
+	ScrolledListModes = XmCreateScrolledList(Row, (char *) "ScrolledListModes",
 						 args, ac);
 	XtAddCallback(ScrolledListModes, XmNbrowseSelectionCallback,
 		      f_ScrolledListModes, NULL);
 	XtManageChild(ScrolledListModes);
 
-	TogglesRow = XmCreateRowColumn(Row, "TogglesRow", NULL, 0);
+	TogglesRow = XmCreateRowColumn(Row, (char *) "TogglesRow", NULL, 0);
 	for (i = 0; i < TOGGLES; i++) {
 #ifndef USE_MB
 		ac = 0;
@@ -366,11 +365,13 @@ Setup_Widget(Widget father)
 	XtManageChild(TogglesRow);
 
 	XtManageChild(Row);
-	for (i = 0; i < numprocs; i++) {
+	for (i = 0; i < (int) numprocs; i++) {
 		XmStringFree(TabXmStr[i]);
 	}
 }
 
+extern void setup_Option(Widget MenuBar);
+
 int
 main(int argc, char **argv)
 {
@@ -393,9 +394,9 @@ main(int argc, char **argv)
 	/* creation Widget */
 /* PURIFY 4.0.1 on Solaris 2 reports an unitialized memory read on the next
    line. */
-	form = XmCreateForm(toplevel, "Form", NULL, 0);
+	form = XmCreateForm(toplevel, (char *) "Form", NULL, 0);
 	Setup_Widget(form);
-	Setup_Option(Menuoption);
+	setup_Option(Menuoption);
 	XtManageChild(form);
 	XtRealizeWidget(toplevel);
 #if USE_XMU
